//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         ExportDlg.cpp
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  Implementation of Class ExportDlg.
//
//////////////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "diary.h"
#include "ExportDlg.h"
#include "diarydlg.h"
#include "setting.h"
#include "settingsfile.h"
#include "toolkit.h"
#include "monitors.h"
#include "multimonitor.h"
#include "monitordc.h"

#define LINE_SPACING    (120)
#define LEFT_MARGIN     (600)
#define START_HEIGHT    (500)
#define PAGE_SIZE       (6000)

// SExportDlg dialog

IMPLEMENT_DYNAMIC(SExportDlg, CDialog)
SExportDlg::SExportDlg(CWnd* pParent /*=NULL*/)
	: CDialog(SExportDlg::IDD, pParent)
{
	mOutputType = enSimpleTextNoImages;
}

SExportDlg::~SExportDlg()
{
}

void SExportDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_FROM_CALENDAR, m_FromCalendar);
	DDX_Control(pDX, IDC_UPTO_CALENDAR, m_UptoCalendar);
}


BEGIN_MESSAGE_MAP(SExportDlg, CDialog)
    ON_NOTIFY(MCN_GETDAYSTATE, IDC_FROM_CALENDAR, OnGetdaystateMonthcalendar)
    ON_NOTIFY(MCN_GETDAYSTATE, IDC_UPTO_CALENDAR, OnGetdaystateMonthcalendar)
	ON_NOTIFY(MCN_SELCHANGE, IDC_UPTO_CALENDAR, OnMcnSelchangeUptoCalendar)
	ON_NOTIFY(MCN_SELCHANGE, IDC_FROM_CALENDAR, OnMcnSelchangeFromCalendar)
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_EN_CHANGE(IDC_MAIN_FILE_SPEC, OnEnChangeMainFileSpec)
	ON_BN_CLICKED(ID_BROWSE, OnBnClickedBrowse)
	ON_BN_CLICKED(IDC_RADIO_10, OnBnClickedSimpleTextNoImages)
	ON_BN_CLICKED(IDC_RADIO_11, OnBnClickedSimpleHTMLNoImages)
	ON_BN_CLICKED(IDC_RADIO_12, OnBnClickedSimpleHTMLImagesInline)
	ON_BN_CLICKED(IDC_RADIO_13, OnBnClickedSimpleHTMLImagesLinked)
	ON_BN_CLICKED(IDC_RADIO_14, OnBnClickedFramedHTML)
	ON_BN_CLICKED(ID_EXPORT, OnBnClickedExport)
END_MESSAGE_MAP()


// SExportDlg message handlers

void SExportDlg::OnMcnSelchangeFromCalendar(NMHDR *pNMHDR, LRESULT *pResult)
{
    this->SavePosition();
	LPNMSELCHANGE pSelChange = reinterpret_cast<LPNMSELCHANGE>(pNMHDR);

    CWaitCursor lWaiter;
    SYSTEMTIME lTime;

    TK_LOG0("Is this message coming from the right Hwnd?");
    if (m_FromCalendar.m_hWnd == pNMHDR->hwndFrom)
    {
        TK_LOG0("Get the newly selected date");
        if (m_FromCalendar.GetCurSel(&lTime))
        {
            TK_LOG0("And calculate the new date in format CCYYMMDD");
            SDate lNewDate(lTime.wYear, lTime.wMonth, lTime.wDay);

            // the CMonthCalCtrl sends spurious SelChange messages, for the 
            // already selected date. This if is supposed to catch these....

            TK_LOG0("If new date doesn't match previous date...");
            if (lNewDate != this->mFromDate)
            {
                TK_LOG0("Start date adjusted");
				this->mFromDate = lNewDate;
	        }
        }
    }

    *pResult = 0;
}

void SExportDlg::OnMcnSelchangeUptoCalendar(NMHDR *pNMHDR, LRESULT *pResult)
{
    this->SavePosition();
	LPNMSELCHANGE pSelChange = reinterpret_cast<LPNMSELCHANGE>(pNMHDR);

	CWaitCursor lWaiter;
    SYSTEMTIME lTime;

    TK_LOG0("Is this message coming from the right Hwnd?");
    if (m_UptoCalendar.m_hWnd == pNMHDR->hwndFrom)
    {
        TK_LOG0("Get the newly selected date");
        if (m_UptoCalendar.GetCurSel(&lTime))
        {
            TK_LOG0("And calculate the new date in format CCYYMMDD");
            SDate lNewDate(lTime.wYear, lTime.wMonth, lTime.wDay);

            // the CMonthCalCtrl sends spurious SelChange messages, for the 
            // already selected date. This if is supposed to catch these....

            TK_LOG0("If new date doesn't match previous date...");
            if (lNewDate != this->mUptoDate)
            {
                TK_LOG0("End date adjusted");
				this->mUptoDate = lNewDate;
	        }
        }
    }

	*pResult = 0;
}

void SExportDlg::OnBnClickedOk()
{
    this->SavePosition();
	OnOK();
}

void SExportDlg::OnEnChangeMainFileSpec()
{
    this->SavePosition();
	this->GetDlgItem(IDC_MAIN_FILE_SPEC)->GetWindowText(this->mFullFileSpec);
}

void SExportDlg::OnBnClickedBrowse()
{
    this->SavePosition();
    char lFilters[255] = "Text Files (*.txt)|*.txt|HTML Files (*.html)|*.html|All Files (*.*)|*.*||";
    HRESULT hResult;

    DWORD lDirReturn = 0;
    char  *lPath = new char[MAX_STRING];
    char  *lExportPath = new char[MAX_STRING];

    TK_LOG0("Retrieve the path of execution");
    lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);

    CFileDialog dlg(TRUE, NULL, NULL, 0, lFilters);

    hResult = (int)dlg.DoModal();
    if (hResult != IDOK) 
    {
        SetCurrentDirectory(lPath);
		delete [] lExportPath;
		delete [] lPath;
        return;
    }

    TK_LOG0("Create the full file spec, and store it in the object and the field");
    lDirReturn = GetCurrentDirectory(MAX_STRING, lExportPath);
    mFullFileSpec.Format("%s\\%s", lExportPath, dlg.GetFileName());
    this->GetDlgItem(IDC_MAIN_FILE_SPEC)->SetWindowText(mFullFileSpec.GetBuffer());

    TK_LOG0("Restore the current directory, after the file selection box closes");
    lDirReturn = SetCurrentDirectory(lPath);

	delete [] lExportPath;
	delete [] lPath;
}

// This method is called when the dialog is to be shown. It is passed the pointer 
// to the SDiaryDlg object, so it will have access to the current date etc.

void SExportDlg::OnBnClickedFramedHTML()
{
    this->SavePosition();
	mOutputType = enFramedHTML;
}

void SExportDlg::OnBnClickedSimpleHTMLImagesLinked()
{
    this->SavePosition();
	mOutputType = enSimpleHTMLLinkedImages;
}

void SExportDlg::OnBnClickedSimpleHTMLImagesInline()
{
    this->SavePosition();
	mOutputType = enSimpleHTMLInlineImages;
}

void SExportDlg::OnBnClickedSimpleHTMLNoImages()
{
    this->SavePosition();
	mOutputType = enSimpleHTMLNoImages;
}

void SExportDlg::OnBnClickedSimpleTextNoImages()
{
    this->SavePosition();
	mOutputType = enSimpleTextNoImages;
}

BOOL SExportDlg::OnInitDialog(void)
{
    BOOL lRetCode = CDialog::OnInitDialog();

	SetWindowText(SDiaryDlg::Instance()->getWindowTitle().GetBuffer());

    TK_LOG0("Read the position settings and restore them");
    SSetting *lSettingTop    = SDiaryDlg::Instance()
                                   ->GetSettings()->GetSetting(SET_EXPORT_TOP);
    SSetting *lSettingLeft   = SDiaryDlg::Instance()
                                   ->GetSettings()->GetSetting(SET_EXPORT_LEFT);

    if (lSettingTop  != NULL &&
        lSettingLeft != NULL)
    {
        TK_LOG0("Position settings found, positioning window");

        int lTop  = lSettingTop->GetValueAsNumber();
        int lLeft = lSettingLeft->GetValueAsNumber();

	    // make sure it will be on-screen, but keep the original position in config 
	    this->SetWindowPos(0, lLeft, lTop, 0, 0, SWP_NOSIZE);
 	    if (CMonitors::IsOnScreen( this ) == false)
	    {
		    CMonitor thePrimary = CMonitors::GetPrimaryMonitor();
		    thePrimary.CenterWindowToMonitor(this, true); 
		    mWindowCentered = true;
            WINDOWPLACEMENT lWinPlace;
            this->GetWindowPlacement(&lWinPlace);
            mTop = lWinPlace.rcNormalPosition.top;
            mLeft = lWinPlace.rcNormalPosition.left;
 	    }
	}

    SYSTEMTIME lTime;
	lTime.wYear  = SDiaryDlg::Instance()->GetCurrentDate().getYear();
    lTime.wMonth = SDiaryDlg::Instance()->GetCurrentDate().getMonth();
	lTime.wDay   = SDiaryDlg::Instance()->GetCurrentDate().getDays();
    this->m_FromCalendar.SetCurSel(&lTime);
    this->m_UptoCalendar.SetCurSel(&lTime);
	this->mFromDate = lTime;
	this->mUptoDate = lTime;

    UpdateRadioButtons();
    this->GetDlgItem(IDC_MAIN_FILE_SPEC)->SetWindowText(mFullFileSpec.GetBuffer());

    return lRetCode;
}

void SExportDlg::UpdateRadioButtons(void)
{
   switch (this->mOutputType)
   {
   case enSimpleTextNoImages:
      this->CheckRadioButton(IDC_RADIO_10, IDC_RADIO_14, IDC_RADIO_10);
	  break; 

   case enSimpleHTMLNoImages:
      this->CheckRadioButton(IDC_RADIO_10, IDC_RADIO_14, IDC_RADIO_11);
	  break; 

   case enSimpleHTMLInlineImages:
      this->CheckRadioButton(IDC_RADIO_10, IDC_RADIO_14, IDC_RADIO_12);
	  break; 

   case enSimpleHTMLLinkedImages:
      this->CheckRadioButton(IDC_RADIO_10, IDC_RADIO_14, IDC_RADIO_13);
	  break; 

   case enFramedHTML:
      this->CheckRadioButton(IDC_RADIO_10, IDC_RADIO_14, IDC_RADIO_14);
	  break; 

   }
}

void SExportDlg::SavePosition(void)
{
   TK_LOG0("Save window position to settings.dat");
   WINDOWPLACEMENT lWinPlace;
   this->GetWindowPlacement(&lWinPlace);

   // cancel save if window centered, and not moved yet...
   if (mWindowCentered == true &&
	   (mTop == lWinPlace.rcNormalPosition.top ||
	    mLeft == lWinPlace.rcNormalPosition.left))
   {
	   return;
   }

   // ok, we save so set the flag to off
   mWindowCentered = false;
   mTop = -1;
   mLeft = -1;

   TK_LOG0("Get the Top position value");
   char *lTop = new char[MAX_STRING];
   _itoa_s(lWinPlace.rcNormalPosition.top, lTop, MAX_STRING - 1, 10);
   SSetting *lTopSSetting = new SSetting(SET_EXPORT_TOP, lTop);
   SDiaryDlg::Instance()->GetSettings()->AddSetting(lTopSSetting);
   delete [] lTop;

   TK_LOG0("Get the Left position value");
   char *lLeft = new char[MAX_STRING];
   _itoa_s(lWinPlace.rcNormalPosition.left, lLeft, MAX_STRING - 1, 10);
   SSetting *lLeftSSetting = new SSetting(SET_EXPORT_LEFT, lLeft);
   SDiaryDlg::Instance()->GetSettings()->AddSetting(lLeftSSetting);
   delete [] lLeft;

   TK_LOG0("Make sure the settings are flushed to disk");
   SDiaryDlg::Instance()->GetSettings()->WriteFile();
}

void SExportDlg::OnBnClickedExport()
{
	// pretest for any pages inbetween start and finish
    SDate lTest = this->mFromDate;
	bool lPageFound = false;
	do 
    {
		if (SDiaryDlg::Instance()->TestPage(lTest) == true)
		{
			lPageFound = true;
		}

		lTest.Add(0, 0, 1);

	} while (lTest <= this->mUptoDate);

	if (lPageFound == false)
	{
		MessageBox("Nothing to export between these dates", 
			       "Error", MB_OK);
		return;
	}

	// prepare and open the file to be written
	FILE *lFile = fopen(this->mFullFileSpec.GetBuffer(), "w");
	if (lFile == NULL)
	{
		MessageBox("Main Output file could not be opened", 
			       "Error", MB_OK);
		return;
	}

	// write any preparing data in it
	if (mOutputType == enSimpleHTMLNoImages ||
		mOutputType == enSimpleHTMLInlineImages ||
		mOutputType == enSimpleHTMLLinkedImages)
	{
        fprintf(lFile, "<html><head><title>");
		fprintf(lFile, "Diary %08d to %08d", 
			    this->mFromDate, this->mUptoDate);
		fprintf(lFile, "</title></head><body><font face=\"Tahoma\" size=2>");
	}
	
	// enumerate the filled days between start and finish
    SDate lCurrentDate = this->mFromDate;
	do 
    {
		if (SDiaryDlg::Instance()->TestPage(lCurrentDate) == true)
		{
      	    // tell each day to log itself according to the type requested
			SDiaryPage *thePage = SDiaryDlg::Instance()->getSDiaryPageForDate(lCurrentDate);
			thePage->Export(lFile, this->mExportPath, this->mOutputType);
		}

    	lCurrentDate.Add(0, 0, 1);

	} while (lCurrentDate <= this->mUptoDate);

	// write any postfixed data to it
	if (mOutputType == enSimpleHTMLNoImages ||
		mOutputType == enSimpleHTMLInlineImages ||
		mOutputType == enSimpleHTMLLinkedImages)
	{
        fprintf(lFile, "</font></body></html>");
	}

	// close the file
	fclose(lFile);
}

void SExportDlg::OnGetdaystateMonthcalendar(NMHDR* pNMHDR, LRESULT* pResult) 
{
   TK_TRACER("SDiaryDlg::OnGetdaystateMonthcalendar(NMHDR* pNMHDR, LRESULT* pResult)");
   CWaitCursor lWaiter;

   NMDAYSTATE* pDayState = (NMDAYSTATE *)pNMHDR;

   MONTHDAYSTATE mdState[3];
   INT i, iMax;

   iMax = pDayState->cDayState;

   for (i = 0; i < iMax; i++)
   {
	   mdState[i] = SDiaryDlg::Instance()->GetDayStatesForMonth(SDiaryDlg::Instance()->GetCurrentDate().getYear(),
		                                                   SDiaryDlg::Instance()->GetCurrentDate().getMonth(),
														   i);
   } 

   pDayState->prgDayState = mdState;
}

void SExportDlg::PrintEntries(int NumDays)
{
	if (NumDays <= 0)
	{
		return;
	}

	CPrintDialog dlg(FALSE);
	dlg.m_pd.hDevMode = NULL;
	dlg.m_pd.hDevNames = NULL;
	dlg.m_pd.Flags = PD_HIDEPRINTTOFILE | PD_NONETWORKBUTTON | 
		             PD_NOPAGENUMS | PD_NOSELECTION | PD_RETURNDC;
	if (dlg.DoModal() == IDOK)
	{
		// Creates a printer DC from the DEVMODE and DEVNAMES structures.
        if (dlg.CreatePrinterDC() == NULL)
		{
			MessageBox("No printer DC");
			return ;
		}

        // A printer device context (DC) has been created.
        // Attach it to a CDC object. The DC is deleted when the CDC
        // object goes out of scope.

		CDC PrintDC;
        PrintDC.Attach(dlg.m_pd.hDC);

		DOCINFO di;
        ::ZeroMemory (&di, sizeof (DOCINFO));
        di.cbSize = sizeof (DOCINFO);
        di.lpszDocName = "Diary Output";
		int x = LEFT_MARGIN;
		int y = START_HEIGHT;
        PrintDC.StartDoc (&di);
		PrintDC.StartPage();

       // get the string from the page
	   TK_LOG0("Build the correct filename, and open the file");
	   CString lFileName;
	   SDate theDate = SDiaryDlg::Instance()->GetCurrentDate();

	   int NumberOfDays = NumDays;
	   while (NumberOfDays > 0)
	   {
		   lFileName.Format("%08d.dry", theDate.asDecimal()); 

		   FILE *lFile = fopen(lFileName, "rb");

		   if (lFile != NULL)
		   {
			  TK_LOG0("File is open, print date first...");
       	      
			  // if every date on a new page...
			  if (NumDays > NumberOfDays)
			  {
			     y += LINE_SPACING * 2;
			     // PrintDC.StartPage();
			     // y = START_HEIGHT;
			  }
			  
			  CString lMsg;
			  lMsg.Format("%s, %s %d%s, %4d", 
						  theDate.dayAsLongString(),
						  SDate::monthAsLongString((theDate.getMonth())), 
	 					  theDate.getDays(), 
		 				  SDate::getDateSuffix(theDate.getDays()), 
			 			  theDate.getYear());
			  PrintDC.TextOutA(x, y, (LPCTSTR)lMsg, lMsg.GetLength());
			  y += LINE_SPACING * 1.4;

			  char lBuffer[MAX_BUFFER + 1];

			  int lBytesRead = fread(lBuffer, 1, MAX_BUFFER, lFile);
			  if (lBytesRead > 0)
			  {
				 lBuffer[lBytesRead] = '\0';
				 theApp.Decode(lBuffer, lBytesRead);
				 if (theApp.IsProperlyDecrypted(lBuffer, lBytesRead))
				 {
					   char * lStart  = lBuffer;
					   char * lEnd    = lBuffer;
					   char * lLength = lBuffer + strlen(lBuffer);

					   // vervang de newlines en returns door eos en space...
					   while (lStart < lLength)
					   {
						  if (*lStart == '\r')
						  {
							 *lStart = ' ';
						  }
						  if (*lStart == '\n')
						  {
							 *lStart = '\0';
						  }
						  lStart++;
					   }

					   // doorloop lBuffer
					   lStart = lBuffer;
					   while (lEnd < lLength)
					   {
						  // schuif een regel op
						  lEnd += 80;
						  
						  if (lEnd > lLength)
						  {
							 // over het einde heen, compenseer tot einde string
							 lEnd = lLength;
						  }
						  else
						  {
							 // knip in een woord, schuif op naar links
							 while (*lEnd != ' ')
							 {
								lEnd--;
							 }

							 // breek de regel af tussen de woorden
							 *lEnd = '\0';
						  }

						  // print de regel tot de eerste eos
						  PrintDC.TextOutA(x, y, lStart, strlen(lStart));
						  y += LINE_SPACING;
						  if (y > PAGE_SIZE)
						  {
							  PrintDC.EndPage();
							  PrintDC.StartPage();
							  y = START_HEIGHT;
						  }

						  // corrigeer indexen
						  if (strlen(lStart) < (unsigned int)(lEnd - lStart - 1))
						  { 
							 *lEnd = ' ';
							 lStart = lStart + strlen((const char *)lStart) + 1;
							 lEnd = lStart;
						  }
						  else
						  {
							 lStart = lEnd + 1;
							 lEnd = lStart;
						  }
					  }
				  }
			  }
		   }
		   // if every date on a new page...
		   // PrintDC.EndPage();
		   NumberOfDays--;
		   theDate.Add(0,0,1);
	   }
	   PrintDC.EndDoc();
	}
}