//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         ImageDlg.cpp
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  One of the more recent additions for Diary, the SImageDlg class displays the
//                image dialog, and handles all interactions with it. This includes loading and
//                deleting images, and switching the image slot visible.
//
//////////////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <conio.h>
#include <direct.h>
#include <io.h>

#include "stdafx.h"
#include "diarydlg.h"
#include "imagedlg.h"
#include "toolkit.h"
#include "diary.h"
#include "imagedlg.h"
#include "monitors.h"
#include "multimonitor.h"
#include "monitordc.h"
#include "ResizeDlg.h"
#include "BitmapEx.h"
#include "Todolist.h"

char slotNumber[MAX_SLOT + 1] = {'0', '1', '2', '3', '4',
						         '5', '6', '7', '8', '9',
							     'A', 'B', 'C', 'D', 'E', 'F'};
// SImageDlg dialog

IMPLEMENT_DYNAMIC(SImageDlg, CResizeDlg)
SImageDlg::SImageDlg(CWnd* pParent)
          :CResizeDlg(SImageDlg::IDD, pParent),
		   mManipulator(0)
{
   mSlotNumber = 0;
}

SImageDlg::~SImageDlg()
{
}

void SImageDlg::CrummyInvalidate()
{
   static int tumbler = -1;
   tumbler = -tumbler;
   WINDOWPLACEMENT lWinPlace;
   this->GetWindowPlacement(&lWinPlace);
   lWinPlace.rcNormalPosition.right += tumbler;
   lWinPlace.rcNormalPosition.bottom += tumbler;
   this->SetWindowPlacement(&lWinPlace);
}

void SImageDlg::DoDataExchange(CDataExchange* pDX)
{
	CResizeDlg::DoDataExchange(pDX);
	DDX_Control(pDX, ID_IMGWEEKDAY, mWeekday);
	DDX_Control(pDX, IDC_IMGSELECT, mImageSelect);
}

BEGIN_MESSAGE_MAP(SImageDlg, CResizeDlg)
   ON_WM_PAINT()
   ON_WM_WINDOWPOSCHANGED()
   ON_BN_CLICKED(ID_IMG_LOAD, OnBnClickedImgLoad)
   ON_BN_CLICKED(ID_IMG_DELETE, OnBnClickedImgDelete)
   ON_BN_CLICKED(ID_IMG_SLIDESHOW, OnBnClickedImgSlideshow)
   ON_BN_CLICKED(ID_IMG_EXPORT, &SImageDlg::OnBnClickedImgExport)
   ON_BN_CLICKED(IDC_IMGPREV, &SImageDlg::OnBnClickedImgprev)
   ON_BN_CLICKED(IDC_IMGNEXT, &SImageDlg::OnBnClickedImgnext)
   ON_CBN_SELCHANGE(IDC_IMGQUALITY, &SImageDlg::OnCbnSelchangeImgquality)
   ON_CBN_SELCHANGE(IDC_IMGSELECT, &SImageDlg::OnCbnSelchangeImgselect)
END_MESSAGE_MAP()


// SImageDlg message handlers

int  SImageDlg::DoModal(int pSlot)
{
   ASSERT(pSlot >= MIN_SLOT);
   ASSERT(pSlot <= MAX_SLOT);

   this->mSlotNumber = pSlot;

   return CResizeDlg::DoModal();
}

void SImageDlg::OnBnClickedImgLoad()
{
   CString strFilter;
   CSimpleArray<GUID> aguidFileTypes;
   HRESULT hResult;

   DWORD lDirReturn = 0;
   char  *lPath = new char[MAX_STRING];

   TK_LOG0("Retrieve the path of execution");
   lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);

   hResult = imgOriginal.GetExporterFilterString(strFilter,aguidFileTypes);
   if (FAILED(hResult))
   {
      CString fmt;
      fmt.Format("GetExporterFilter failed");
      MessageBox(fmt);
      SetCurrentDirectory(lPath);
	  delete [] lPath;
      return;
   }

   CFileDialog dlg(TRUE, NULL, NULL, OFN_FILEMUSTEXIST, strFilter);
   m_nFilterLoad = 2;
   dlg.m_ofn.nFilterIndex = m_nFilterLoad;

   hResult = (int)dlg.DoModal();
   if(hResult != IDOK) 
   {
      SetCurrentDirectory(lPath);
	  delete [] lPath;
      return;
   }

   m_nFilterLoad = dlg.m_ofn.nFilterIndex;
   TK_LOG0("This loads an external image, no need to decrypt");
   CString inPath = "";
   OSVERSIONINFO theInfo;
   ZeroMemory(&theInfo, sizeof(OSVERSIONINFO));
   theInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
   ::GetVersionEx(&theInfo);
   if (theInfo.dwMajorVersion <= 5)
   {
	   // this should work for older versions...
	   char buffer[MAX_STRING];
	   _getcwd(buffer, MAX_STRING - 1);
	   inPath = buffer;
   }
   else
   {
	   // and this works on Vista...
       inPath = dlg.GetFolderPath();
   }

   inPath += "\\";
   inPath += dlg.GetFileName();
   imgOriginal.Destroy();
   hResult = imgOriginal.Load(inPath);
   if (FAILED(hResult)) 
   {
      CString fmt;
      fmt.Format("Load image failed");
      MessageBox(fmt);
	  SetCurrentDirectory(lPath);
	  delete [] lPath;
      return;
   }

   SetCurrentDirectory(lPath);
   delete [] lPath;

   CString strFileName = this->GetCurrentImageFilename(this->mSlotNumber);
   hResult = imgOriginal.Save(strFileName);
   if (FAILED(hResult)) 
   {
      CString fmt;
      fmt.Format("Save image failed");
      MessageBox(fmt);
      return;
   }

   TK_LOG0("Encrypt image after saving");
   theApp.EncryptFile(strFileName.GetBuffer());

   this->GetDlgItem(ID_IMG_DELETE)->EnableWindow(true);
   this->GetDlgItem(ID_IMG_EXPORT)->EnableWindow(true);

   this->SetChecks();
   CrummyInvalidate();

   if (DoesImageFileExistForCurrentDate() == true)
   {
      this->GetDlgItem(ID_IMG_SLIDESHOW)->EnableWindow(true);
   }
   else
   {
      this->GetDlgItem(ID_IMG_SLIDESHOW)->EnableWindow(false);
   }
}

// this paint routine does one thing on top of what CResizeDlg::OnPaint() does:
// it paints the image in the image dialog. Funny thing is that we are not 
// really painting it into the window that shows because of the resource, 
// but directly on the dialog, inside the border set by the image frame.

void SImageDlg::OnPaint()
{
   CPaintDC dc(this); // device context for painting
   CResizeDlg::OnPaint();
 
   TK_LOG0("Is there an image to paint?");
   TK_LOG0("Determine the window size first");
   WINDOWPLACEMENT lWinPlace;
   this->GetDlgItem(IDC_PICTURE)->GetWindowPlacement(&lWinPlace);

   int lTop    = lWinPlace.rcNormalPosition.top;
   int lLeft   = lWinPlace.rcNormalPosition.left;
   int lBottom = lWinPlace.rcNormalPosition.bottom;
   int lRight  = lWinPlace.rcNormalPosition.right;

#define H_OFFSET   (14)
#define V_OFFSET   (36)

   TK_LOG0("Yes, so calculate the scaling to make it fit");
   int lMaxWidth = lRight - lLeft - 5; // - 140;
   int lMaxHeight = lBottom - lTop - 5; // - 65;

   // report the pixelsize for easy reference
   CString lPixelSize;
   lPixelSize.Format("%dx%d", lMaxWidth, lMaxHeight);
   this->GetDlgItem(IDC_PIXELSIZE)->SetWindowText(lPixelSize);

   // set a clipping rect
   mRect.SetRect(H_OFFSET, V_OFFSET, H_OFFSET + lMaxWidth, V_OFFSET + lMaxHeight);
   dc.SetBoundsRect(&mRect, DCB_DISABLE);

   // clear out the window
   CBrush lBrush;
   lBrush.CreateSysColorBrush(COLOR_APPWORKSPACE);
   dc.FillRect(&mRect, &lBrush);

   if (!imgOriginal.IsNull()) 
   {
      int lRealWidth = imgOriginal.GetWidth();
      int lRealHeight = imgOriginal.GetHeight();
      double lHorScalefactor = (double)lMaxWidth / (double)lRealWidth;
      double lVertScalefactor = (double)lMaxHeight / (double)lRealHeight;
      double lScalefactor = lHorScalefactor < lVertScalefactor 
                          ? lHorScalefactor : lVertScalefactor;
      int lScaledWidth = (int)(lRealWidth * lScalefactor);
      int lScaledHeight = (int)(lRealHeight * lScalefactor);
      int lHorOffset = H_OFFSET + (lMaxWidth - lScaledWidth) / 2;
      int lVertOffset = V_OFFSET + (lMaxHeight - lScaledHeight) / 2;

      TK_LOG0("and draw the image.");

      // Actually, the StretchBlt function is much too rough to correctly 
      // display the image, a true color pixel resize should be done here...

		  if (mManipulator == 0)
		  {
			  imgOriginal.Save("1D2T3H6JK.bmp");
			  mManipulator = new CBitmapEx();
			  mManipulator->Load("1D2T3H6JK.bmp");
			  _unlink("1D2T3H6JK.bmp");

			  SSetting *lSettingQuality = SDiaryDlg::Instance()->GetSettings()->GetSetting(SET_IMAGE_QUALITY);
			  if (lSettingQuality != 0)
			  {
				  _RESAMPLE_MODE lNewMode = (_RESAMPLE_MODE)(lSettingQuality->GetValueAsNumber());
			      mManipulator->SetResampleMode(lNewMode);
			  }
			  else
			  {
	  			  mManipulator->SetResampleMode(RM_BILINEAR);
			  }

    		  mManipulator->Scale((long)(lScalefactor * 100), (long)(lScalefactor * 100));
		  }

		  mManipulator->Draw(dc, lHorOffset, lVertOffset);
   }
}

// OnInitDialog is used to restore the position settings from the settings 
// object, and use them to position the window. Also, the correct slot 
// is selected, based on the slot number passed in with the DoModal() call.

BOOL SImageDlg::OnInitDialog(void)
{
   BOOL lRetCode = CResizeDlg::OnInitDialog();

   AddControl(IDC_IMGNEXT, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_PICTURE, CST_NONE, CST_RESIZE, CST_NONE, CST_RESIZE, 1);
   AddControl(IDOK, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(ID_IMG_LOAD, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(ID_IMG_DELETE, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_IMGSELECT, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_FILLED_SLOTS, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_SELECT_IMAGE, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_MARKER_0, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);   
   AddControl(IDC_MARKER_7, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);   
   AddControl(IDC_MARKER_8, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);   
   AddControl(IDC_MARKER_15, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK0, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK1, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK2, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK3, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK4, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK5, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK6, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK7, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK8, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK9, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK10, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK11, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK12, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK13, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK14, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK15, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(ID_IMG_SLIDESHOW, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(ID_IMG_EXPORT, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_PIXELSIZE, CST_NONE, CST_REPOS, CST_NONE, CST_REPOS, 1);
   AddControl(IDC_IMGQUALITY, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_IMGSTATIC, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);

   TK_LOG0("Read the position settings and restore them");
   SSetting *lSettingTop    = SDiaryDlg::Instance()->GetSettings()->GetSetting(SET_IMAGE_TOP);
   SSetting *lSettingLeft   = SDiaryDlg::Instance()->GetSettings()->GetSetting(SET_IMAGE_LEFT);
   SSetting *lSettingWidth  = SDiaryDlg::Instance()->GetSettings()->GetSetting(SET_IMAGE_WIDTH);
   SSetting *lSettingHeight = SDiaryDlg::Instance()->GetSettings()->GetSetting(SET_IMAGE_HEIGHT);

   if (lSettingTop    != NULL &&
       lSettingLeft   != NULL &&
	   lSettingWidth  != NULL &&
	   lSettingHeight != NULL)
   {
      TK_LOG0("Position settings found, positioning window");

      int lTop  = lSettingTop->GetValueAsNumber();
      int lLeft = lSettingLeft->GetValueAsNumber();
	  int lWidth = lSettingWidth->GetValueAsNumber();
	  int lHeight = lSettingHeight->GetValueAsNumber();

	  // make sure it will be on-screen, but keep the original position in config
	  this->SetWindowPos(0, lLeft, lTop, lWidth, lHeight, 0);
	  if (CMonitors::IsOnScreen( this ) == false)
	  {
		  CMonitor thePrimary = CMonitors::GetPrimaryMonitor();
		  thePrimary.CenterWindowToMonitor(this, true);
		  mWindowCentered = true;
          WINDOWPLACEMENT lWinPlace;
          this->GetWindowPlacement(&lWinPlace);
          mTop = lWinPlace.rcNormalPosition.top;
          mLeft = lWinPlace.rcNormalPosition.left;
	  }
   }

   // fill the image quality combobox
   CComboBox *theImageQuality = (CComboBox *)(this->GetDlgItem(IDC_IMGQUALITY));
   theImageQuality->Clear();
   theImageQuality->InsertString(0, "Basic/Fast");
   theImageQuality->InsertString(1, "Better/Average");
   theImageQuality->InsertString(2, "Best/Slow");
   SSetting *lSettingQuality = SDiaryDlg::Instance()->GetSettings()->GetSetting(SET_IMAGE_QUALITY);
   if (lSettingQuality != 0)
   {
	   theImageQuality->SetCurSel(lSettingQuality->GetValueAsNumber());
   }
   else
   {
       theImageQuality->SetCurSel(1);
   }

   this->SelectSlot(this->mSlotNumber);

   CString strFileName = this->GetCurrentImageFilename();

#ifdef DEFAULT_LOAD_SLOT0_IF_NO_IMAGES
   if (!this->DoesImageFileExistForCurrentDate())
   {
      this->OnBnClickedImgLoad();

      if (!this->DoesImageFileExistForCurrentDate())
      {
         CResizeDlg::OnCancel();
      }
   }
#endif

   // fill the combobox with digits and letters
   mImageSelect.ResetContent();
   for (int i = 0; i <= MAX_SLOT; i++)
   {
	   char theSlot[7];
	   strcpy_s(theSlot, sizeof(theSlot), "Slot  ");
	   theSlot[5] = slotNumber[i];
	   theSlot[6] = '\0';
	   mImageSelect.AddString(theSlot);
   }
   if (mImageSelect.GetCurSel() == -1)
   {
       mImageSelect.SetCurSel(this->mSlotNumber);
   }

   this->SetChecks();

   UpdateWindow();

   return lRetCode;
}

void SImageDlg::OnBnClickedImgDelete()
{
   TK_LOG0("Image Delete Clicked");
   CString lMsg = "Do you really want to\ndelete the image?";
   if (MessageBox(lMsg, "Image Delete", MB_YESNO) == IDYES)
   {
      TK_LOG0("Image Delete Confirmed");
      CString lFileName = this->GetCurrentImageFilename(this->mSlotNumber);

      lFileName = lFileName.Left(lFileName.GetLength() - 1);
      lFileName += "x";

      TK_LOG1("File to delete: %s", lFileName.GetBuffer());

      imgOriginal.Destroy();
      Invalidate();
      UpdateWindow();

      int lRet = _unlink(lFileName.GetBuffer());
      TK_LOG1("Return code from _unlink = %d", lRet);

      if (lRet == -1)
      {
         TK_LOG0("Delete failed");
         CString fmt;
         fmt.Format("Delete image failed");
         MessageBox(fmt);
         return;
      }
      TK_LOG0("Delete succeeded");
   }
   this->SetChecks();
   CrummyInvalidate();

   this->GetDlgItem(ID_IMG_DELETE)->EnableWindow(false);
   this->GetDlgItem(ID_IMG_EXPORT)->EnableWindow(false);
   
   if (DoesImageFileExistForCurrentDate() == true)
   {
      this->GetDlgItem(ID_IMG_SLIDESHOW)->EnableWindow(true);
   }
   else
   {
      this->GetDlgItem(ID_IMG_SLIDESHOW)->EnableWindow(false);
   }

#ifdef CANCEL_IMAGE_DIALOG_WHEN_EMPTY
   if (!this->DoesImageFileExistForCurrentDate())
   {
      CResizeDlg::OnCancel();
   }
#endif
}

// SavePosition does nothing else than retrieve the window position, 
// and store it in the settings object. 

void SImageDlg::SavePosition(int x, int y, int width, int height)
{
   TK_LOG0("Save window position to settings.dat");
   WINDOWPLACEMENT lWinPlace;
   this->GetWindowPlacement(&lWinPlace);

   // cancel save if window centered, and not moved yet...
   if (mWindowCentered == true &&
	   (mTop == lWinPlace.rcNormalPosition.top ||
	    mLeft == lWinPlace.rcNormalPosition.left))
   {
	   return;
   }

   // ok, we save so set the flag to off
   mWindowCentered = false;
   mTop = -1;
   mLeft = -1;

   char *lTop = new char[MAX_STRING];
   if (y == -1)
   {
	   y = lWinPlace.rcNormalPosition.top;
   }
   _itoa_s(y, lTop, MAX_STRING - 1, 10);
   SSetting *lTopSSetting = new SSetting(SET_IMAGE_TOP, lTop);
   SDiaryDlg::Instance()->GetSettings()->AddSetting(lTopSSetting);
   delete [] lTop;

   char *lLeft = new char[MAX_STRING];
   if (x == -1)
   {
	   x = lWinPlace.rcNormalPosition.left;
   }
   _itoa_s(x, lLeft, MAX_STRING - 1, 10);
   SSetting *lLeftSSetting = new SSetting(SET_IMAGE_LEFT, lLeft);
   SDiaryDlg::Instance()->GetSettings()->AddSetting(lLeftSSetting);
   delete [] lLeft;

   char *lWidth = new char[MAX_STRING];
   if (width == -1)
   {
       width = lWinPlace.rcNormalPosition.right - 
	           lWinPlace.rcNormalPosition.left;
   }
   _itoa_s(width, lWidth, MAX_STRING - 1, 10);
   SSetting *lWidthSSetting = new SSetting(SET_IMAGE_WIDTH, lWidth);
   SDiaryDlg::Instance()->GetSettings()->AddSetting(lWidthSSetting);
   delete [] lWidth;

   char *lHeight = new char[MAX_STRING];
   if (height == -1)
   {
       height = lWinPlace.rcNormalPosition.bottom - 
	            lWinPlace.rcNormalPosition.top;
   }
   _itoa_s(height, lHeight, MAX_STRING - 1, 10);
   SSetting *lHeightSSetting = new SSetting(SET_IMAGE_HEIGHT, lHeight);
   SDiaryDlg::Instance()->GetSettings()->AddSetting(lHeightSSetting);
   delete [] lHeight;

   TK_LOG0("Make sure the settings are flushed to disk");
   SDiaryDlg::Instance()->GetSettings()->WriteFile();
}

// This basically just cancels the dialog leaving the Status Quo

void SImageDlg::OnOK() 
{
   CResizeDlg::OnOK();
}

// this routine is called if one of the radiobutton handlers kicks in.
// it makes sure the correct image is displayed.

void SImageDlg::SelectSlot(int pSlotNumber)
{
   TK_LOG0("Perform any actions you need on the old slot");

   TK_LOG1("Set the slot number of the dialog to %d", pSlotNumber);
   this->mSlotNumber = pSlotNumber;

   TK_LOG0("Perform actions to update the dialog for the new slot");
   imgOriginal.Destroy();

   HRESULT hResult;
   if (this->DoesImageFileExistForCurrentDate(this->mSlotNumber))
   {
      CString lFileName = this->GetCurrentImageFilename(this->mSlotNumber);

      TK_LOG0("Decrypt internal image before loading");
      theApp.DecryptFile(lFileName.GetBuffer());

      hResult = imgOriginal.Load(lFileName);
      if (FAILED(hResult)) 
      {
         CString fmt;
         fmt.Format("Load image failed");
         MessageBox(fmt);
         TK_LOG0("Remove the unencrypted file again");
         _unlink(lFileName.GetBuffer());
		 this->GetDlgItem(ID_IMG_DELETE)->EnableWindow(true);
         this->GetDlgItem(ID_IMG_EXPORT)->EnableWindow(false);
         return;
      }

      TK_LOG0("Remove the unencrypted file again");
      _unlink(lFileName.GetBuffer());
     
      this->GetDlgItem(ID_IMG_DELETE)->EnableWindow(true);
      this->GetDlgItem(ID_IMG_EXPORT)->EnableWindow(true);
   }
   else
   {
      this->GetDlgItem(ID_IMG_DELETE)->EnableWindow(false);
      this->GetDlgItem(ID_IMG_EXPORT)->EnableWindow(false);
   }

   if (DoesImageFileExistForCurrentDate() == true)
   {
      this->GetDlgItem(ID_IMG_SLIDESHOW)->EnableWindow(true);
   }
   else
   {
      this->GetDlgItem(ID_IMG_SLIDESHOW)->EnableWindow(false);
   }

   this->mImageSelect.SetCurSel(pSlotNumber);

   char  *lTitle = new char[MAX_STRING];

   sprintf_s(lTitle, MAX_STRING - 1, 
             "%s - Image", 
             this->GetCurrentImageFilename(this->mSlotNumber, true));
   TK_LOG1("Set Window title to %s", lTitle);
   SetWindowText(lTitle);
   delete [] lTitle;

   delete mManipulator;
   mManipulator = 0;

   // UpdateWindow();   // just isn't enough...
   UpdateWeekday();
   CrummyInvalidate();
}

// this routine is called to update the ten checkboxes according to the 
// existence of image files for the corresponding slots...

void SImageDlg::SetChecks(void)
{
   for (int lSlotNumber = MIN_SLOT; lSlotNumber <= MAX_SLOT; lSlotNumber++)
   {
      UINT lCheck;

      TK_LOG0("Determine check value based on existence of file");
      if (this->DoesImageFileExistForCurrentDate(lSlotNumber))
      {
         lCheck = BST_CHECKED;
      }
      else
      {
         lCheck = BST_UNCHECKED;
      }

      TK_LOG0("Update corresponding checkbox");
      switch (lSlotNumber)
      {
      case 0:
         this->CheckDlgButton(IDC_CHECK0, lCheck);
         break;
      case 1:
         this->CheckDlgButton(IDC_CHECK1, lCheck);
         break;   
      case 2:
         this->CheckDlgButton(IDC_CHECK2, lCheck);
         break;   
      case 3:
         this->CheckDlgButton(IDC_CHECK3, lCheck);
         break;   
      case 4:
         this->CheckDlgButton(IDC_CHECK4, lCheck);
         break;   
      case 5:
         this->CheckDlgButton(IDC_CHECK5, lCheck);
         break;   
      case 6:
         this->CheckDlgButton(IDC_CHECK6, lCheck);
         break;   
      case 7:
         this->CheckDlgButton(IDC_CHECK7, lCheck);
         break;   
      case 8:
         this->CheckDlgButton(IDC_CHECK8, lCheck);
         break;   
      case 9:
         this->CheckDlgButton(IDC_CHECK9, lCheck);
         break;   
      case 10:
         this->CheckDlgButton(IDC_CHECK10, lCheck);
         break;        
	  case 11:
         this->CheckDlgButton(IDC_CHECK11, lCheck);
         break;        
	  case 12:
         this->CheckDlgButton(IDC_CHECK12, lCheck);
         break;        
	  case 13:
         this->CheckDlgButton(IDC_CHECK13, lCheck);
         break;        
	  case 14:
         this->CheckDlgButton(IDC_CHECK14, lCheck);
         break;        
	  case 15:
         this->CheckDlgButton(IDC_CHECK15, lCheck);
         break;        
	  default:
         MessageBox("Invalid Slot number in SImageDlg::SetCheck()");
      }
   }
}

void SImageDlg::OnBnClickedImgSlideshow()
{
	for (int i = MIN_SLOT; i <= MAX_SLOT; i++)
	{
		if (this->DoesImageFileExistForCurrentDate(i))
		{
			this->SelectSlot(i);
            OnPaint();
		    Sleep(1500);
		}
	}
}

bool SImageDlg::DoesImageFileExistForCurrentDate(int pSlotNumber)
{
   TK_TRACER("SImageDlg::DoesImageFileExistForCurrentDate(int pSlotNumber)");

   if (pSlotNumber == ANY_SLOT)
   {
      bool lTotalResult = false;
      for (int i = MIN_SLOT; i <= MAX_SLOT; i++)
      {
         TK_LOG0("Check all ten slots anyway, because of old image conversion");
         boolean lResult = this->DoesImageFileExistForCurrentDate(i);
         if (lResult) 
         {
            lTotalResult = true;
         }
      }
      return lTotalResult;
   }
   else
   {
      CString lFileName = this->GetCurrentImageFilename(pSlotNumber, true);

      FILE *lFile = fopen(lFileName.GetBuffer(), "r");
      if (lFile != NULL)
      {
         fclose(lFile); 
         return true;
      }
   }

   return false;
}

CString SImageDlg::GetCurrentImageFilename(int pSlotNumber, bool pEncryptedForm)
{
   TK_TRACER("SImageDlg::GetCurrentImageFilename(int pSlotNumber, bool pEncryptedForm)");

   CString lFilename;
   DWORD lDirReturn = 0;
   char  *lPath = new char[MAX_STRING];

   TK_LOG0("Retrieve the path of execution");
   lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);

   char lExtension[5];
   if (pEncryptedForm)
   {
      strcpy_s(lExtension, 4, "jpx");
   }
   else
   {
      strcpy_s(lExtension, 4, "jpg");
   }
   if (pSlotNumber == 0)
   {
      lFilename.Format("%s\\%08d.%s", lPath, SDiaryDlg::Instance()->GetCurrentDate().asDecimal(), lExtension);
   }
   else if (pSlotNumber < 10)
   {
      lFilename.Format("%s\\%08d%01d.%s", lPath, 
                       SDiaryDlg::Instance()->GetCurrentDate().asDecimal(), pSlotNumber, lExtension);
   }
   else 
   {
      lFilename.Format("%s\\%08d%c.%s", lPath, 
                       SDiaryDlg::Instance()->GetCurrentDate().asDecimal(), 
					   'A' + pSlotNumber - 10, lExtension);
   }

   TK_LOG1("Image filename: %s", lFilename.GetBuffer());
   delete [] lPath;
   return lFilename;
}

int SImageDlg::CountImageFiles(void)
{
   TK_TRACER("SImageDlg::CountImageFiles(void)");
   int lCount = 0;
   for (int i = MIN_SLOT; i <= MAX_SLOT; i++)
   {
      if (DoesImageFileExistForCurrentDate(i))
      {
         lCount++;
      }
   }
   return lCount;
}


void SImageDlg::OnBnClickedImgExport()
{
    TK_LOG0("Image Export Clicked");

	CString lInputFileName = this->GetCurrentImageFilename(this->mSlotNumber);

	lInputFileName = lInputFileName.Left(lInputFileName.GetLength() - 1);
	lInputFileName += "x";

	TK_LOG1("File to export: %s", lInputFileName.GetBuffer());
   
   CString strFilter;
   CSimpleArray<GUID> aguidFileTypes;
   HRESULT hResult;

   hResult = imgOriginal.GetExporterFilterString(strFilter,aguidFileTypes);
   if (FAILED(hResult))
   {
      CString fmt;
      fmt.Format("GetExporterFilter failed");
      MessageBox(fmt);
      return;
   }

   CFileDialog dlg(TRUE, 
	               NULL, 
				   NULL, 
				   OFN_NOCHANGEDIR|OFN_HIDEREADONLY|OFN_NOREADONLYRETURN, 
				   strFilter);
   m_nFilterLoad = 2;
   dlg.m_ofn.nFilterIndex = m_nFilterLoad;

   hResult = (int)dlg.DoModal();
   if(hResult != IDOK) 
   {
      return;
   }

   m_nFilterLoad = dlg.m_ofn.nFilterIndex;

   // complete the filepath
   CString lNewPath = dlg.GetPathName();
   CString lExt = dlg.GetFileExt();
   if (lExt.GetLength() == 0)
   {
	   lNewPath += ".jpg";
   }

   TK_LOG0("Save it straight from the image");
   hResult = imgOriginal.Save(lNewPath);
}

void SImageDlg::OnWindowPosChanged(WINDOWPOS* lpwndpos)
{
	CResizeDlg::OnWindowPosChanged(lpwndpos);

    delete mManipulator;
    mManipulator = 0;

	SavePosition(lpwndpos->x, lpwndpos->y, lpwndpos->cx, lpwndpos->cy);
	UpdateWindow();
}

void SImageDlg::OnBnClickedImgprev()
{
    TK_TRACER("SDiaryDlg::OnBnClickedPrev(void)");
	// iterate backward until we find another diary page
	SDate theDate = SDiaryDlg::Instance()->GetCurrentDate();
	int daysTried = 0;
    do
	{
		theDate.Subtract(0, 0, 1);
		daysTried++;

	} while (DoImagesExistForDate(theDate) == false && daysTried < MAX_DAYS_TO_SCAN);

	if (DoImagesExistForDate(theDate) == false)
	{
		MessageBox("No previous image found in over five years", 
			       "Warning", 
				   MB_OK);
		return;
	}
    TK_LOG0("But save diary page and todolist before doing so");
    SDiaryDlg::Instance()->mCurrentTodoList->Save();
    SDiaryDlg::Instance()->SaveOldPage(true);
	SDiaryDlg::Instance()->mNewSelDate = theDate;
    SYSTEMTIME lTime;
	lTime.wYear  = theDate.getYear();
    lTime.wMonth = theDate.getMonth();
	lTime.wDay   = theDate.getDays();
	SDiaryDlg::Instance()->m_pMonthCalendar.SetCurSel(&lTime);
    SDiaryDlg::Instance()->LoadNewPage();
    SDiaryDlg::Instance()->UpdateWeekday();
    SDiaryDlg::Instance()->UpdateStatus();
	this->SetChecks();
    this->UpdateWeekday();
	this->SelectSlot(GetLowestFilledSlot(theDate));
}

void SImageDlg::OnBnClickedImgnext()
{
    TK_TRACER("SDiaryDlg::OnBnClickedNext(void)");
	// iterate forward until we find another diary page
	SDate theDate = SDiaryDlg::Instance()->GetCurrentDate();
	int daysTried = 0;
    do
	{
		theDate.Add(0, 0, 1);
		daysTried++;

	} while (DoImagesExistForDate(theDate) == false && daysTried < MAX_DAYS_TO_SCAN);

	if (DoImagesExistForDate(theDate) == false)
	{
		MessageBox("No next image found in over five years", 
			       "Warning", 
				   MB_OK);
		return;
	}
    TK_LOG0("But save diary page and todolist before doing so");
    SDiaryDlg::Instance()->mCurrentTodoList->Save();
    SDiaryDlg::Instance()->SaveOldPage(true);
	SDiaryDlg::Instance()->mNewSelDate = theDate;
    SYSTEMTIME lTime;
	lTime.wYear  = theDate.getYear();
    lTime.wMonth = theDate.getMonth();
	lTime.wDay   = theDate.getDays();
	SDiaryDlg::Instance()->m_pMonthCalendar.SetCurSel(&lTime);
    SDiaryDlg::Instance()->LoadNewPage();
    SDiaryDlg::Instance()->UpdateWeekday();
    SDiaryDlg::Instance()->UpdateStatus();
	this->SetChecks();
    this->UpdateWeekday();
	this->SelectSlot(GetLowestFilledSlot(theDate));
}

bool SImageDlg::DoImagesExistForDate(SDate &theDate)
{
	CString theFilename;
	CString theExtension = ".jpx";
	CString theFullSpec;
	theFilename.Format("%08d", theDate.asDecimal());

	for (int i = MIN_SLOT; i <= MAX_SLOT; i++)
	{
		if (i == 0)
		{
			theFullSpec = theFilename + theExtension;
		}
		else if ( i < 10)
		{
			theFullSpec.Format("%s%d%s", theFilename, i, theExtension);
		}
		else
		{
			theFullSpec.Format("%s%c%s", theFilename, 'A' + i - 10, theExtension);
		}

		if (_access(theFullSpec, 6) != -1)
		{
			return true;
		}
	}

	return false;
}

int SImageDlg::GetLowestFilledSlot(SDate &theDate)
{
	CString theFilename;
	CString theExtension = ".jpx";
	CString theFullSpec;
	theFilename.Format("%08d", theDate.asDecimal());

	for (int i = MIN_SLOT; i <= MAX_SLOT; i++)
	{
		if (i == 0)
		{
			theFullSpec = theFilename + theExtension;
		}
		else if ( i < 10)
		{
			theFullSpec.Format("%s%d%s", theFilename, i, theExtension);
		}
		else
		{
			theFullSpec.Format("%s%c%s", theFilename, 'A' + i - 10, theExtension);
		}

		if (_access(theFullSpec, 6) != -1)
		{
			return i;
		}
	}

	return -1;
}

void SImageDlg::UpdateWeekday(void)
{
   TK_TRACER("SImageDlg::UpdateWeekday()");
   CWaitCursor lWaiter;

   CString lMsg;
   lMsg.Format("%s, %s %d%s, %4d", 
               SDiaryDlg::Instance()->GetCurrentDate().dayAsLongString(),
               SDate::monthAsLongString((SDiaryDlg::Instance()->GetCurrentDate().getMonth())), 
               SDiaryDlg::Instance()->GetCurrentDate().getDays(), 
               SDate::getDateSuffix(SDiaryDlg::Instance()->GetCurrentDate().getDays()), 
               SDiaryDlg::Instance()->GetCurrentDate().getYear());
   mWeekday.SetWindowText(lMsg.GetBuffer());
}

void SImageDlg::OnCbnSelchangeImgquality()
{
    CComboBox *theImageQuality = (CComboBox *)(this->GetDlgItem(IDC_IMGQUALITY));
	int theChoice = theImageQuality->GetCurSel();

    char *lChoice = new char[MAX_STRING];
    _itoa_s(theChoice, lChoice, MAX_STRING - 1, 10);
    SSetting *lChoiceSSetting = new SSetting(SET_IMAGE_QUALITY, lChoice);
    SDiaryDlg::Instance()->GetSettings()->AddSetting(lChoiceSSetting);
    delete [] lChoice;

    TK_LOG0("Make sure the settings are flushed to disk");
    SDiaryDlg::Instance()->GetSettings()->WriteFile();

	CrummyInvalidate();
}

int SImageDlg::SlotNumberFromChar(char theSlotChar)
{
	int lSlot = -1;
	switch (theSlotChar)
	{
	case '0': lSlot = 0; break;
	case '1': lSlot = 1; break;
	case '2': lSlot = 2; break;
	case '3': lSlot = 3; break;
	case '4': lSlot = 4; break;
	case '5': lSlot = 5; break;
	case '6': lSlot = 6; break;
	case '7': lSlot = 7; break;
	case '8': lSlot = 8; break;
	case '9': lSlot = 9; break;
	case 'A': lSlot = 10; break;
	case 'B': lSlot = 11; break;
	case 'C': lSlot = 12; break;
	case 'D': lSlot = 13; break;
	case 'E': lSlot = 14; break;
	case 'F': lSlot = 15; break;
	default: lSlot = -1; break;
	}

	return lSlot;
}




void SImageDlg::OnCbnSelchangeImgselect()
{
	// Select the slot that was selected
	int theNewSlot = mImageSelect.GetCurSel();
	SelectSlot(theNewSlot);
}
