//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         SShortcut.cpp
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  Implementation of Class SShortcut.
//
//////////////////////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "DiaryDlg.h"
#include "SShortcut.h"

SShortcut::SShortcut(SLabelDlg *pLabelDlg, 
					 int pSlotNumber)
          :mShortcutDlg(pLabelDlg), 
		   mSlotNumber(pSlotNumber),
		   mLabelSetting(0),
	       mTimeSetting(0),
	       mInsertSetting(0)
{
}

SShortcut::~SShortcut()
{
}

void SShortcut::ReadFromSettings()
{
	// set the corresponding settings objects if not done yet...
	if (mLabelSetting == 0)
	{
		CString theSetting;
		theSetting.Format("SET_LABEL_TEXT_%02d", mSlotNumber);
		mLabelSetting = SDiaryDlg::Instance()->GetSettings()->GetSetting(theSetting.GetBuffer(0));
		if (mLabelSetting == 0)
		{
			mLabelSetting = new SSetting(theSetting.GetBuffer(0), "");
		}
	}

	if (mTimeSetting == 0)
	{
		CString theSetting;
		theSetting.Format("SET_LABEL_TIMESTAMP_%02d", mSlotNumber);
		mTimeSetting = SDiaryDlg::Instance()->GetSettings()->GetSetting(theSetting.GetBuffer(0));
		if (mTimeSetting == 0)
		{
			mTimeSetting = new SSetting(theSetting.GetBuffer(0), "false");
		}
	}
	
	if(mInsertSetting == 0)
	{
		CString theSetting;
		theSetting.Format("SET_LABEL_WORDS_%02d", mSlotNumber);
		mInsertSetting = SDiaryDlg::Instance()->GetSettings()->GetSetting(theSetting.GetBuffer(0));
		if (mInsertSetting == 0)
		{
			mInsertSetting = new SSetting(theSetting.GetBuffer(0), "");
		}
	}
}

void SShortcut::WriteToSettings(bool WriteFile)
{
	SDiaryDlg::Instance()->GetSettings()->AddSetting(mLabelSetting);
	SDiaryDlg::Instance()->GetSettings()->AddSetting(mTimeSetting);
	SDiaryDlg::Instance()->GetSettings()->AddSetting(mInsertSetting);

	if (WriteFile)
	{
        SDiaryDlg::Instance()->GetSettings()->WriteFile();
	}
}

void SShortcut::Revert()
{
	if (mLabelSetting == 0)
	{
		ReadFromSettings();
	}

	mLabelText = mLabelSetting->GetValue();
	CString theBool = mTimeSetting->GetValue();
	if (theBool == "true")
	{
		mTimestamp = true;
	}
	else
	{
		mTimestamp = false;
	}
	mInsertText = mInsertSetting->GetValue();
}

void SShortcut::Apply()
{
	if (mLabelSetting == 0)
	{
		ReadFromSettings();
	}

	mLabelSetting->SetValue(mLabelText);
	CString theBool = mTimestamp?"true":"false";
	mTimeSetting->SetValue(theBool);
	mInsertSetting->SetValue(mInsertText);
}

void SShortcut::ReadFromDialog()
{
	CEdit *theLabel = (CEdit *)(mShortcutDlg->GetDlgItem(mLabelTextControl));
	theLabel->GetWindowText(mLabelText);

	BOOL theCheckboxIsChecked = mShortcutDlg->IsDlgButtonChecked(mTimestampControl);
	if (theCheckboxIsChecked == TRUE)
	{
		mTimestamp = true;
	}
	else
	{
		mTimestamp = false;
	}

	CEdit *theWords = (CEdit *)(mShortcutDlg->GetDlgItem(mInsertTextControl));
	theWords->GetWindowText(mInsertText);
}

void SShortcut::SetOnDialog()
{
	CEdit *theLabel = (CEdit *)(mShortcutDlg->GetDlgItem(mLabelTextControl));
	theLabel->SetWindowText(mLabelText);

	mShortcutDlg->CheckDlgButton(mTimestampControl, mTimestamp);
	
	CEdit *theWords = (CEdit *)(mShortcutDlg->GetDlgItem(mInsertTextControl));
	theWords->SetWindowText(mInsertText);
}

void SShortcut::Couple(int pLabelTextControl, int pTimestampControl, int pInsertTextControl)
{
	ASSERT(pLabelTextControl != 0);
	ASSERT(pTimestampControl != 0);
	ASSERT(pInsertTextControl != 0);

	mLabelTextControl = pLabelTextControl;
    mTimestampControl = pTimestampControl;
	mInsertTextControl = pInsertTextControl;

	CEdit *theLabel = (CEdit *)(mShortcutDlg->GetDlgItem(mLabelTextControl));
	theLabel->SetLimitText(16);
	CEdit *theText  = (CEdit *)(mShortcutDlg->GetDlgItem(mInsertTextControl));
	theText->SetLimitText(MAX_STRING - 1);

	// since Couple is always called from OnInitDialog,
	// we can call ReadFromSettings and SetOnDialog right away...
	SettingsToDialog();
}

void SShortcut::SettingsToDialog()
{
	ReadFromSettings();
	Revert();
	SetOnDialog();
}

void SShortcut::DialogToSettings(bool WriteFile)
{
	ReadFromDialog();
	Apply();
	WriteToSettings(WriteFile);
}

void SShortcut::AddToCombo(CComboBox *theBox)
{
	if (theBox && !mLabelText.IsEmpty())
	{
		theBox->AddString(mLabelText);
	}
}

CString SShortcut::GetLabelText()
{
	return mLabelText;
}

CString SShortcut::GetInsertText()
{
	CString theInsertText = mInsertText;
	if (theInsertText.IsEmpty())
	{
		theInsertText = GetLabelText();
	}
	else
	{
	    theInsertText.Replace("|", "\n");
	}
	return theInsertText;
}

bool SShortcut::NeedsTimestamp()
{
	return mTimestamp;
}

void SShortcut::Clear()
{
	mLabelText = "";
	mTimestamp = false;
	mInsertText = "";
	SetOnDialog();
}

void SShortcut::SetLabelText(CString &theName)
{
	mLabelText = theName;
	mLabelSetting->SetValue(theName);
}