//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         SearchDlg.cpp
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  Implementation of Class SearchDlg.
//
//////////////////////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "diary.h"
#include "SearchDlg.h"
#include "toolkit.h"
#include "searchdlg.h"
#include "setting.h"
#include "settingsfile.h"
#include "diary.h"
#include "diarydlg.h"
#include "searchdlg.h"
#include "monitors.h"
#include "multimonitor.h"
#include "monitordc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define NUMBER_OF_LIMITS  (12)

CString timeLimits[] = { "Anytime", 
						 "1 Year", 
						 "6 Months", 
						 "3 Months", 
						 "2 Months",
						 "1 Month", 
						 "3 Weeks", 
						 "2 Weeks", 
						 "1 Week", 
						 "4 Days", 
						 "2 Days", 
						 "1 Day" };

/////////////////////////////////////////////////////////////////////////////
// SSearchDlg dialog


SSearchDlg::SSearchDlg(CWnd* pParent /*=NULL*/)
   : CResizeDlg(SSearchDlg::IDD, pParent)
{
   //{{AFX_DATA_INIT(SSearchDlg)
   //}}AFX_DATA_INIT
   mSelDate.wYear          = 0; 
   mSelDate.wMonth         = 0; 
   mSelDate.wDayOfWeek     = 0; 
   mSelDate.wDay           = 0; 
   mSelDate.wHour          = 0; 
   mSelDate.wMinute        = 0; 
   mSelDate.wSecond        = 0; 
   mSelDate.wMilliseconds  = 0; 
   mAlways = true;
   mYearOffset = 0;
   mMonthOffset = 0;
   mDaysOffset = 0;
   mSelection = -1;
}


void SSearchDlg::DoDataExchange(CDataExchange* pDX)
{
	CResizeDlg::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SSearchDlg)
	DDX_Control(pDX, IDC_STATUS, m_Status);
	DDX_Control(pDX, IDC_SEARCH_RESULTS, m_SearchResults);
	DDX_Control(pDX, IDC_SEARCH_STRING, m_SearchString);
	//}}AFX_DATA_MAP
	DDX_Control(pDX, IDC_FIND_STRING, m_FindString);
	DDX_Control(pDX, IDC_TIMELIMIT, mTimeLimit);
}


BEGIN_MESSAGE_MAP(SSearchDlg, CResizeDlg)
   //{{AFX_MSG_MAP(SSearchDlg)
   ON_WM_WINDOWPOSCHANGED()
   ON_BN_CLICKED(IDSELECT, OnSelect)
   //}}AFX_MSG_MAP
   ON_BN_CLICKED(ID_EXPORT, OnBnClickedExport)
   ON_LBN_SELCHANGE(IDC_SEARCH_RESULTS, OnLbnSelchangeSearchResults)
   ON_LBN_DBLCLK(IDC_SEARCH_RESULTS, OnLbnDblclkSearchResults)
   ON_BN_CLICKED(IDOK, &SSearchDlg::OnOK)
   ON_BN_CLICKED(IDFIND, &SSearchDlg::OnBnClickedFind)
   ON_EN_CHANGE(IDC_SEARCH_STRING, &SSearchDlg::OnEnChangeSearchString)
   ON_EN_CHANGE(IDC_FIND_STRING, &SSearchDlg::OnEnChangeFindString)
   ON_CBN_SELCHANGE(IDC_TIMELIMIT, &SSearchDlg::OnCbnSelchangeTimelimit)
END_MESSAGE_MAP()

// This method sets the search parameter, and searches all .dry
// files in the current directory for occurences of that string. 
// decryption is used if an encryption keyword has been entered. 

void SSearchDlg::OnOK() 
{
   CWaitCursor lWaiter; 

   // Windows insists on calling this when enter is pressed, 
   // despite the fact that the Find or Select button has the default style. 
   // Let's determine this, and call the right function.....

   CButton *theFindButton = (CButton *)(this->GetDlgItem(IDFIND));
   if (theFindButton->GetButtonStyle() && BS_DEFPUSHBUTTON == BS_DEFPUSHBUTTON)
   {
	   // Find is default, do the find...
	   OnBnClickedFind();
	   return; 
   }

   CButton *theSelectButton = (CButton *)(this->GetDlgItem(IDSELECT));
   if (theSelectButton->GetButtonStyle() && BS_DEFPUSHBUTTON == BS_DEFPUSHBUTTON)
   {
	   // Select is default, do the select...
	   OnSelect();
	   return; 
   }

   CButton *theSearchButton = (CButton *)(this->GetDlgItem(IDOK));
   if (theSearchButton->GetButtonStyle() && BS_DEFPUSHBUTTON == 0)
   {
	   // if not Search as default, cancel the rest...
	   MessageBox("No proper handling of default button programmed");
	   return; 
   }

   // determine the size of the context....
   WINDOWPLACEMENT lWinPlace;
   this->GetWindowPlacement(&lWinPlace);
   int lContextSize = (lWinPlace.rcNormalPosition.right - 
                       lWinPlace.rcNormalPosition.left ) / 13;

   SDate lCurrDate = SDiaryDlg::Instance()->GetCurrentDate();
   SDate lLimitDate = lCurrDate;
   if (mAlways)
   {
      lLimitDate.Set(1900, 1, 1);
   }
   else
   {
      lLimitDate.Subtract(mYearOffset, mMonthOffset, mDaysOffset);
   }

   TK_TRACER("SSearchDlg::OnOK()");
   m_SearchString.GetWindowText(mSearchString);

   if (mSearchString == "")
   {
      AfxMessageBox("Can't search for nothing...");
      return;
   }
   mSearchString.MakeLower();

   m_SearchResults.ResetContent();

   CFileFind finder;
   bool foundOne = false;
   BOOL bWorking = finder.FindFile("*.dry");
   while (bWorking)
   {   
      bWorking = finder.FindNextFile();

      SDate lScanDate;
      lScanDate.Set(finder.GetFileName().GetBuffer());
      
      if (mAlways || finder.GetFileName() == "todolist.dry" || lScanDate > lLimitDate)
      {
         CString lMsg;
         lMsg.Format("Scanning %s", finder.GetFileName());
         m_Status.SetWindowText(lMsg);
         TK_LOG1("Opening file %s", finder.GetFileName().GetBuffer());
         FILE *lFile = fopen(finder.GetFileName(), "rb");
         if (lFile != NULL)
         {
            char lBuffer[MAX_BUFFER + 1];
            int lBytesRead = 0;
            lBytesRead = fread(lBuffer, 1, MAX_BUFFER, lFile);
            TK_LOG1("Opening file %s", finder.GetFileName().GetBuffer());
            fclose(lFile);
            lBuffer[lBytesRead] = '\0';
            theApp.Decode(lBuffer, lBytesRead);
            for (int i = 0; i < lBytesRead; i++)
            {
               if (lBuffer[i] == '\n' ||
                   lBuffer[i] == '\r')
               {
                  lBuffer[i] = ' ';
               }
               lBuffer[i] = tolower(lBuffer[i]);
            }
            TK_LOG0("Take the page buffer in all lowercase");
            char *lSearchWindow = lBuffer;
            while (strstr(lSearchWindow, mSearchString) != NULL)
            {
               TK_LOG0("For each occurrence of the Searchstring");
               char *lTextString = new char[MAX_STRING + 1];
               TK_LOG0("Set a pointer at the start of the found string");
               char *lContext = strstr(lSearchWindow, mSearchString);
               int lCharPos = lContext - lBuffer;
               TK_LOG0("Update the search from pointer to point after the found string");
               lSearchWindow = lContext + mSearchString.GetLength();
               TK_LOG0("Try to go back some characters");
               lContext -= lContextSize;
               if (lContext < lBuffer)
               {
                  TK_LOG0("if underflow, set to start of page");
                  lContext = lBuffer;
               }
               TK_LOG0("If not at start of page, find next space");
               while ((lContext != lBuffer) && (lContext[0] != ' '))
               {
                  lContext++;
               }
               TK_LOG0("Copy the string to display it in the list");
               strncpy_s(lTextString, MAX_STRING, lContext, 2 * lContextSize + mSearchString.GetLength());
               lTextString[2 * lContextSize + mSearchString.GetLength()] = '\0';

#ifdef FIND_LAST_WORDBREAK_TO_END_STRING
               TK_LOG0("Turn the last space into a '\0', thus ending the string");
               char *lLastSpace = strrchr(lTextString, ' ');
               if (lLastSpace != NULL)
               {
                  lLastSpace[1] = '\0';
               }
#endif 
			   foundOne = true;

               CString lMsg;
               if (lContext == lBuffer)
               {
                  lMsg.Format("%s  %06d   %s", finder.GetFileName(), lCharPos, lTextString);
               }
               else
               {
                  lMsg.Format("%s  %06d   ...%s...", finder.GetFileName(), lCharPos, lTextString);
               }
               m_SearchResults.AddString(lMsg);
			   delete [] lTextString;
            }
         }
      }
   }

   m_Status.SetWindowText("");

   if (foundOne == false)
   {
	   MessageBox("Nothing found...", "Info", MB_ICONINFORMATION|MB_OK);
   }
}

void SSearchDlg::OnCancel() 
{
   CWaitCursor lWaiter; 
   
   TK_TRACER("SSearchDlg::OnCancel()");
   CResizeDlg::OnCancel();
}

// This method gets the selected search result from the listbox, 
// and sets the SDiaryDlg to that date. 

void SSearchDlg::OnSelect() 
{
   CWaitCursor lWaiter; 
   
   TK_TRACER("SSearchDlg::OnSelect()");

   // get the currently selected text
   int lCurSel = m_SearchResults.GetCurSel();
   if (lCurSel == LB_ERR)
   {
      AfxMessageBox("Select an occurrence first");
      return;
   }

   CString lSelString;
   m_SearchResults.GetText(lCurSel, lSelString);

   // extract the date from it
   char *lRawDate = new char[MAX_STRING + 1];
   strcpy_s(lRawDate, MAX_STRING, lSelString);
   int lYear;
   int lMonth;
   int lDay;
   sscanf_s(lRawDate, "%04d%02d%02d.dry  %06d", &lYear, &lMonth, &lDay, &mCharPos);
   delete [] lRawDate;
   mSelDate.wYear  = lYear;
   mSelDate.wMonth = lMonth;
   mSelDate.wDay   = lDay;

   CResizeDlg::OnOK();
}


BOOL SSearchDlg::OnInitDialog() 
{
   CWaitCursor lWaiter; 
   
   TK_TRACER("SSearchDlg::OnInitDialog()");
   CResizeDlg::OnInitDialog();

   AddControl(IDC_TIMELIMIT, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 0);
   AddControl(IDC_HOWLONG, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDCANCEL, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(ID_EXPORT, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDSELECT, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDFIND, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDOK, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_FIND_STRING, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_SEARCH_STRING, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_SEARCH_RESULTS, CST_NONE, CST_RESIZE, CST_NONE, CST_RESIZE, 1);

   DWORD lDirReturn = 0;
   char  *lPath = new char[MAX_STRING];
   char  *lTitle = new char[MAX_STRING];

   TK_LOG0("Retrieve the path of execution");
   lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);
   sprintf_s(lTitle, MAX_STRING - 1, "%s - Search", lPath);
   TK_LOG1("Set Window title to %s", lTitle);
   SetWindowText(lTitle);

   TK_LOG0("Read the position settings and restore them");
   SSetting *lSettingTop    = SDiaryDlg::Instance()->GetSettings()->GetSetting(SET_SEARCH_TOP);
   SSetting *lSettingLeft   = SDiaryDlg::Instance()->GetSettings()->GetSetting(SET_SEARCH_LEFT);
   SSetting *lSettingWidth  = SDiaryDlg::Instance()->GetSettings()->GetSetting(SET_SEARCH_WIDTH);
   SSetting *lSettingHeight = SDiaryDlg::Instance()->GetSettings()->GetSetting(SET_SEARCH_HEIGHT);

   if (lSettingTop    != NULL &&
       lSettingLeft   != NULL &&
	   lSettingWidth  != NULL &&
	   lSettingHeight != NULL)
   {
      TK_LOG0("Position settings found, positioning window");

      int lTop = lSettingTop->GetValueAsNumber();
      int lLeft = lSettingLeft->GetValueAsNumber();
	  int lWidth = lSettingWidth->GetValueAsNumber();
	  int lHeight = lSettingHeight->GetValueAsNumber();

	  // make sure it will be on-screen, but keep the original position in config
	  this->SetWindowPos(0, lLeft, lTop, lWidth, lHeight, 0);
	  if (CMonitors::IsOnScreen( this ) == false)
	  {
		  CMonitor thePrimary = CMonitors::GetPrimaryMonitor();
		  thePrimary.CenterWindowToMonitor(this, true);
		  mWindowCentered = true;
          WINDOWPLACEMENT lWinPlace;
          this->GetWindowPlacement(&lWinPlace);
          mTop = lWinPlace.rcNormalPosition.top;
          mLeft = lWinPlace.rcNormalPosition.left;
	  }
   }

   if (mSearchString != "")
   {
      m_SearchString.SetWindowText(mSearchString);
      OnOK();
   }

   if (mSelection != -1)
   {
	   m_SearchResults.SetCurSel(mSelection);
   }

   SetTimeLimitCombobox();

   delete [] lPath;
   delete [] lTitle;
      
   return TRUE;  // return TRUE unless you set the focus to a control
                 // EXCEPTION: OCX Property Pages should return FALSE
}


void SSearchDlg::OnBnClickedExport()
{
   CWaitCursor lWaiter; 
   
   FILE *lFile = fopen("search.txt", "w");
   if (lFile != NULL)
   {
      char *lSelString = new char[MAX_STRING]; 
      for (int i=0;i < m_SearchResults.GetCount();i++)
      {         
       if (m_SearchResults.GetText(i, lSelString) > 0)
         {
            fprintf(lFile, "%s\n", lSelString);
         }
      }
     fclose(lFile);
	 delete [] lSelString;
   }
   else
   {
      MessageBox("Export file could not be created");
   }
}  

void SSearchDlg::SavePosition(int x, int y, int width, int height)
{
   TK_LOG0("Save window position to settings.dat");
   WINDOWPLACEMENT lWinPlace;
   this->GetWindowPlacement(&lWinPlace);

   // cancel save if window centered, and not moved yet...
   if (mWindowCentered == true &&
	   (mTop == lWinPlace.rcNormalPosition.top ||
	    mLeft == lWinPlace.rcNormalPosition.left))
   {
	   return;
   }

   // ok, we save so set the flag to off
   mWindowCentered = false;
   mTop = -1;
   mLeft = -1;

   char *lTop = new char[MAX_STRING];
   if (y == -1)
   {
	   y = lWinPlace.rcNormalPosition.top;
   }
   _itoa_s(y, lTop, MAX_STRING - 1, 10);
   SSetting *lTopSSetting = new SSetting(SET_SEARCH_TOP, lTop);
   SDiaryDlg::Instance()->GetSettings()->AddSetting(lTopSSetting);
   delete [] lTop;

   char *lLeft = new char[MAX_STRING];
   if (x == -1)
   {
	   x = lWinPlace.rcNormalPosition.left;
   }
   _itoa_s(x, lLeft, MAX_STRING - 1, 10);
   SSetting *lLeftSSetting = new SSetting(SET_SEARCH_LEFT, lLeft);
   SDiaryDlg::Instance()->GetSettings()->AddSetting(lLeftSSetting);
   delete [] lLeft;

   char *lWidth = new char[MAX_STRING];
   if (width == -1)
   {
       width = lWinPlace.rcNormalPosition.right - 
	           lWinPlace.rcNormalPosition.left;
   }
   _itoa_s(width, lWidth, MAX_STRING - 1, 10);
   SSetting *lWidthSSetting = new SSetting(SET_SEARCH_WIDTH, lWidth);
   SDiaryDlg::Instance()->GetSettings()->AddSetting(lWidthSSetting);
   delete [] lWidth;

   char *lHeight = new char[MAX_STRING];
   if (height == -1)
   {
       height = lWinPlace.rcNormalPosition.bottom - 
	            lWinPlace.rcNormalPosition.top;
   }
   _itoa_s(height, lHeight, MAX_STRING - 1, 10);
   SSetting *lHeightSSetting = new SSetting(SET_SEARCH_HEIGHT, lHeight);
   SDiaryDlg::Instance()->GetSettings()->AddSetting(lHeightSSetting);
   delete [] lHeight;

   SDiaryDlg::Instance()->GetSettings()->WriteFile();
}

void SSearchDlg::OnLbnSelchangeSearchResults()
{
   CWaitCursor lWaiter; 
   int lCurSel = m_SearchResults.GetCurSel();
   if (lCurSel != LB_ERR)
   {
	   mSelection = lCurSel;
   }
   this->GetDlgItem(IDSELECT)->SetFocus();
}

void SSearchDlg::SetTimeLimitCombobox(void)
{
	for (int i = 0; i < NUMBER_OF_LIMITS; i++)
	{
		mTimeLimit.AddString(timeLimits[i]);
	}
	mTimeLimit.SetCurSel(0);
}


void SSearchDlg::OnLbnDblclkSearchResults()
{
	OnSelect();
}

void SSearchDlg::OnBnClickedFind()
{
   CWaitCursor lWaiter; 
   
   TK_TRACER("SSearchDlg::OnBnClickedFind()");

   // get the currently selected text
   int lCurSel = m_SearchResults.GetCurSel();
   if (lCurSel == LB_ERR)
   {
      lCurSel = -1;
   }

   // find the next hit...
   char *lSelString = new char[MAX_STRING]; 
   bool lFound = false;
   for (int i = ++lCurSel; i < m_SearchResults.GetCount(); i++)
   {      
	   if (m_SearchResults.GetText(i, lSelString) > 0)
       {
           //check if find string is in there....
		   CString lFindString = "";
           m_FindString.GetWindowText(lFindString);
		   lFindString.MakeLower();
           CString theLine = lSelString;
		   if (theLine.Find(lFindString) != -1)
		   {
	           m_SearchResults.SetCurSel(i);
			   lFound = true;
			   break;
		   }
       }
   }
   if (lFound == false)
   {
	   MessageBox("Nothing found...", "Info", MB_ICONINFORMATION|MB_OK);
   }
#ifdef SWITCH_FROM_FIND_TO_SELECT
   else
   {
	   // we only want the select button to become the default....
	   CButton *theSelectButton = (CButton *)(this->GetDlgItem(IDSELECT));
	   theSelectButton->SetButtonStyle(BS_DEFPUSHBUTTON);

	   // so make the others normal...
	   CButton *theSearchButton = (CButton *)(this->GetDlgItem(IDOK));
	   theSearchButton->SetButtonStyle(0);
	   CButton *theFindButton = (CButton *)(this->GetDlgItem(IDFIND));
	   theFindButton->SetButtonStyle(0);
   }
#endif

   delete [] lSelString;
}

void SSearchDlg::OnWindowPosChanged(WINDOWPOS* lpwndpos)
{
	CResizeDlg::OnWindowPosChanged(lpwndpos);

	SavePosition(lpwndpos->x, lpwndpos->y, lpwndpos->cx, lpwndpos->cy);
}

void SSearchDlg::OnEnChangeSearchString()
{
	// we only want the search button to become the default....
	CButton *theSearchButton = (CButton *)(this->GetDlgItem(IDOK));
	theSearchButton->SetButtonStyle(BS_DEFPUSHBUTTON);

	// so make the others normal....
	CButton *theFindButton = (CButton *)(this->GetDlgItem(IDFIND));
	theFindButton->SetButtonStyle(0);
	CButton *theSelectButton = (CButton *)(this->GetDlgItem(IDSELECT));
	theSelectButton->SetButtonStyle(0);
}

void SSearchDlg::OnEnChangeFindString()
{
	// we only want the search button to become the default....
	CButton *theFindButton = (CButton *)(this->GetDlgItem(IDFIND));
	theFindButton->SetButtonStyle(BS_DEFPUSHBUTTON);

	// so make the others normal...
	CButton *theSearchButton = (CButton *)(this->GetDlgItem(IDOK));
	theSearchButton->SetButtonStyle(0);
	CButton *theSelectButton = (CButton *)(this->GetDlgItem(IDSELECT));
	theSelectButton->SetButtonStyle(0);

	m_FindString.GetWindowTextA(mFindString);
}

void SSearchDlg::OnCbnSelchangeTimelimit()
{
    CWaitCursor lWaiter; 
	int theChoice = mTimeLimit.GetCurSel();

	switch (theChoice)
	{
	default:
	case 0:
        TK_LOG0("Anytime choice made...");
        mAlways = true;
        mYearOffset = 0;
        mMonthOffset = 0;
        mDaysOffset = 0;
		break;

	case 1:
        TK_LOG0("1 Year choice made...");
        mAlways = false;
        mYearOffset = 1;
        mMonthOffset = 0;
        mDaysOffset = 0;
		break;

	case 2:
        TK_LOG0("6 Months choice made...");
        mAlways = false;
        mYearOffset = 0;
        mMonthOffset = 6;
        mDaysOffset = 0;
		break;

	case 3:
		TK_LOG0("3 Months choice made...");
        mAlways = false;
        mYearOffset = 0;
        mMonthOffset = 3;
        mDaysOffset = 0;
		break;

	case 4:
        TK_LOG0("2 Months choice made...");
        mAlways = false;
        mYearOffset = 0;
        mMonthOffset = 2;
        mDaysOffset = 0;
		break;

	case 5:
        TK_LOG0("1 Month choice made...");
        mAlways = false;
        mYearOffset = 0;
        mMonthOffset = 1;
        mDaysOffset = 0;
		break;

	case 6:
        TK_LOG0("3 Weeks choice made...");
        mAlways = false;
        mYearOffset = 0;
        mMonthOffset = 0;
        mDaysOffset = 21;
		break;

	case 7:
        TK_LOG0("2 Weeks choice made...");
        mAlways = false;
        mYearOffset = 0;
        mMonthOffset = 0;
        mDaysOffset = 14;
		break;

	case 8:
        TK_LOG0("1 Week choice made...");
        mAlways = false;
        mYearOffset = 0;
        mMonthOffset = 0;
        mDaysOffset = 7;
		break;

	case 9:
        TK_LOG0("4 Days choice made...");
        mAlways = false;
        mYearOffset = 0;
        mMonthOffset = 0;
        mDaysOffset = 4;
		break;

	case 10:
        TK_LOG0("2 Days choice made...");
        mAlways = false;
        mYearOffset = 0;
        mMonthOffset = 0;
        mDaysOffset = 2;
		break;

	case 11: 
        TK_LOG0("1 Day choice made...");
        mAlways = false;
        mYearOffset = 0;
        mMonthOffset = 0;
        mDaysOffset = 1;
		break;
	}
}
