//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         SearchDlg.h
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  Declaration of Class SearchDlg.
//
//////////////////////////////////////////////////////////////////////////////////////////////
#ifndef SEARCHDLG_H_INCLUDED
#define SEARCHDLG_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "diary.h"
#include "settingsfile.h"
#include "setting.h"
#include "afxwin.h"
#include "ResizeDlg.h"

class SDiaryDlg;

//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Class SSearchDlg
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  SSearchDlg handles the interaction with the Search dialog. It is fully 
//                resizeable, and provides for Find functionality within the Search results. 
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Date		Name                  Comments
//	20080921	Andr Slabber         Created
//
//////////////////////////////////////////////////////////////////////////////////////////////
class SSearchDlg : public CResizeDlg
{
// Construction
public:
   SSearchDlg(CWnd* pParent = NULL);   // standard constructor

   SYSTEMTIME GetSelectedDate(void)    { return mSelDate;      };
   CString    GetSearchString(void)    { return mSearchString; };
   CString    GetFindString(void)      { return mFindString;   };
   int        GetCharPos(void)         { return mCharPos;      };
   void       SavePosition(int x, int y, int width, int height);

// Dialog Data
   //{{AFX_DATA(SSearchDlg)
   enum { IDD = IDD_SEARCH };
   CStatic   m_Status;
   CListBox   m_SearchResults;
   CEdit   m_SearchString;
   //}}AFX_DATA


// Overrides
   // ClassWizard generated virtual function overrides
   //{{AFX_VIRTUAL(SSearchDlg)
   protected:
   virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
   //}}AFX_VIRTUAL

private:
   SYSTEMTIME mSelDate;
   int        mCharPos;
   CString    mSearchString;
   bool       mAlways;
   int        mYearOffset;
   int        mMonthOffset;
   int        mDaysOffset;
   int        mSelection;

   // stuff to correctly reposition if offscreen
   bool mWindowCentered;
   int  mTop;
   int  mLeft;

// Implementation
protected:

   // Generated message map functions
   //{{AFX_MSG(SSearchDlg)
   virtual void OnOK();
   virtual void OnCancel();
   afx_msg void OnSelect();
   virtual BOOL OnInitDialog();
   //}}AFX_MSG
   DECLARE_MESSAGE_MAP()
public:
   afx_msg void OnWindowPosChanged(WINDOWPOS* lpwndpos);
   afx_msg void OnBnClickedExport();
   afx_msg void OnLbnSelchangeSearchResults();
   afx_msg void OnBnClickedRadio2();
   afx_msg void OnBnClickedRadio3();
   afx_msg void OnBnClickedRadio4();
   afx_msg void OnBnClickedRadio5();
   afx_msg void OnBnClickedRadio6();
   afx_msg void OnBnClickedRadio7();
   afx_msg void OnBnClickedRadio8();
   afx_msg void OnBnClickedRadio9();
   afx_msg void OnBnClickedRadio11();
   afx_msg void OnBnClickedRadio10();
   afx_msg void OnBnClickedRadio12();
   afx_msg void OnBnClickedRadio13();
   void SetTimeLimitCombobox(void);
   afx_msg void OnLbnDblclkSearchResults();
   afx_msg void OnBnClickedFind();
   CEdit m_FindString;
   afx_msg void OnEnChangeSearchString();
   afx_msg void OnEnChangeFindString();
   afx_msg void OnCbnSelchangeTimelimit();
   CComboBox mTimeLimit;
   CString mFindString;
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif
