//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         Todolist.cpp
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  Implementation of Class Todolist.
//
//////////////////////////////////////////////////////////////////////////////////////////////

#include "TodoList.h"
#include "toolkit.h"

STodoList::STodoList(int pSlot, 
					 SDiaryDlg *theDiaryDlg, 
					 SLabelDlg *pLabelDlg )
{
	mName = "";
	mFullText = "";
	mNameControl = 0;
	mSlotNumber = pSlot;
	mNameSetting = 0;
	mLabelDlg = pLabelDlg;
}

STodoList::~STodoList()
{
}

void STodoList::SetName(CString &pName)
{
	mName = pName;
	mNameSetting->SetValue(pName);
}

void STodoList::Couple(int pLabelTextControl)
{
	ASSERT(pLabelTextControl != 0);

	mNameControl = pLabelTextControl;
}

void STodoList::ReadFromSettings()
{
	// set the corresponding settings objects if not done yet...
	if (mNameSetting == 0)
	{
		CString theSetting;
		theSetting.Format("SET_TODOLIST_NAME_%02d", mSlotNumber);
		mNameSetting = SDiaryDlg::Instance()->GetSettings()->GetSetting(theSetting.GetBuffer(0));
		if (mNameSetting == 0)
		{
			mNameSetting = new SSetting(theSetting.GetBuffer(0), "");
		}
	}
}

void STodoList::WriteToSettings(bool WriteFile)
{
	SDiaryDlg::Instance()->GetSettings()->AddSetting(mNameSetting);

	if (WriteFile)
	{
        SDiaryDlg::Instance()->GetSettings()->WriteFile();
	}
}

void STodoList::Revert()
{
	if (mNameSetting == 0)
	{
		ReadFromSettings();
	}

	mName = mNameSetting->GetValue();
}

void STodoList::Apply()
{
	if (mNameSetting == 0)
	{
		ReadFromSettings();
	}

	mNameSetting->SetValue(mName.c_str());
}

void STodoList::ReadFromDialog()
{
	CEdit *theLabel = (CEdit *)(this->mLabelDlg->GetDlgItem(mNameControl));
	CString theText;
	theLabel->GetWindowText(theText);
	mName = theText;
}

void STodoList::SetOnDialog()
{
	if (mNameControl)
	{
	    CEdit *theLabel = (CEdit *)(this->mLabelDlg->GetDlgItem(mNameControl));
		if (theLabel)
		{
	        theLabel->SetWindowText(mName.c_str());
		}
	}
}

void STodoList::SettingsToDialog()
{
	ReadFromSettings();
	Revert();
	SetOnDialog();
}

void STodoList::DialogToSettings(bool WriteFile)
{
	ReadFromDialog();
	Apply();
	WriteToSettings(WriteFile);
}

std::string STodoList::GetName(bool ellipsis)
{
	if (mName == "")
	{
		if (ellipsis)
		{
			return "...";
		}
		else
		{
		    return "todolist";
		}
	}
	return mName;
}

bool STodoList::Save()
{
   TK_TRACER("SDiaryDlg::SaveTodoList(void)");
   CWaitCursor lWaiter;

   if (SDiaryDlg::Instance()->isListDirty())
   {
      TK_LOG0("something to save, so lets do it");
      CString lFileName;
      lFileName.Format(GetName(false).c_str());
	  lFileName += ".dry";

      TK_LOG0("Open the todo file");
      FILE *lFile = fopen(lFileName.GetBuffer(0), "wb");

      if (lFile == NULL)
      {
         CString lMsg;
         lMsg.Format("File %s could not be opened for writing, \ncontinue with intended action?", 
                     lFileName.GetBuffer());
         if (AfxMessageBox(lMsg, MB_YESNO) == IDNO)
		 {
			 return false;
		 }
		 else
		 {
			 return true;
		 }
      }

      TK_LOG0("file now open, proceed");
      long lMin = 0;
      long lMax = 0;
      char lBuffer[MAX_BUFFER + 1];
      SDiaryDlg::Instance()->m_TodoList.SetSel(0, -1);
      SDiaryDlg::Instance()->m_TodoList.GetSel(lMin, lMax);
      CString theStr = SDiaryDlg::Instance()->m_TodoList.GetSelText();
	  strcpy_s(lBuffer, MAX_BUFFER, theStr);
      SDiaryDlg::Instance()->m_TodoList.SetSel(-1, -1);
      theApp.Encode(lBuffer, lMax - lMin);
      fwrite(lBuffer, 1, lMax - lMin, lFile);

      fclose(lFile);
   }
   return true; 
}

void STodoList::Load()
{
   TK_TRACER("STodoList::Load(void)");
   CWaitCursor lWaiter;

   SDiaryDlg::Instance()->m_TodoList.SetWindowText("");

   CString lFileName;
   lFileName.Format(GetName(false).c_str());
   lFileName += ".dry";

   TK_LOG0("Open the todo file for reading");
   FILE *lFile = fopen(lFileName, "rb");

   if (lFile != NULL)
   {
      TK_LOG0("File open, read and decrypt if needed"); 
      char lBuffer[MAX_BUFFER + 1];

      int lBytesRead = fread(lBuffer, 1, MAX_BUFFER, lFile);
      if (lBytesRead > 0)
      {
         lBuffer[lBytesRead] = '\0';
         theApp.Decode(lBuffer, lBytesRead);
         if (theApp.IsProperlyDecrypted(lBuffer, lBytesRead))
         {
            SDiaryDlg::Instance()->m_TodoList.SetWindowText(lBuffer);
            SDiaryDlg::Instance()->m_TodoList.SetSel(-1, -1);
            SDiaryDlg::Instance()->m_TodoList.EnableWindow(true);
            SDiaryDlg::Instance()->clearListDirty();
         }
         else
         {
            SDiaryDlg::Instance()->m_TodoList.SetWindowText("");
            SDiaryDlg::Instance()->m_TodoList.EnableWindow(false);
         }
      }
      else
      {
         SDiaryDlg::Instance()->m_TodoList.SetWindowText("");
         SDiaryDlg::Instance()->m_TodoList.EnableWindow(true);
      }
   }
   else
   {
      SDiaryDlg::Instance()->m_TodoList.SetWindowText("");
      SDiaryDlg::Instance()->m_TodoList.EnableWindow(true);
   }
}