//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         diaryDlg.cpp
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  Implementation of Class diaryDlg.
//
//////////////////////////////////////////////////////////////////////////////////////////////

// system-wide includes
#include "stdafx.h"
#include <time.h>
#include <sys/timeb.h>
#include <atlimage.h>

// application-specific includes
#include "diary.h"
#include "diaryDlg.h"
#include "RecodeDlg.h"
#include "toolkit.h"
#include "sdate.h"
#include "monitors.h"
#include "multimonitor.h"
#include "monitordc.h"
#include "todolist.h"
#include "tools.h"
#include "afxwin.h"
#include "updatedialog.h"

#define MAX_TODO_LISTS   (12)

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define BOLDDAY(ds,iDay) if (iDay>0 && iDay<32)(ds)|=(0x00000001 << (iDay-1));

#define MAX_SHORTCUT  (16)

SDiaryDlg *SDiaryDlg::mInstance = 0;

// In revolt to the one class one file rule, the CAboutDlg is declared with 
// the SDiaryDlg, because it is a short class, that requires virtually nothing.
// also, it is private to the SDiaryDlg class.

class CAboutDlg : public CDialog
{
public:
   CAboutDlg();

   //{{AFX_DATA(CAboutDlg)
   enum { IDD = IDD_ABOUTBOX };
   //}}AFX_DATA

   // ClassWizard generated virtual function overrides
   //{{AFX_VIRTUAL(CAboutDlg)
protected:
   virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
   void OnPaint();
   void OnLButtonDown(UINT nFlags, CPoint point);

   //}}AFX_VIRTUAL

   //{{AFX_MSG(CAboutDlg)
   //}}AFX_MSG
   DECLARE_MESSAGE_MAP()

public:
   virtual BOOL OnInitDialog(void);
   afx_msg void OnBnClickedWhatsnew();
   afx_msg void OnBnClickedUpgrade();
   CButton mUpgrade;
   CButton mWhatsNew;
   CComboBox mAutoUpgrade;
   afx_msg void OnCbnSelchangeAutoUpgrade();
};

// the constructor of CAboutDlg...

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function CAboutDlg::CAboutDlg
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  Constructor
//
//  Return Value: None
//
//  Parameters:
//   None
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20081004	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
   TK_TRACER("CAboutDlg::CAboutDlg(void)");
   //{{AFX_DATA_INIT(CAboutDlg)
   //}}AFX_DATA_INIT
}

// useless, since no data is exchanged between the dialogs...

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function CAboutDlg::DoDataExchange
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  moves data to and from dialog to controls
//
//  Return Value: None (void)
//
//  Parameters:
//   CDataExchange* pDX                  [out] See MSDN documentation
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20081004	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	TK_TRACER("CAboutDlg::DoDataExchange(void)");
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
	DDX_Control(pDX, IDC_WHATSNEW, mWhatsNew);
	DDX_Control(pDX, ID_UPGRADE, mUpgrade);
	DDX_Control(pDX, IDC_AUTO_UPGRADE, mAutoUpgrade);
}

// we do however change the version line before showing the dialog...

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function CAboutDlg::OnInitDialog
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  Called before the About dialog is to be put on the screen
//
//  Return Value: TRUE if OK, FALSE otherwise (BOOL)
//
//  Parameters:
//   None
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20081004	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
BOOL CAboutDlg::OnInitDialog(void)
{
   TK_TRACER("CAboutDlg::OnInitDialog(void)");

   BOOL lResult = CDialog::OnInitDialog();

   TK_LOG0("Set the version numbers on the first line of the dialog");
   CString lString;
   std::string lCurrent   = ::GetCurrentVersionNumber();
   std::string lAvailable = ::GetAvailableVersionNumber();
   lAvailable[4] = '\0';
   int lCurrentNumeric = ::GetCurrentVersionNumberAsDecimal();
   int lAvailableNumeric = ::GetAvailableVersionNumberAsDecimal();
   if (lCurrentNumeric == lAvailableNumeric)
   {
       lString.Format("Diary %s   (most recent version)", lCurrent.c_str()); 
	   mUpgrade.EnableWindow(FALSE);
	   mWhatsNew.EnableWindow(FALSE);
   }
   else if (lCurrentNumeric > lAvailableNumeric)
   {
       lString.Format("Diary %s   (unpublished Beta version)", lCurrent.c_str()); 
	   mUpgrade.EnableWindow(FALSE);
	   mWhatsNew.EnableWindow(FALSE);
   }
   else
   {
       lString.Format("Diary %s   (%s available)", 
	                  lCurrent.c_str(),
		     		  lAvailable.c_str());
	   mUpgrade.EnableWindow(TRUE);
	   mWhatsNew.EnableWindow(TRUE);
   }

   this->GetDlgItem(IDC_DIARY_VERSION)->SetWindowText(lString.GetBuffer());

   // fill the auto-upgrade combobox...
   mAutoUpgrade.AddString("Ask");
   mAutoUpgrade.AddString("No");
   mAutoUpgrade.AddString("Yes");

   // and set it according to settings...
   SSetting *lSettingAnswer = SDiaryDlg::Instance()->GetSettings()->GetSetting(SET_UPGRADE_ANSWER);

   std::string lAnswer = "";
   if (lSettingAnswer != NULL)
   {
      TK_LOG0("Position settings found, positioning window");
	  lAnswer  = lSettingAnswer->GetValue();

	  // Ask is default if no setting stored....
	  mAutoUpgrade.SetCurSel(0);
	  if (lAnswer == "Yes")
	  {
		  mAutoUpgrade.SetCurSel(2);
	  }
	  else if (lAnswer == "No")
	  {
		  mAutoUpgrade.SetCurSel(1);
	  }
   }

   return lResult;
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
   //{{AFX_MSG_MAP(CAboutDlg)
      ON_WM_LBUTTONDOWN()
	  ON_WM_PAINT()
   //}}AFX_MSG_MAP
   ON_BN_CLICKED(IDC_WHATSNEW, &CAboutDlg::OnBnClickedWhatsnew)
   ON_BN_CLICKED(ID_UPGRADE, &CAboutDlg::OnBnClickedUpgrade)
   ON_CBN_SELCHANGE(IDC_AUTO_UPGRADE, &CAboutDlg::OnCbnSelchangeAutoUpgrade)
END_MESSAGE_MAP()

void CAboutDlg::OnPaint( )
{
	PAINTSTRUCT theStruct;
	CDC *theCDC = BeginPaint(&theStruct);
	theCDC->MoveTo(61,83);
	theCDC->LineTo(167,83);
	theCDC->MoveTo(108,224);
	theCDC->LineTo(268,224);
	theCDC->MoveTo(139,244);
	theCDC->LineTo(229,244);
	EndPaint(&theStruct);
}

void CAboutDlg::OnLButtonDown(UINT nFlags, CPoint point)
{
	/*
	CString theMsg;
	theMsg.Format("%d, %d", point.x, point.y);
	MessageBox(theMsg);
	return;
    */

    if (point.y >= 72 && point.y <= 83 &&
		point.x >= 61  && point.x <= 167)
	{
		ShellExecute(0, _T("open"), "http://www.codeproject.com", 
                     0, 0, SW_SHOWNORMAL);
	}

    if (point.y >= 213 && point.y <= 224 &&
		point.x >= 108  && point.x <= 268)
	{
		ShellExecute(0, _T("open"), "http://www.moorelife.nl/diary.html", 
                     0, 0, SW_SHOWNORMAL);
	}

    if (point.y >= 233 && point.y <= 244 &&
		point.x >= 139 && point.x <= 229)
	{
		ShellExecute(0, _T("open"), "mailto:andre@moorelife.nl", 
                     0, 0, SW_SHOWNORMAL);
	}
}

void CAboutDlg::OnBnClickedWhatsnew()
{
	CString theURL = "http://www.moorelife.nl/diarywhatsnew.txt";
	CString theContents;
	STools::ReadWebFile(theURL, theContents);
	std::string theCurrentVersion = ::GetCurrentVersionNumber();
	int theEnd = theContents.Find(theCurrentVersion.c_str()) - 1;
	if (theEnd <= 0)
	{
		// weird, user probably has a version that was never 
		// officially published. Try to find the last older version 
		// that was, and publish what's new from there.....

		int theStart = 0;
		while (theContents.Find("[", theStart) >= 0)
		{
			std::string theVersion = theContents.Mid(theContents.Find("[", theStart) + 1, 4);
			int theNumber = MakeVersionDecimal(theVersion);
			if (GetCurrentVersionNumberAsDecimal() > theNumber)
			{
				theEnd = theContents.Find("[", theStart) - 1;
				break;
			}

			theStart = theContents.Find("[", theStart) + 5;
		}

		if (theEnd <= 0)
		{
	        MessageBox(theContents, "What's New", MB_OK);			
			return;
		}
	}

	theContents = theContents.Left(theEnd);
	MessageBox(theContents, "What's New", MB_OK);

}

void CAboutDlg::OnBnClickedUpgrade()
{
    TK_LOG0("But save diary page and todolist before doing so");
	SDiaryDlg *theDiary = dynamic_cast<SDiaryDlg *>(AfxGetApp()->GetMainWnd());
    theDiary->mCurrentTodoList->Save();
    theDiary->SaveOldPage(true);

	// retrieve the most recent diaryinstaller from the website...
	CString theURL = "http://www.moorelife.nl/DiaryInstaller.exe";
	CString theFileSpec = "DiaryInstaller.exe";
	STools::ReadBinaryWebFile(theURL, 
		                      theFileSpec, 
							  0, 
							  0);

	// now start it...
	ShellExecute(0, _T("open"), "DiaryInstaller.exe", 
                 0, 0, SW_SHOWNORMAL);

	// then quit the app...
	theDiary->PostMessage(WM_QUIT);
}

SDiaryDlg *SDiaryDlg::Instance()
{
	if (mInstance == 0)
	{
		mInstance = new SDiaryDlg();
	}

	ASSERT(mInstance != 0);

	return mInstance;
}

void SDiaryDlg::DeleteInstance()
{
	if (mInstance != 0)
	{
		delete mInstance;
		mInstance = 0;
	}
}


// SDiaryDlg constructor basically loads the icon for the app, and initialises
// certain instance variables for the dialog. 

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function SDiaryDlg::SDiaryDlg
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  Constructor
//
//  Return Value: None
//
//  Parameters:
//   CWnd *pParent  [in]   the parent window of the SDiaryDlg. 
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20081004	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
SDiaryDlg::SDiaryDlg(CWnd* pParent)
          :CResizeDlg(SDiaryDlg::IDD, pParent),
		   mSearchDlg(0),
           mImageDlg(0),
           mImportDlg(0),
           mExportDlg(0),
           mFilesDlg(0),
           mShortcutDlg(0),
		   mCurrentTodoList(0),
		   mMouseClicked(false)
{
   mConfigRead = false;

   TK_TRACER("SDiaryDlg::SDiaryDlg(CWnd* pParent)");
   TK_PRMLHEX("pParent", pParent);
   //{{AFX_DATA_INIT(SDiaryDlg)
   //}}AFX_DATA_INIT

   TK_LOG0("Note that LoadIcon doesn't require subsequent DestroyIcon in Win32");
   m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

   TK_LOG0("Reset the page & list dirty flags");
   mPageDirty = false;
   mListDirty = false;

   TK_LOG0("Enable showing of the image dialog");
   mShowImages = true;
   mDiaryPage = NULL;
}

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function SDiaryDlg::~SDiaryDlg
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  Destructor
//
//  Return Value: None
//
//  Parameters:
//   None
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20081004	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
SDiaryDlg::~SDiaryDlg(void)
{
   TK_TRACER("SDiaryDlg::~SDiaryDlg()");
   if (this->mDiaryPage != NULL)
   {
      TK_LOG0("Delete the SDiaryPage object, because we're dying ourselves");
      delete this->mDiaryPage;
   }

   if (mSearchDlg != NULL)
   {
      delete mSearchDlg;
	  mSearchDlg = NULL;
   }   

   if (mImportDlg != NULL)
   {
      delete mImportDlg;
	  mImportDlg = NULL;
   }   

   if (mExportDlg != NULL)
   {
      delete mExportDlg;
	  mExportDlg = NULL;
   }   

   if (mImageDlg != NULL)
   {
      delete mImageDlg;
	  mImageDlg = NULL;
   }   

   if (mFilesDlg != NULL)
   {
      delete mFilesDlg;
	  mFilesDlg = NULL;
   }   

   if (mShortcutDlg != NULL)
   {
      delete mShortcutDlg;
	  mShortcutDlg = NULL;
   }   
}

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function SDiaryDlg::DoDataExchange
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  This is actually used to get the controls to exchange data...
//
//  Return Value: None (void)
//
//  Parameters:
//   CDataExchange* pDX                  [out] See MSDN documentation
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20081004	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
void SDiaryDlg::DoDataExchange(CDataExchange* pDX)
{
	TK_TRACER("SDiaryDlg::DoDataExchange(CDataExchange* pDX)");
	TK_PRMLHEX("pDX", pDX);
	CResizeDlg::DoDataExchange(pDX);

	//{{AFX_DATA_MAP(SDiaryDlg)
	DDX_Control(pDX, IDC_ENCRYPTION_KEY, m_EncryptionKey);
	DDX_Control(pDX, IDC_RICHEDIT1, m_DiaryPage);
	DDX_Control(pDX, IDC_RICHEDIT2, m_TodoList);
	DDX_Control(pDX, IDC_MONTHCALENDAR1, m_pMonthCalendar);
	//}}AFX_DATA_MAP
	DDX_Control(pDX, ID_WEEKDAY, mWeekday);
	DDX_Control(pDX, IDC_COMBO_SHORTCUTS, mComboShortcuts);
	DDX_Control(pDX, IDC_COMBO_TODO, mComboTodolists);
}

BEGIN_MESSAGE_MAP(SDiaryDlg, CResizeDlg)
   //{{AFX_MSG_MAP(SDiaryDlg)
   ON_WM_SYSCOMMAND()
   ON_WM_WINDOWPOSCHANGED()
   ON_WM_PAINT()
   ON_WM_LBUTTONDOWN()
   ON_WM_QUERYDRAGICON()
   ON_NOTIFY(MCN_GETDAYSTATE, IDC_MONTHCALENDAR1, OnGetdaystateMonthcalendar1)
   ON_NOTIFY(MCN_SELCHANGE, IDC_MONTHCALENDAR1, OnSelchangeMonthcalendar1)
   ON_EN_CHANGE(IDC_RICHEDIT1, OnChangeRichedit1)
   ON_EN_CHANGE(IDC_RICHEDIT2, OnChangeRichedit2)
   ON_BN_CLICKED(IDC_INSERT_TIME, OnInsertTime)
   ON_BN_CLICKED(IDC_SEARCH, OnSearch)
   ON_BN_CLICKED(IDC_IMAGE, OnBnClickedImage)
   ON_NOTIFY(EN_SELCHANGE, IDC_RICHEDIT1, OnEnSelchangeRichedit1)
   ON_NOTIFY(EN_SELCHANGE, IDC_RICHEDIT2, OnEnSelchangeRichedit2)
   //}}AFX_MSG_MAP
   ON_BN_CLICKED(IDC_PREV, OnBnClickedPrev)
   ON_BN_CLICKED(IDC_NEXT, OnBnClickedNext)
   ON_BN_CLICKED(IDC_FILES, OnBnClickedFiles)
   ON_BN_CLICKED(IDC_LBL_CONFIGURE, OnBnClickedLblConfigure)
   ON_BN_CLICKED(IDC_PRINT, &SDiaryDlg::OnBnClickedPrint)
   ON_EN_CHANGE(IDC_NUMDAYS, &SDiaryDlg::OnEnChangeNumdays)
   ON_CBN_SELCHANGE(IDC_COMBO_SHORTCUTS, &SDiaryDlg::OnCbnSelchangeComboShortcuts)
   ON_CBN_SELCHANGE(IDC_COMBO_TODO, &SDiaryDlg::OnCbnSelchangeComboTodo)
END_MESSAGE_MAP()

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function SDiaryDlg::OnInitDialog
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  SDiaryDlg::OnInitDialog adds the about box to the system menu, and 
//                further initialises the CRichEditCtrls. It also determines the first 
//                date to show, and loads the diary page and the todo list. And it determines
//                how the controls move along with the sizing of the dialog. 
//
//  Return Value: TRUE if OK, FALSE otherwise (BOOL)
//
//  Parameters:
//   None
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20081004	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
BOOL SDiaryDlg::OnInitDialog()
{
   TK_TRACER("SDiaryDlg::OnInitDialog(void)");

   CWaitCursor lWaiter;

   CResizeDlg::OnInitDialog();

   AddControl(IDC_RICHEDIT1, CST_NONE, CST_RESIZE, CST_NONE, CST_RESIZE, 1);
   AddControl(IDC_RICHEDIT2, CST_NONE, CST_NONE, CST_NONE, CST_RESIZE, 1);
   AddControl(IDC_NEXT, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);

   SetWindowText(getWindowTitle().GetBuffer());

   TK_LOG0("Initialize the SSettings object thats part of the SDiaryDlg");
   this->GetSettings()->SetFileName("settings.dat");
   this->GetSettings()->Dump();

   TK_LOG0("Read the position settings and restore them");
   SSetting *lSettingTop    = this->GetSettings()->GetSetting(SET_TOP);
   SSetting *lSettingLeft   = this->GetSettings()->GetSetting(SET_LEFT);
   SSetting *lSettingWidth  = this->GetSettings()->GetSetting(SET_WIDTH);
   SSetting *lSettingHeight = this->GetSettings()->GetSetting(SET_HEIGHT);

   if (lSettingTop    != NULL &&
       lSettingLeft   != NULL &&
	   lSettingWidth  != NULL &&
	   lSettingHeight != NULL)
   {
      TK_LOG0("Position settings found, positioning window");
      int lTop  = lSettingTop->GetValueAsNumber();
      int lLeft = lSettingLeft->GetValueAsNumber();
	  int lWidth = lSettingWidth->GetValueAsNumber();
	  int lHeight = lSettingHeight->GetValueAsNumber();

	  // make sure it will be on-screen, but keep the original position in config
	  this->SetWindowPos(0, lLeft, lTop, lWidth, lHeight, 0);
	  if (CMonitors::IsOnScreen( this ) == false)
	  {
		  CMonitor thePrimary = CMonitors::GetPrimaryMonitor();
		  thePrimary.CenterWindowToMonitor(this, true);
		  mWindowCentered = true;
          WINDOWPLACEMENT lWinPlace;
          this->GetWindowPlacement(&lWinPlace);
          mTop = lWinPlace.rcNormalPosition.top;
          mLeft = lWinPlace.rcNormalPosition.left;
	  }
   }

   mConfigRead = true;

   TK_LOG0("Lock the other controls, so we can't get to work");
   this->EnableAllControls(FALSE);
   
   TK_LOG0("Add other menu items to system menu");
   TK_LOG0("IDM_values must be in the system command range");
   ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
   ASSERT(IDM_ABOUTBOX < 0xF000);
   ASSERT((IDM_RECODE & 0xFFF0) == IDM_RECODE);
   ASSERT(IDM_RECODE < 0xF000);
   ASSERT((IDM_8POINTS & 0xFFF0) == IDM_8POINTS);
   ASSERT(IDM_8POINTS < 0xF000);
   ASSERT((IDM_10POINTS & 0xFFF0) == IDM_10POINTS);
   ASSERT(IDM_10POINTS < 0xF000);
   ASSERT((IDM_12POINTS & 0xFFF0) == IDM_12POINTS);
   ASSERT(IDM_12POINTS < 0xF000);
   ASSERT((IDM_14POINTS & 0xFFF0) == IDM_14POINTS);
   ASSERT(IDM_14POINTS < 0xF000);
   ASSERT((IDM_16POINTS & 0xFFF0) == IDM_16POINTS);
   ASSERT(IDM_16POINTS < 0xF000);

   TK_LOG0("Building the System Menu");
   CMenu* pSysMenu = GetSystemMenu(FALSE);
   if (pSysMenu != NULL)
   {
      TK_LOG0("First the Recode Data Files entry");
      CString strRecodeMenu;
      strRecodeMenu.LoadString(IDS_RECODE);
      if (!strRecodeMenu.IsEmpty())
      { 
          pSysMenu->AppendMenu(MF_SEPARATOR);
          pSysMenu->AppendMenu(MF_STRING, IDM_RECODE, strRecodeMenu);
      }   

	  TK_LOG0("Add Import & Export...");
      CString importMenu;
      importMenu.LoadString(IDS_IMPORT);
      CString exportMenu;
      exportMenu.LoadString(IDS_EXPORT);
      pSysMenu->AppendMenu(MF_SEPARATOR);
      pSysMenu->AppendMenu(MF_STRING, IDM_IMPORT, importMenu);
      pSysMenu->AppendMenu(MF_STRING, IDM_EXPORT, exportMenu);

      TK_LOG0("Add the font size entries to the menu");
      CString str8PointsMenu;
      CString str10PointsMenu;
      CString str12PointsMenu;
      CString str14PointsMenu;
      CString str18PointsMenu;

      str8PointsMenu.LoadString(IDS_8POINTS);
      str10PointsMenu.LoadString(IDS_10POINTS);
      str12PointsMenu.LoadString(IDS_12POINTS);
      str14PointsMenu.LoadString(IDS_14POINTS);
      str18PointsMenu.LoadString(IDS_16POINTS);

      if (!str8PointsMenu.IsEmpty())
      { 
          pSysMenu->AppendMenu(MF_SEPARATOR);
          pSysMenu->AppendMenu(MF_STRING, IDM_8POINTS, str8PointsMenu);
          if (!str10PointsMenu.IsEmpty())
          {
             pSysMenu->AppendMenu(MF_STRING, IDM_10POINTS, str10PointsMenu);
          }
          if (!str12PointsMenu.IsEmpty())
          {
             pSysMenu->AppendMenu(MF_STRING, IDM_12POINTS, str12PointsMenu);
          }
          if (!str14PointsMenu.IsEmpty())
          {
             pSysMenu->AppendMenu(MF_STRING, IDM_14POINTS, str14PointsMenu);
          }
          if (!str18PointsMenu.IsEmpty())
          {
             pSysMenu->AppendMenu(MF_STRING, IDM_16POINTS, str18PointsMenu);
          }
      }   

      TK_LOG0("Add the about entry to the menu");
      CString strAboutMenu;
      strAboutMenu.LoadString(IDS_ABOUTBOX);
      if (!strAboutMenu.IsEmpty())
      { 
          pSysMenu->AppendMenu(MF_SEPARATOR);
          pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
      }   
   }

   // set the number of days to print to one...
   SSetting *lSettingNumDays   = this->GetSettings()->GetSetting(SET_NUMDAYS);
   if (lSettingNumDays != NULL)
   {
       this->GetDlgItem(IDC_NUMDAYS)->SetWindowTextA(lSettingNumDays->GetValue());
   }
   else
   {
       this->GetDlgItem(IDC_NUMDAYS)->SetWindowText("1");
   }

   this->CheckDlgButton(IDC_TODO0, TRUE);
   mSelectedTodoList = 0;
   mOldSelectedTodoList = 0;

   // Set the icon for this dialog.  The framework does this automatically
   // when the application's main window is not a dialog
   SetIcon(m_hIcon, TRUE);       // Set big icon
   SetIcon(m_hIcon, FALSE);      // Set small icon
   
   TK_LOG0("Instantiate the subdialogs");
   mSearchDlg   = new SSearchDlg(this);      
   mImageDlg    = new SImageDlg(this);
   mImportDlg   = new SImportDlg(this);
   mExportDlg   = new SExportDlg(this);
   mFilesDlg    = new SFilesDlg(this);
   mShortcutDlg = new SLabelDlg(this);
   mShortcutDlg->SetDiaryDlg();

   mCurrentTodoList = mShortcutDlg->GetDefaultTodoList();

   TK_LOG0("Retrieve the current date");
   SYSTEMTIME lTime;
   m_pMonthCalendar.GetCurSel(&lTime);

   TK_LOG0("get the passed in date, if applicable");
   GetTimeFromPassedInFile(&lTime);
   m_pMonthCalendar.SetCurSel(&lTime);

   TK_LOG0("Retrieve the current fontsize and set it");
   SSetting *lFontSizeSSetting = this->mSettings.GetSetting(SET_FONT_SIZE);
   if (lFontSizeSSetting == NULL)
   {
      lFontSizeSSetting = new SSetting(SET_FONT_SIZE, "12");
      this->mSettings.AddSetting(lFontSizeSSetting);
   }
   this->SetFont(lFontSizeSSetting->GetValueAsNumber());

   TK_LOG0("set the new date, and load diary page and todo list");
   this->mNewSelDate.Set(lTime.wYear, lTime.wMonth, lTime.wDay);
   if (this->mCurrentTodoList)
   {
       this->mCurrentTodoList->Load();
   }

   this->LoadNewPage();

   TK_LOG0("Lock the Rich Edit Controls until password has been set");
   this->m_DiaryPage.EnableWindow(false);
   this->m_TodoList.EnableWindow(false);

   CString lString;
   lString.Format("%s", ::GetCurrentVersionNumber().c_str());
   this->GetDlgItem(IDC_VERSIONNUMBER)->SetWindowText(lString.GetBuffer());

   // to get the rich edit controls to send EN_CHANGE events
   // and EN_SELCHANGE events, we need
   // to tell them we want these notifications

   TK_LOG0("Set the RichEditCtrls to send ENM_CHANGE & ENM_SELCHANGE messages");
   long lEventMask = m_DiaryPage.GetEventMask();
   lEventMask |= ENM_CHANGE;
   lEventMask |= ENM_SELCHANGE;
   m_DiaryPage.SetEventMask(lEventMask);

   lEventMask = m_TodoList.GetEventMask();
   lEventMask |= ENM_CHANGE;
   lEventMask |= ENM_SELCHANGE;
   m_TodoList.SetEventMask(lEventMask);

   this->UpdateStatus();
   this->UpdateWeekday();

   int lCurrentNumeric = ::GetCurrentVersionNumberAsDecimal();
   int lAvailableNumeric = ::GetAvailableVersionNumberAsDecimal();
   if (lCurrentNumeric < lAvailableNumeric)
   {
	   CString theMsg;
	   theMsg.Format("Version %d.%02d is available,\ndo you want to upgrade?", 
		             lAvailableNumeric / 100, lAvailableNumeric % 100);
	   CUpdateDialog lDlg;
	   if (IDOK == lDlg.DoModal())
	   {
			// retrieve the most recent diaryinstaller from the website...
			CString theURL = "http://www.moorelife.nl/DiaryInstaller.exe";
			CString theFileSpec = "DiaryInstaller.exe";
			STools::ReadBinaryWebFile(theURL, 
									  theFileSpec, 
									  0, 
									  0);

			// now start it...
			ShellExecute(0, _T("open"), "DiaryInstaller.exe", 
						 0, 0, SW_SHOWNORMAL);

			// then quit the app...
			this->PostMessage(WM_QUIT);
	   }
   }

   return TRUE;  // return TRUE  unless you set the focus to a control
} 

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function SDiaryDlg::SetFont
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  this method sets the fontsize to a given number between 8 and 18.
//                in fact it can be set to any number between 8 and 18, but the menu
//                restricts it to values that really differ...
//
//  Return Value: None (void)
//
//  Parameters:
//   int pFontSize                       [in]  the requested font size
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20081004	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
void SDiaryDlg::SetFont(int pFontSize)
{
   TK_TRACER("SDiaryDlg::SetFont(int pFontSize)");
   CWaitCursor lWaiter;

   TK_LOG0("Limit fontsize to 8..18");
   if (pFontSize < 8)
   {
      pFontSize = 8;
   }
   if (pFontSize > 18)
   {
      pFontSize = 18;
   }

   TK_LOG0("Store fontsize setting in settings.dat");
   char *lValue = new char[MAX_STRING];
   _itoa_s(pFontSize, lValue, MAX_STRING - 1, 10);
   SSetting *lNewSSettingObject = new SSetting(SET_FONT_SIZE, lValue);
   this->mSettings.AddSetting(lNewSSettingObject);
   this->mSettings.WriteFile();

   TK_LOG0("Set fontsize on both RichEditCtrls");
   CHARFORMAT lDefaultCharFormat;

   lDefaultCharFormat.cbSize = sizeof(CHARFORMAT); 
   lDefaultCharFormat.dwMask = CFM_CHARSET | CFM_SIZE; 
   lDefaultCharFormat.dwEffects = 0; 
   lDefaultCharFormat.yHeight = pFontSize * 20; 
   lDefaultCharFormat.yOffset = 0; 
   lDefaultCharFormat.crTextColor = 0x00000000; 
   lDefaultCharFormat.bCharSet = ANSI_CHARSET; 
   lDefaultCharFormat.bPitchAndFamily = DEFAULT_PITCH | FF_DONTCARE; 
   strcpy_s(lDefaultCharFormat.szFaceName, 10, "Arial"); 

   TK_LOG0("But save diary page and todolist before doing so");
   this->mMouseClicked = false;
   this->mCurrentTodoList->Save();
   this->SaveOldPage(true);
   
   m_DiaryPage.SetDefaultCharFormat(lDefaultCharFormat);
   m_TodoList.SetDefaultCharFormat(lDefaultCharFormat);

   TK_LOG0("And of course restore them afterwards");
   if (this->mCurrentTodoList)
   {
       this->mCurrentTodoList->Load();
   }
   this->LoadNewPage();
   
   delete [] lValue;
}

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function SDiaryDlg::OnSysCommand
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  This method gets called when the user selects a menu entry from the 
//                system menu. If it isn't one of the special ones, we pass it on to
//                the CResizeDlg object.  
//
//  Return Value: None (void)
//
//  Parameters:
//   UINT nID                            [in]  the system command
//   LPARAM lParam                       [in]  and the parameter
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20081004	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
void SDiaryDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
   TK_TRACER("SDiaryDlg::OnSysCommand(UINT nID, LPARAM lParam)");
   CWaitCursor lWaiter;

   if ((nID & 0xFFF0) == IDM_ABOUTBOX)
   {
      TK_LOG0("About Diary selected, show About box");
      CAboutDlg dlgAbout;
      dlgAbout.DoModal();
   }
   else if ((nID & 0xFFF0) == IDM_8POINTS)
   {
      TK_LOG0("Font Size 8 selected, set it");
      this->SetFont(8);
   }
   else if ((nID & 0xFFF0) == IDM_10POINTS)
   {
      TK_LOG0("Font Size 10 selected, set it");
      this->SetFont(10);
   }
   else if ((nID & 0xFFF0) == IDM_12POINTS)
   {
      TK_LOG0("Font Size 12 selected, set it");
      this->SetFont(12);
   }
   else if ((nID & 0xFFF0) == IDM_14POINTS)
   {
      TK_LOG0("Font Size 14 selected, set it");
      this->SetFont(14);
   }
   else if ((nID & 0xFFF0) == IDM_16POINTS)
   {
      TK_LOG0("Font Size 18 selected, set it");
      this->SetFont(18);
   }
   else if ((nID & 0xFFF0) == IDM_IMPORT)
   {
      TK_LOG0("Import selected, call it");
      this->OnBnClickedImport();
   }
   else if ((nID & 0xFFF0) == IDM_EXPORT)
   {
      TK_LOG0("Export selected, call it");
      this->OnExport();	
   }
   else if ((nID & 0xFFF0) == IDM_RECODE)
   {
      TK_LOG0("Recode data files selected, execute dialog");
      this->mMouseClicked = false;
      this->mCurrentTodoList->Save();
      this->SaveOldPage(true);

      SRecodeDlg dlgRecode;
      dlgRecode.DoModal();

      this->mCurrentTodoList->Load();
      this->LoadNewPage();
   }
   else
   {
      TK_LOG0("Other syscommand selected, relay to CResizeDlg");
      CResizeDlg::OnSysCommand(nID, lParam);
   }
}

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function SDiaryDlg::OnPaint
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  If you add a minimize button to your dialog, you will need the code below
//                to draw the icon.  For MFC applications using the document/view model,
//                this is automatically done for you by the framework.
//
//  Return Value: None (void)
//
//  Parameters:
//   None
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20081004	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
void SDiaryDlg::OnPaint() 
{
   TK_TRACER("SDiaryDlg::OnPaint()");
   CWaitCursor lWaiter;

   if (IsIconic())
   {
      TK_LOG0("Repaint Icon");
      CPaintDC dc(this); // device context for painting

      SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

      TK_LOG0("Center icon in client rectangle");
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      TK_LOG0("Draw the icon");
      dc.DrawIcon(x, y, m_hIcon);
   }
   else
   {
      TK_LOG0("Repaint Dialog, through CResizeDlg::OnPaint");
      CResizeDlg::OnPaint();
   }
}

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function SDiaryDlg::OnQueryDragIcon
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  The system calls this to obtain the cursor to display while the user drags
//                the minimized window.
//
//  Return Value: the cursor to use (HCURSOR)
//
//  Parameters:
//   None
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20081004	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
HCURSOR SDiaryDlg::OnQueryDragIcon()
{
   TK_TRACER("SDiaryDlg::OnQueryDragIcon()");
   return (HCURSOR) m_hIcon;
}

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function SDiaryDlg::OnGetdaystateMonthcalendar1
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  this method handles the daystate messages that the CCalendarCtrl sends to 
//                the application. it basically boldens those dates for which a data file 
//                exists. 
//
//  Return Value: None (void)
//
//  Parameters:
//   NMHDR* pNMHDR                       [out] See MSDN documentation
//   LRESULT* pResult                    [out] See MSDN documentation
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20081004	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
void SDiaryDlg::OnGetdaystateMonthcalendar1(NMHDR* pNMHDR, LRESULT* pResult) 
{
   TK_TRACER("SDiaryDlg::OnGetdaystateMonthcalendar1(NMHDR* pNMHDR, LRESULT* pResult)");
   CWaitCursor lWaiter;

   NMDAYSTATE* pDayState = (NMDAYSTATE *)pNMHDR;

   MONTHDAYSTATE mdState[3];
   INT i, iMax;

   iMax = pDayState->cDayState;

   for (i = 0; i < iMax; i++)
   {
      mdState[i] = GetDayStatesForMonth(this->mNewSelDate.getYear(), this->mNewSelDate.getMonth(), i);
   } 

   pDayState->prgDayState = mdState;
}

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function SDiaryDlg::OnSelchangeMonthcalendar1
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  this method gets called if the user selects a different date from the 
//                calendar control. These messages sometimes get sent even if the date 
//                didn't change, so be careful to skip those messages. 
//
//  Return Value: None (void)
//
//  Parameters:
//   NMHDR* pNMHDR                       [out] See MSDN documentation
//   LRESULT* pResult                    [out] See MSDN documentation
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20081004	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
void SDiaryDlg::OnSelchangeMonthcalendar1(NMHDR* pNMHDR, LRESULT* pResult) 
{
   TK_TRACER("OnSelchangeMonthcalendar1(NMHDR* pNMHDR, LRESULT* pResult)");
   SYSTEMTIME lTime;

   CWaitCursor lWaiter;

   TK_LOG0("Is this message coming from the right Hwnd?");
   if (m_pMonthCalendar.m_hWnd == pNMHDR->hwndFrom)
   {
      TK_LOG0("Get the newly selected date");
      if (m_pMonthCalendar.GetCurSel(&lTime))
      {
         TK_LOG0("And calculate the new date in format CCYYMMDD");
         SDate lNewDate(lTime.wYear, lTime.wMonth, lTime.wDay);
		 SetNewDate(lNewDate);
      }
   }

   TK_LOG0("Pages updated, recalc Image count again...");
   this->UpdateStatus();
   this->UpdateWeekday();

   *pResult = 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function SDiaryDlg::SaveOldPage
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  this method saves the contents of the diary page edit control, based 
//                on the value of pFinal. If set, then mNewSelectionDate is used, otherwise
//                mOldSelectionData is used. Saving includes encryption, if an encryption 
//                keyword has been set. 
//
//  Return Value: true if OK, false otherwise (bool)
//
//  Parameters:
//   bool pFinal                         [in]  set to true if saving without new selection. 
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20081004	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
bool SDiaryDlg::SaveOldPage(bool pFinal)
{
   TK_TRACER("SDiaryDlg::SaveOldPage(bool pFinal)");
   if (mPageDirty)
   {
      TK_LOG0("something to save, so lets do it");
      CString lFileName;
      lFileName.Format("%08d.dry", this->mOldSelDate.asDecimal());
      if (pFinal)
      {
         lFileName.Format("%08d.dry", this->mNewSelDate.asDecimal());
      }

      TK_LOG0("Open the file for writing");
      FILE *lFile = fopen(lFileName.GetBuffer(0), "wb");

      if (lFile == NULL)
      {
         CString lMsg;
         lMsg.Format("File %s could not be opened for writing.\ncontinue intended action?", lFileName);
         if (AfxMessageBox(lMsg, MB_YESNO) == IDNO)
		 {
			 return false;
		 }
		 else
		 {
			 return true;
		 }
      }

      TK_LOG0("file now open, proceed");
      long lMin = 0;
      long lMax = 0;
      char lBuffer[MAX_BUFFER + 1];
      m_DiaryPage.SetSel(0, -1);
      m_DiaryPage.GetSel(lMin, lMax);

      if (lMin == 0 && lMax == 2)
      { 
         // nothing to save, delete datafile
         // for some funny reason, this seems to not work. 
         // _unlink keeps returning -1.....
         fclose(lFile);
         lFile = NULL;

         CString lFilename;
         DWORD lDirReturn = 0;
         char  *lPath = new char[MAX_STRING];

         TK_LOG0("Retrieve the path of execution");
         lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);

         lFilename.Format("%s\\%08d.dry", lPath, this->mOldSelDate.asDecimal());

         TK_LOG1("Attempting to delete %s", lFilename.GetBuffer());
         int lRet = -1;
         long lCount = 0;
         while ((lRet = _unlink(lFilename.GetBuffer(0))) == -1 && lCount < 10)
         {
            lCount++;
         }

#ifdef TIME_TO_INVESTIGATE_THE_UNLINK_PROBLEM
         TK_LOG1("Return code from _unlink = %d", lRet);
         if (lRet == -1)
         {
            switch (errno)
            {
            case EACCES:
               MessageBox("Read-only file, cannot delete");
               break;
            case ENOENT:
               MessageBox("Path specifies a directory, or file not found");
               break;
            }
         }
#endif

         m_DiaryPage.SetWindowText("");
		 delete [] lPath;
         return true;
      }

      TK_LOG0("Encode the page, and save it to the file");
      CString theStr = m_DiaryPage.GetSelText();
	  strcpy_s(lBuffer, MAX_BUFFER, theStr);
      m_DiaryPage.SetSel(-1, -1);
      theApp.Encode(lBuffer, lMax - lMin);
      fwrite(lBuffer, 1, lMax - lMin, lFile);

      fclose(lFile);

      m_DiaryPage.SetWindowText("");
   }
   return true;
}

// this method loads a new page into the diary page edit control. 
// mNewSelectionDate is used to determine the correct data file. 
// if an encryption keyword has been entered, decryption will occur. 

void SDiaryDlg::LoadNewPage(void)
{
   TK_TRACER("SDiaryDlg::LoadNewPage(void)");
   CWaitCursor lWaiter;

   m_DiaryPage.SetWindowText("");
   this->mKillSelChangeProcessing = true;

   TK_LOG0("Build the correct filename, and open the file");
   CString lFileName;
   lFileName.Format("%08d.dry", this->mNewSelDate.asDecimal());

   FILE *lFile = fopen(lFileName, "rb");

   if (lFile != NULL)
   {
      TK_LOG0("File is open, read it and decode");
      char lBuffer[MAX_BUFFER + 1];

      int lBytesRead = fread(lBuffer, 1, MAX_BUFFER, lFile);
      if (lBytesRead > 0)
      {
         lBuffer[lBytesRead] = '\0';
         theApp.Decode(lBuffer, lBytesRead);
         if (theApp.IsProperlyDecrypted(lBuffer, lBytesRead))
         {
            m_DiaryPage.SetWindowText(lBuffer);
            PerformHighlighting();
            m_DiaryPage.EnableWindow(true);
         }
         else
         {
            m_DiaryPage.SetWindowText("");
            m_DiaryPage.EnableWindow(false);
         }
      }
      else
      {
         m_DiaryPage.SetWindowText("");
         m_DiaryPage.EnableWindow(true);
      }
   }
   else
   {
      m_DiaryPage.SetWindowText("");
      m_DiaryPage.EnableWindow(true);
   }

   mPageDirty = false;
   this->mKillSelChangeProcessing = false;

   this->UpdateWeekday();
}

// This method gets called if the user presses enter while the 
// encryption keyword edit control is focused. It basically saves
// any unsaved stuff, sets the keyword, and loads the relevant 
// pages again. 

void SDiaryDlg::OnOK() 
{
   TK_TRACER("SDiaryDlg::OnOK()");
   CWaitCursor lWaiter;

   TK_LOG0("Unlock the Rich Edit Controls because password has been set");
   this->m_DiaryPage.EnableWindow(true);
   this->m_TodoList.EnableWindow(true);
   
   TK_LOG0("Unlock the other controls, so we can get to work");
   this->EnableAllControls(TRUE);

   TK_LOG0("Save todolist and diary page");
   if (this->mPageDirty)
   {
       this->mMouseClicked = false;
       this->mCurrentTodoList->Save();
       this->SaveOldPage(true);
   }

   TK_LOG0("Set the new encryption keyword");
   char *lCryptKey = new char[MAX_STRING];
   m_EncryptionKey.GetWindowText(lCryptKey, MAX_STRING);

   // Sneaky: if it is a !, fill in my current keyword.....
   if (lCryptKey[0] == '!' && lCryptKey[1] == '\0')
   {
	   strcpy_s(lCryptKey, MAX_STRING, "Don't Collapse It");
   }

   theApp.SetCryptKey(lCryptKey);
   
   TK_LOG0("...and load the diary page and the todolist again");
   this->mCurrentTodoList->Load();
   this->LoadNewPage();

   TK_LOG0("Last of all, set the encryption field to empty");
   m_EncryptionKey.SetWindowText("");
   m_DiaryPage.SetFocus();

   TK_LOG0("This is the right point to encrypt any hitherto unencrypted images");
      
   CFileFind finder;
   BOOL bWorking = finder.FindFile("*.jpg");
   bool lAsked = false;
   while (bWorking)
   {   
      if (lAsked == false)
      {
         if (AfxMessageBox("Did you fill in your keyword correctly?", MB_YESNO) == IDNO)
         {
            break;
         }
         lAsked = true;
      }

      bWorking = finder.FindNextFile();

      CString lMsg;
      lMsg.Format("encrypting image file %s", finder.GetFileName().GetBuffer());
      mWeekday.SetWindowText(lMsg.GetBuffer());

      theApp.EncryptFile(finder.GetFilePath().GetBuffer());
      
   }
   mWeekday.SetWindowText("");
   this->UpdateWeekday();
   delete [] lCryptKey;
   
   m_DiaryPage.SetSel(-1, -1);
}

// this gets called if the user clicks the close button. It saves any 
// unsaved information, before exiting the application. 

void SDiaryDlg::OnCancel() 
{
   TK_TRACER("SDiaryDlg::OnCancel()");
   CWaitCursor lWaiter;

   this->mShowImages = false;

   TK_LOG0("Save the todolist and the diary page, and exit");
   this->mMouseClicked = false;
   bool todoSaved = this->mCurrentTodoList->Save();
   bool pageSaved = this->SaveOldPage(true);
   
   if (pageSaved && todoSaved)
   {
       CResizeDlg::OnCancel();
   }
}

// the next two methods make sure that the dirty flags for both 
// CRichEditCtrls are set properly, and the cursor stays visible
// while editing. 

void SDiaryDlg::OnChangeRichedit1() 
{
   TK_TRACER("SDiaryDlg::OnChangeRichedit1()");
   long lStartOfSelection = 0;
   long lStartOfPage = 0;
   
   TK_LOG0("Get position of cursor, and first visible line");
   lStartOfSelection = m_DiaryPage.LineFromChar(-1);
   lStartOfPage      = m_DiaryPage.GetFirstVisibleLine();

   // Now that there are different sized fonts possible in the diary 
   // page, we work out with the maximum number of lines for different font sizes.

   TK_LOG0("If needed, correct first visible line to keep cursor on screen");
   if (lStartOfSelection - lStartOfPage < 2 ||
       lStartOfSelection - lStartOfPage > this->GetNumberOfLinesForDiaryWindow() - 2)
   {
      m_DiaryPage.LineScroll(lStartOfSelection 
                           - lStartOfPage 
                           - this->GetNumberOfLinesForDiaryWindow() / 2);
   }
 
   this->mShowImages = true;
   PerformHighlighting();

   // set the modification flag
   mPageDirty = true;
}

void SDiaryDlg::OnChangeRichedit2() 
{
   TK_TRACER("SDiaryDlg::OnChangeRichedit2()");
   long lStartOfSelection = 0;
   long lStartOfPage = 0;
   
   TK_LOG0("Get position of cursor, and first visible line");
   lStartOfSelection = m_TodoList.LineFromChar(-1);
   lStartOfPage      = m_TodoList.GetFirstVisibleLine();
  
   // Now that there are different sized fonts possible in the todo
   // list, and the window itself is smaller, these limits are incorrect. 
   // Must work out something that is correct for any fontsize....

   TK_LOG0("If needed, correct first visible line to keep cursor on screen");
   if (lStartOfSelection - lStartOfPage < 0 ||
       lStartOfSelection - lStartOfPage > this->GetNumberOfLinesForTodoWindow() - 2)
   {
      m_TodoList.LineScroll(lStartOfSelection 
                          - lStartOfPage 
                          - this->GetNumberOfLinesForTodoWindow() / 2);
   }

   TK_LOG0("set the modification flag");   
   mListDirty = true;
}

// This method determines the daystates for a single month

MONTHDAYSTATE SDiaryDlg::GetDayStatesForMonth(int pYear,
											  int pMonth, 
                                              int pMonthOffset)
{
   TK_TRACER("SDiaryDlg::GetDayStatesForMonth(NMDAYSTATE *pDayState, int pMonthOffset)");

   CWaitCursor lWaiter;
   MONTHDAYSTATE lDayState = 0;
   int lMonth = pMonth - 1 + pMonthOffset;
   int lYear  = pYear;

   if (lMonth < 1)
   {
	   lMonth = 1;
   }

   if (lMonth > 12)
   {
      lMonth -= 12;
      lYear  += 1;
   }

   TK_LOG0("for all days in the month");
   for (int lDay = 1; lDay <= 31; lDay++)
   {
      TK_LOG0("Check for existence of a diary file");
      CString lFileName;
      lFileName.Format("%04d%02d%02d.dry", 
                       lYear, 
                       lMonth,
                       lDay);
      FILE *lFile = fopen(lFileName.GetBuffer(0), "r");
      if (lFile != NULL)
      {
         TK_LOG0("and set the corresponding date to bold");
         BOLDDAY(lDayState, lDay);
         fclose(lFile);
      }
   }

   return lDayState;
}

// This method inserts the current time in the diary page edit control

void SDiaryDlg::OnInsertTime() 
{
   TK_TRACER("SDiaryDlg::OnInsertTime()");
   CWaitCursor lWaiter;

   long lMin = 0;
   long lMax = 0;

   TK_LOG0("set the selection to a single point of insertion");
   m_DiaryPage.GetSel(lMin, lMax);
   m_DiaryPage.SetSel(lMin, lMin);

   time_t          lTime;
   struct tm       lToday;
   static char     lTimeStamp[21];

   TK_LOG0("Use time structure to build a customized time string.");
   time(&lTime);
   localtime_s( &lToday, &lTime );
   strftime(lTimeStamp, 20, "%H:%M  ", &lToday);

   TK_LOG0("check date to be on yesterday, if so advance day");
   SDate thePage = this->GetCurrentDate();
   SDate theTime = SDate(lToday.tm_year + 1900, lToday.tm_mon + 1, lToday.tm_mday);
   if (thePage != theTime)
   {
	   // if we did a timestamp, that implies today...
	   SetNewDate(theTime);
	   m_DiaryPage.SetSel(-1, -1);
   }

   TK_LOG0("insert the time string into the CRichEditCtrl of the diary page");
   m_DiaryPage.ReplaceSel(lTimeStamp);
   m_DiaryPage.SetFocus();
}

// This method starts the search dialog

void SDiaryDlg::OnSearch() 
{
   TK_TRACER("SDiaryDlg::OnSearch()");
   SYSTEMTIME lTime;
   int lCharPos = 0;

   CWaitCursor lWaiter;

   TK_LOG0("Save todolist and diary page");
   this->mMouseClicked = false;
   this->mCurrentTodoList->Save();
   this->SaveOldPage(true);

   TK_LOG0("Start the search dialog");
   ASSERT(mSearchDlg != NULL);
   if (mSearchDlg->DoModal() == IDOK)
   {
      TK_LOG0("If OK pressed, get the selected date and position the diary likewise");
      lTime = mSearchDlg->GetSelectedDate();
      m_pMonthCalendar.SetCurSel(&lTime);
      long lNewDate = lTime.wYear  * 10000 +
                      lTime.wMonth *   100 +
                      lTime.wDay;

      TK_LOG0("Save the position of the sought string");
      lCharPos = mSearchDlg->GetCharPos();

      this->mOldSelDate = this->mNewSelDate;
      this->mNewSelDate.Set(lNewDate);
   }

   TK_LOG0("Load the new page");
   this->LoadNewPage();

   if (lCharPos > 0)
   {
      TK_LOG0("Position the page five lines above the correct line");
      int lLine = m_DiaryPage.LineFromChar(lCharPos);
      m_DiaryPage.LineScroll(-100000);
      m_DiaryPage.LineScroll(lLine - 5);
   }
}

// Performs ALL highlighting required

void SDiaryDlg::PerformHighlighting()
{
    this->mKillSelChangeProcessing = true;

	this->PerformHighlighting(m_DiaryPage);
	this->PerformHighlighting(m_TodoList);

	this->ColorSearchWords(m_DiaryPage);

    this->mKillSelChangeProcessing = false;
}

void SDiaryDlg::PerformHighlighting(CRichEditCtrl &pControl)
{
	// make sure we don't re-enter while it's being executed...
	static bool stillRunning = false;
	if (stillRunning)
	{
		return;
	}
	stillRunning = true;

	//- huidige selectie onthouden
	CHARRANGE  lLeftAt;
    pControl.GetSel(lLeftAt);

	//- volledige tekst ophalen
    CString thePageBuffer;
    long theLastChar = pControl.GetTextLength();
    pControl.GetTextRange(0, theLastChar, thePageBuffer);

	//- woord voor woord erdoorheen itereren
	long theStart = 0;
	long theEnd = 0;

	while (theEnd < theLastChar)
	{
		// skip any whitespace...
		while (theEnd < theLastChar && STools::isWhiteSpace(thePageBuffer[theStart]))
		{
			theStart++;
			theEnd++;
		}

		// select anything upto the next whitespace
		while (theEnd < theLastChar && STools::isWhiteSpace(thePageBuffer[theEnd]) == false)
		{
			theEnd++;
		}

		if (theEnd >= theLastChar)
		{
			break;
		}

		while ((STools::isWhiteSpace(thePageBuffer[theEnd]) ||
			    thePageBuffer[theEnd] == '.' || 
			    thePageBuffer[theEnd] == ',') &&
				theEnd > theStart)
		{
			theEnd--;
		}

		// rip the word off the page...
		std::string theWord = thePageBuffer.Mid(theStart, theEnd - theStart + 1);

		//- alle highlighters aanroepen, tot er n true teruggeeft
		HighlightHyperJump(pControl, theStart, theEnd, theWord);
		HighlightURL(pControl, theStart, theEnd, theWord);
		HighlightMail(pControl, theStart, theEnd, theWord);
        HighlightImageSlot(pControl, theStart, theEnd, theWord);
 
		//- nog niet alles gehad, dan volgende woord kiezen
		theEnd++;
		theStart = theEnd;
	}

	//- originele selectie terugzetten
    pControl.SetSel(lLeftAt);

    stillRunning = false;
}

void SDiaryDlg::ColorSearchWords(CRichEditCtrl &pControl)
{
	// make sure we don't re-enter while it's being executed...
	static bool stillRunning2 = false;
	if (stillRunning2)
	{
		return;
	}
	stillRunning2 = true;

	//- huidige selectie onthouden
	CHARRANGE  lLeftAt;
    pControl.GetSel(lLeftAt);

	//- volledige tekst ophalen
    CString thePageBuffer;
    long theLastChar = pControl.GetTextLength();
    pControl.GetTextRange(0, theLastChar, thePageBuffer);
	thePageBuffer.MakeLower();

	//- woord voor woord erdoorheen itereren
	long theStart = 0;
	long theEnd = 0;

	CString theSearchString = mSearchDlg->GetSearchString();
	theSearchString.MakeLower();
	if (theSearchString.GetLength() == 0)
	{
        stillRunning2 = false;
		return;
	}

	while (theEnd < theLastChar)
	{
		theStart = thePageBuffer.Find(theSearchString, theEnd);

		// no more found, then end...
		if (theStart == -1)
		{
			break;
		}

		// otherwise, mark and call the highlighter....
		theEnd = theStart + theSearchString.GetLength();
        ColorSearchWord(pControl, theStart, theEnd);

		//- nog niet alles gehad, dan volgende woord kiezen
		theStart = theEnd;
	}

	//- originele selectie terugzetten
    pControl.SetSel(lLeftAt);

    stillRunning2 = false;
}

bool SDiaryDlg::IsHyperJump(std::string &theWord)
{
   if (theWord.size() != 8)
   {
	   return false;
   }

   if (!STools::isDigit(theWord[0]) ||
       !STools::isDigit(theWord[1]) ||
       !STools::isDigit(theWord[2]) ||
       !STools::isDigit(theWord[3]) ||
       !STools::isDigit(theWord[4]) ||
       !STools::isDigit(theWord[5]) ||
       !STools::isDigit(theWord[6]) ||
       !STools::isDigit(theWord[7]))
   {
	   return false;
   }

   int lYear  = atoi(theWord.substr(0, 4).c_str());
   int lMonth = atoi(theWord.substr(4, 2).c_str());
   int lDay   = atoi(theWord.substr(6, 2).c_str());
   if (SDate::isValidDate(lYear, lMonth, lDay))
   {
       return true;
   }

   return false;
}

bool  SDiaryDlg::IsURL(std::string &theWord)
{
   if (theWord.substr(0,6) == "ftp://"   ||
	   theWord.substr(0,7) == "http://"  ||
	   theWord.substr(0,8) == "https://" ||
	   theWord.substr(0,4) == "www.")
   {
       return true;
   }

   return false;
}

bool  SDiaryDlg::IsMailAddress(std::string &theWord)
{
	int theApetail = theWord.find("@");
	int theDot = theWord.rfind(".");

	if (theApetail != std::string::npos &&
		theDot != std::string::npos &&
		theApetail < theDot)
	{
        return true;
    }

    return false;
}

bool SDiaryDlg::IsImageSlot(std::string &theWord)
{
   if (theWord.substr(0,4) == "Slot" && theWord.size() == 5)
   {
	   int theSlotNumber = mImageDlg->SlotNumberFromChar(theWord[4]);

	   if (mImageDlg->DoesImageFileExistForCurrentDate(theSlotNumber))
       {
           return true;
	   }
   }

   return false; 
}

bool SDiaryDlg::IsSearchWords(std::string &theWord)
{
	return false;
}

void SDiaryDlg::HighlightHyperJump(CRichEditCtrl &theEdit, 
								   int theStart,
								   int theEnd, 
								   std::string &theWord)
{
   // Date hyperlinks are underlined...
   CHARFORMAT lCf;

   lCf.cbSize           = sizeof(CHARFORMAT); 
   lCf.dwMask           = CFM_UNDERLINE; 
   lCf.dwEffects        = CFE_UNDERLINE; 
   lCf.yHeight          = 0; 
   lCf.yOffset          = 0; 
   lCf.crTextColor      = 0;      
   lCf.bCharSet         = 0; 
   lCf.bPitchAndFamily  = 0; 
   lCf.szFaceName[0]    = '\0'; 

   if (IsHyperJump(theWord))
   {
       // if so, underline
       theEdit.SetSel(theStart, theEnd + 1);
       theEdit.SetSelectionCharFormat(lCf);
	   theEdit.SetSel(theEnd + 1, theEnd + 1);
   }
}

void SDiaryDlg::HighlightURL(CRichEditCtrl &theEdit, 
							 int theStart, 
							 int theEnd,
							 std::string &theWord)
{
   // URL hyperlinks are underlined...
   CHARFORMAT lCf;

   lCf.cbSize           = sizeof(CHARFORMAT); 
   lCf.dwMask           = CFM_UNDERLINE; 
   lCf.dwEffects        = CFE_UNDERLINE; 
   lCf.yHeight          = 0; 
   lCf.yOffset          = 0; 
   lCf.crTextColor      = 0;      
   lCf.bCharSet         = 0; 
   lCf.bPitchAndFamily  = 0; 
   lCf.szFaceName[0]    = '\0'; 

   if (IsURL(theWord))
   {
       // if so, underline
       theEdit.SetSel(theStart, theEnd + 1);
       theEdit.SetSelectionCharFormat(lCf);
	   theEdit.SetSel(theEnd + 1, theEnd + 1);
   }
}

void SDiaryDlg::HighlightMail(CRichEditCtrl &theEdit, 
							  int theStart, 
							  int theEnd,
							  std::string &theWord)
{
   // mail addresses are underlined...
   CHARFORMAT lCf;

   lCf.cbSize           = sizeof(CHARFORMAT); 
   lCf.dwMask           = CFM_UNDERLINE; 
   lCf.dwEffects        = CFE_UNDERLINE; 
   lCf.yHeight          = 0; 
   lCf.yOffset          = 0; 
   lCf.crTextColor      = 0;      
   lCf.bCharSet         = 0; 
   lCf.bPitchAndFamily  = 0; 
   lCf.szFaceName[0]    = '\0'; 

   if (IsMailAddress(theWord))
   {
       // if so, underline
       theEdit.SetSel(theStart, theEnd + 1);
       theEdit.SetSelectionCharFormat(lCf);
	   theEdit.SetSel(theEnd + 1, theEnd + 1);
   }
}

void SDiaryDlg::HighlightImageSlot(CRichEditCtrl &theEdit,
								   int theStart,
								   int theEnd,
								   std::string &theWord)
{
   // Image Slot hyperlinks are underlined...
   CHARFORMAT lCf;

   lCf.cbSize           = sizeof(CHARFORMAT); 
   lCf.dwMask           = CFM_UNDERLINE; 
   lCf.dwEffects        = CFE_UNDERLINE; 
   lCf.yHeight          = 0; 
   lCf.yOffset          = 0; 
   lCf.crTextColor      = 0;      
   lCf.bCharSet         = 0; 
   lCf.bPitchAndFamily  = 0; 
   lCf.szFaceName[0]    = '\0'; 

   if (IsImageSlot(theWord))
   {
       // if so, underline
       theEdit.SetSel(theStart, theEnd + 1);
       theEdit.SetSelectionCharFormat(lCf);
	   theEdit.SetSel(theEnd + 1, theEnd + 1);
   }
}

void SDiaryDlg::ColorSearchWord(CRichEditCtrl &theEdit, 
							    int theStart,
							    int theEnd)
{
   CHARFORMAT lCf;

   lCf.cbSize           = sizeof(CHARFORMAT); 
   lCf.dwMask           = CFM_COLOR; 
   lCf.dwEffects        = 0; 
   lCf.yHeight          = 0; 
   lCf.yOffset          = 0; 
   lCf.crTextColor      = 0x000000FF;
   lCf.bCharSet         = 0; 
   lCf.bPitchAndFamily  = 0; 
   lCf.szFaceName[0]    = '\0'; 

   TK_LOG0("... turn it to RED");
   theEdit.SetSel(theStart, theEnd);
   theEdit.SetSelectionCharFormat(lCf);
}

// This method adapt the passed in SYSTEMTIME structure to contain the 
// time corresponding to the passed in filename. 

void SDiaryDlg::GetTimeFromPassedInFile(SYSTEMTIME *pTime)
{
   TK_TRACER("SDiaryDlg::GetTimeFromPassedInFile(SYSTEMTIME *pTime)");
   CCommandLineInfo *lInfo = theApp.GetCmdLineInfo();

   CString lFileName = lInfo->m_strFileName;

   if (lFileName.GetLength() == 0)
   {
      return;
   }

   char *lFullPath = new char[MAX_STRING];
   char *lFloater;
   strcpy_s(lFullPath, MAX_STRING - 1, lFileName);
   lFloater = strrchr(lFullPath, '\\');
   lFloater++;
   if (atoi(lFloater) == 0)
   {
      return;
   }
   sscanf_s(lFloater, "%04d%02d%02d", 
            &(pTime->wYear),
            &(pTime->wMonth), 
            &(pTime->wDay));

   delete [] lFullPath;
}

/*
void SDiaryDlg::GetPassedInConfigFile(std::string &theFilespec)
{
	TK_TRACER("SDiaryDlg::GetPassedInConfigFile(std::string &theFilespec)");
    CCommandLineInfo *lInfo = theApp.GetCmdLineInfo();

    CString lFileName = lInfo->m_strFileName;

    if (lFileName.GetLength() == 0)
    {
		theFileSpec = "settings.dat");
        return;
    }

}
*/

void SDiaryDlg::OnExport() 
{
   TK_TRACER("SDiaryDlg::OnExport(void)");
   CWaitCursor lWaiter;

   TK_LOG0("Determine if we need the SExportDlg, or that a simple export will do");
   TK_LOG0("Save todolist and diary page");
   this->mMouseClicked = false;
   this->mCurrentTodoList->Save();
   this->SaveOldPage(true);      
  
   TK_LOG0("Smart Export requested, complying...");
   ASSERT(mExportDlg != NULL);
   this->mExportDlg->DoModal();

   TK_LOG0("Load the new page");
   this->LoadNewPage();
   return;
}

void SDiaryDlg::OnBnClickedImage()
{
   TK_TRACER("SDiaryDlg::OnBnClickedImage()");
   CWaitCursor lWaiter;

#ifdef THIS_MAY_NOT_BE_NEEDED
   TK_LOG0("Save todolist and diary page");
   this->mCurrentTodoList->Save();
   this->SaveOldPage(true);
#endif

   TK_LOG0("Start the image dialog");
   ASSERT(mImageDlg != NULL);
   mImageDlg->DoModal();
   mMouseClicked = false;
   this->UpdateStatus();
   PerformHighlighting();
}

void SDiaryDlg::UpdateStatus(void)
{
   TK_TRACER("SDiaryDlg::UpdateStatus()");
   CWaitCursor lWaiter;

   TK_LOG0("Update the caption on the image button if needed");
   ASSERT(mImageDlg != NULL);
   int lCount = mImageDlg->CountImageFiles();
   CString lMsg;
   switch (lCount)
   {
   case 0:
      lMsg = "no Images";
      break;
   case 1: 
      lMsg = "1 Image";
      break;
   default:
      lMsg.Format("%d Images", lCount);
      break;
   }

   this->GetDlgItem(IDC_IMG_PRESENT)->SetWindowText(lMsg.GetBuffer());

   TK_LOG0("Update the caption on the files button if needed");
   ASSERT(mFilesDlg != NULL);
   lCount = mFilesDlg->CountFiles();
   switch (lCount)
   {
   case 0:
      lMsg = "no Files";
      break;
   case 1: 
      lMsg = "1 File";
      break;
   default:
      lMsg.Format("%d Files", lCount);
      break;
   }

   this->GetDlgItem(IDC_FILES_PRESENT)->SetWindowText(lMsg.GetBuffer());
}

int SDiaryDlg::GetNumberOfLinesForDiaryWindow(void)
{
   TK_TRACER("SDiaryDlg::GetNumberOfLinesForDiaryWindow()");
   CString lFontSize = this->mSettings.GetSetting(SET_FONT_SIZE)->GetValue();
   int lNumericFontSize = atoi(lFontSize);
   WINDOWPLACEMENT lWinPlace;
   this->GetWindowPlacement(&lWinPlace);
   int lHeight = lWinPlace.rcNormalPosition.bottom - 
                 lWinPlace.rcNormalPosition.top - 80;
   int lLines = (int)((double)lHeight * (30.0 / 61.0) / (double)lNumericFontSize);
   return lLines;
}

int SDiaryDlg::GetNumberOfLinesForTodoWindow(void)
{
   TK_TRACER("SDiaryDlg::GetNumberOfLinesForTodoWindow()");
   CString lFontSize = this->mSettings.GetSetting(SET_FONT_SIZE)->GetValue();
   int lNumericFontSize = atoi(lFontSize);
   WINDOWPLACEMENT lWinPlace;
   this->GetWindowPlacement(&lWinPlace);
   int lHeight = lWinPlace.rcNormalPosition.bottom - 
                 lWinPlace.rcNormalPosition.top - 360;
   int lLines = (int)((double)lHeight * (30.0 / 55.0) / (double)lNumericFontSize);
   return lLines;
}

void SDiaryDlg::OnEnSelchangeRichedit1(NMHDR *pNMHDR, LRESULT *pResult)
{
	TK_TRACER("SDiaryDlg::OnEnSelchangeRichedit1()");
    CWaitCursor lWaiter;

    SELCHANGE *pSelChange = reinterpret_cast<SELCHANGE *>(pNMHDR);

    if (this->mKillSelChangeProcessing == true)
    {
        return;
    }

    this->mKillChangeProcessing = true;

    int lMax = pSelChange->chrg.cpMax;
    int lMin = pSelChange->chrg.cpMin;

	// We only pick up selections of zero, so single clicks...
	long theLastChar = m_DiaryPage.GetTextLength();
    int lDiff = lMax - lMin;
    if (lDiff == 0 && lMin != theLastChar  && lMin != 0)
    {
		// retrieve the full text
		CString thePageBuffer;
		long theLastChar = m_DiaryPage.GetTextLength();
		m_DiaryPage.GetTextRange(0, theLastChar, thePageBuffer);

		// find the word clicked on....
		int theStart = lMin;
		int theEnd   = lMax;

		// find the start...
		while (theStart >= 0 && STools::isWhiteSpace(thePageBuffer[theStart]) == false)
		{
			theStart--;
		}
		theStart++;

		// find the end...
		while (theEnd <= theLastChar && STools::isWhiteSpace(thePageBuffer[theEnd]) == false)
		{
			theEnd++;
		}
		
		if (theEnd > theLastChar)
		{
			theEnd = theLastChar;
		}

		while ((STools::isWhiteSpace(thePageBuffer[theEnd]) ||
			    thePageBuffer[theEnd] == '.' || 
			    thePageBuffer[theEnd] == ',') &&
				theEnd > theStart)
		{
			theEnd--;
		}


		if (theStart <= theEnd && lMin > theStart && lMax < theEnd)
		{
			// isolate the word...
			std::string theWord = thePageBuffer.Mid(theStart, theEnd - theStart + 1);

			// then see if any of the jumptypes apply....
			if (IsHyperJump(theWord))
			{
				JumpToDate(theWord);
			}
			else if (IsURL(theWord))
			{
				JumpToURL(theWord);
			}
			else if (IsMailAddress(theWord))
			{
				JumpToMail(theWord);
			}
			else if (IsImageSlot(theWord))
			{
				JumpToImageSlot(theWord);
			}
			mMouseClicked = false;
		}
    }

	this->mKillChangeProcessing = false;
	this->mShowImages = true;
	*pResult = 0;
}

void SDiaryDlg::OnEnSelchangeRichedit2(NMHDR *pNMHDR, LRESULT *pResult)
{
	TK_TRACER("SDiaryDlg::OnEnSelchangeRichedit2()");
    CWaitCursor lWaiter;

    SELCHANGE *pSelChange = reinterpret_cast<SELCHANGE *>(pNMHDR);

    if (this->mKillSelChangeProcessing == true)
    {
        return;
    }

    this->mKillChangeProcessing = true;

    int lMax = pSelChange->chrg.cpMax;
    int lMin = pSelChange->chrg.cpMin;

	// We only pick up selections of zero, so single clicks...
	long theLastChar = m_TodoList.GetTextLength();
    int lDiff = lMax - lMin;
    if (lDiff == 0 && lMin != theLastChar  && lMin != 0)
    {
		// retrieve the full text
		CString thePageBuffer;
		long theLastChar = m_TodoList.GetTextLength();
		m_TodoList.GetTextRange(0, theLastChar, thePageBuffer);

		// find the word clicked on....
		int theStart = lMin;
		int theEnd   = lMax;

		// find the start...
		while (theStart >= 0 && STools::isWhiteSpace(thePageBuffer[theStart]) == false)
		{
			theStart--;
		}
		theStart++;

		// find the end...
		while (theEnd <= theLastChar && STools::isWhiteSpace(thePageBuffer[theEnd]) == false)
		{
			theEnd++;
		}

		while ((STools::isWhiteSpace(thePageBuffer[theEnd]) ||
			    thePageBuffer[theEnd] == '.' || 
			    thePageBuffer[theEnd] == ',') && 
				theEnd > theStart)
		{
			theEnd--;
		}

		if (theStart <= theEnd)
		{
			// isolate the word...
			std::string theWord = thePageBuffer.Mid(theStart, theEnd - theStart + 1);

			// then see if any of the jumptypes apply....
			if (IsHyperJump(theWord))
			{
				JumpToDate(theWord);
			}
			else if (IsURL(theWord))
			{
				JumpToURL(theWord);
			}
			else if (IsMailAddress(theWord))
			{
				JumpToMail(theWord);
			}
			mMouseClicked = false;
		}
    }

	this->mKillChangeProcessing = false;
	this->mShowImages = true;
	*pResult = 0;
}

void SDiaryDlg::UpdateWeekday(void)
{
   TK_TRACER("SDiaryDlg::UpdateWeekday()");
   CWaitCursor lWaiter;

   CString lMsg;
   lMsg.Format("%s, %s %d%s, %4d", 
               this->mNewSelDate.dayAsLongString(),
               SDate::monthAsLongString((this->mNewSelDate.getMonth())), 
               this->mNewSelDate.getDays(), 
               SDate::getDateSuffix(this->mNewSelDate.getDays()), 
               this->mNewSelDate.getYear());
   mWeekday.SetWindowText(lMsg.GetBuffer());

   // update the bold dates in the calendar control...
   NMDAYSTATE* pDayState = (NMDAYSTATE *)0;
   MONTHDAYSTATE lpDayStateArray[3] = {0, 0, 0};
   lpDayStateArray[0] = GetDayStatesForMonth(this->mNewSelDate.getYear(), this->mNewSelDate.getMonth(), 0);
   lpDayStateArray[1] = GetDayStatesForMonth(this->mNewSelDate.getYear(), this->mNewSelDate.getMonth(), 1);
   lpDayStateArray[2] = GetDayStatesForMonth(this->mNewSelDate.getYear(), this->mNewSelDate.getMonth(), 2);
   MonthCal_SetDayState(m_pMonthCalendar.GetSafeHwnd(),
                        3,
                        lpDayStateArray);

}

void SDiaryDlg::OnBnClickedImport()
{
   TK_TRACER("SDiaryDlg::OnBnClickedImport()");
   CWaitCursor lWaiter;

   TK_LOG0("Determine if we need the SImportDlg, or that a simple import will do");
   if (true /*this->IsDlgButtonChecked(IDC_SMART) != 0*/)
   { 
      TK_LOG0("Save todolist and diary page");
      this->mMouseClicked = false;
      this->mCurrentTodoList->Save();
      this->SaveOldPage(true);      
      
      TK_LOG0("Smart Import requested, complying...");
      ASSERT(mImportDlg != NULL);
      this->mImportDlg->DoModal();

      TK_LOG0("Load the new page");
      this->LoadNewPage();
      return;
   }

   TK_LOG0("Simple Import wanted, complying...");
   char lFilters[255] = "Text Files (*.txt)|*.txt|Diary Files (*.dry)|*.dry|All Files (*.*)|*.*||";
   HRESULT hResult;

   DWORD lDirReturn = 0;
   char  *lPath = new char[MAX_STRING];
   char  *lImportPath = new char[MAX_STRING];

   TK_LOG0("Retrieve the path of execution");
   lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);

   CFileDialog dlg(TRUE, NULL, NULL, OFN_FILEMUSTEXIST, lFilters);

   hResult = (int)dlg.DoModal();
   if(hResult != IDOK) 
   {
      SetCurrentDirectory(lPath);
	  delete [] lImportPath;
	  delete [] lPath;
      return;
   }

   CString lFullFileSpec;
   lDirReturn = GetCurrentDirectory(MAX_STRING, lImportPath);

   lFullFileSpec.Format("%s\\%s", lImportPath, dlg.GetFileName());

   TK_LOG1("Opening file %s", lFullFileSpec.GetBuffer());
   FILE *lFile = fopen(lFullFileSpec.GetBuffer(), "rb");
   if (lFile != NULL)
   {
      TK_LOG0("File open, reading it and appending to current page");
      long lMin = 0;
      long lMax = 0;
      char lBuffer[MAX_BUFFER];
      char lAppendBuffer[MAX_BUFFER];
      m_DiaryPage.GetSel(lMin, lMax);
      m_DiaryPage.SetSel(0, -1);
      CString theStr = m_DiaryPage.GetSelText();
	  strcpy_s(lBuffer, MAX_BUFFER, theStr);
      m_DiaryPage.SetSel(lMin, lMax);

      int lBytesRead = fread(lAppendBuffer, 1, MAX_BUFFER, lFile);
      if (lBytesRead > 0)
      {
         TK_LOG0("Import Text read, closing off buffer, and appending");
         lAppendBuffer[lBytesRead] = '\0';
         for (int i = strlen(lBuffer); i >= 0; i--)
         {
            if (lBuffer[i] == '\n')
            {
               lBuffer[i] = '\0';
            }
            else
            {
               break;
            }
         }
         strcat_s(lBuffer, MAX_BUFFER - 1, "\n\n");
         strcat_s(lBuffer, MAX_BUFFER - 1, lAppendBuffer);
         m_DiaryPage.SetWindowText(lBuffer);
         m_DiaryPage.SetSel(lMin, lMax);
      }
      fclose(lFile);
   }
   else
   {
      CString lMsg;
      lMsg.Format("File %s could not be opened for reading", lFullFileSpec.GetBuffer());
      MessageBox(lMsg.GetBuffer());
      TK_LOG0(lMsg.GetBuffer());
   }

   SetCurrentDirectory(lPath);
   PerformHighlighting();

   delete [] lImportPath;
   delete [] lPath;
}

void SDiaryDlg::SavePosition(int x, int y, int width, int height)
{
   TK_TRACER("SDiaryDlg::SavePosition(void)");
   CWaitCursor lWaiter;

   TK_LOG0("Save window position to settings.dat");
   WINDOWPLACEMENT lWinPlace;
   this->GetWindowPlacement(&lWinPlace);

   // cancel save if window centered, and not moved yet...
   if (mWindowCentered == true &&
	   (mTop == lWinPlace.rcNormalPosition.top ||
	    mLeft == lWinPlace.rcNormalPosition.left))
   {
	   return;
   }

   // ok, we save so set the flag to off
   mWindowCentered = false;
   mTop = -1;
   mLeft = -1;

   char *lTop = new char[MAX_STRING];
   if (y == -1)
   {
	   y = lWinPlace.rcNormalPosition.top;
   }
   _itoa_s(y, lTop, MAX_STRING - 1, 10);
   SSetting *lTopSSetting = new SSetting(SET_TOP, lTop);
   this->mSettings.AddSetting(lTopSSetting);
   delete [] lTop;

   char *lLeft = new char[MAX_STRING];
   if (x == -1)
   {
	   x = lWinPlace.rcNormalPosition.left;
   }
   _itoa_s(x, lLeft, MAX_STRING - 1, 10);
   SSetting *lLeftSSetting = new SSetting(SET_LEFT, lLeft);
   this->mSettings.AddSetting(lLeftSSetting);
   delete [] lLeft;

   char *lWidth = new char[MAX_STRING];
   if (width == -1)
   {
       width = lWinPlace.rcNormalPosition.right - 
	           lWinPlace.rcNormalPosition.left;
   }
   _itoa_s(width, lWidth, MAX_STRING - 1, 10);
   SSetting *lWidthSSetting = new SSetting(SET_WIDTH, lWidth);
   this->mSettings.AddSetting(lWidthSSetting);
   delete [] lWidth;

   char *lHeight = new char[MAX_STRING];
   if (height == -1)
   {
       height = lWinPlace.rcNormalPosition.bottom - 
	            lWinPlace.rcNormalPosition.top;
   }
   _itoa_s(height, lHeight, MAX_STRING - 1, 10);
   SSetting *lHeightSSetting = new SSetting(SET_HEIGHT, lHeight);
   this->mSettings.AddSetting(lHeightSSetting);
   delete [] lHeight;

   this->mSettings.WriteFile();
}

SDiaryPage *SDiaryDlg::getSDiaryPageForCurrentDate(void)
{
   return getSDiaryPageForDate(this->mNewSelDate);
}

SDiaryPage *SDiaryDlg::getSDiaryPageForDate(SDate pDate)
{
   TK_TRACER("SDiaryDlg::GetSDiaryPageForCurrentDate(void)");
   CWaitCursor lWaiter;
   
   if (this->mDiaryPage != NULL)
   {
      TK_LOG0("Delete the previous object, before creating a new one");
      delete this->mDiaryPage;
   }

   DWORD lDirReturn = 0;
   char  *lPath = new char[MAX_STRING];
   CString lCurDir; 
   TK_LOG0("Retrieve the path of execution");
   lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);
   lCurDir.Format("%s\\%08d.dry", lPath, pDate.asDecimal());
   delete [] lPath;

   this->mDiaryPage = new SDiaryPage(lCurDir,
                                      pDate, 
                                      theApp.GetCryptKey());
   return this->mDiaryPage; 
}

CString SDiaryDlg::getFilenameForDate(SDate pDate)
{
   TK_TRACER("SDiaryDlg::getFilenameForDate(SDate pDate)");
   CString lReturn;
   DWORD lDirReturn = 0;
   char  *lPath = new char[MAX_STRING];
   TK_LOG0("Retrieve the path of execution");
   lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);
   lReturn.Format("%s\\%08d.dry", 
                  lPath, 
                  pDate.asDecimal());
   delete [] lPath;

   return lReturn;               
}

CString SDiaryDlg::getWindowTitle(void)
{
   TK_TRACER("SDiaryDlg::getWindowTitle(void)");
   DWORD lDirReturn = 0;
   char  *lPath = new char[MAX_STRING];
   CString lReturn;

   TK_LOG0("Retrieve the path of execution");
   lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);
   lReturn.Format("%s", lPath);
   delete [] lPath;

   return lReturn; 
}


void SDiaryDlg::OnBnClickedPrev()
{
    TK_TRACER("SDiaryDlg::OnBnClickedPrev(void)");
	// iterate backward until we find another diary page
	SDate theDate = this->mNewSelDate;
	int daysTried = 0;
    do
	{
		theDate.Subtract(0, 0, 1);
		daysTried++;

	} while (TestPage(theDate) == false && daysTried < MAX_DAYS_TO_SCAN);
	if (TestPage(theDate) == false)
	{
		MessageBox("No previous entry found in over five years", 
			       "Warning", 
				   MB_OK);
		return;
	}
    TK_LOG0("But save diary page and todolist before doing so");
    this->mMouseClicked = false;
    this->mCurrentTodoList->Save();
    this->SaveOldPage(true);
	this->mNewSelDate = theDate;
    SYSTEMTIME lTime;
	lTime.wYear  = theDate.getYear();
    lTime.wMonth = theDate.getMonth();
	lTime.wDay   = theDate.getDays();
	m_pMonthCalendar.SetCurSel(&lTime);
    this->LoadNewPage();
    this->UpdateWeekday();
    this->UpdateStatus();
}

void SDiaryDlg::OnBnClickedNext()
{
    TK_TRACER("SDiaryDlg::OnBnClickedNext(void)");
	// iterate forward until we find another diary page
	SDate theDate = this->mNewSelDate;
	int daysTried = 0;
    do
	{
		theDate.Add(0, 0, 1);
		daysTried++;

	} while (TestPage(theDate) == false && daysTried < MAX_DAYS_TO_SCAN);
	
	if (TestPage(theDate) == false)
	{
		MessageBox("No next entry found in over five years", 
			       "Warning", 
				   MB_OK);
		return;
	}
    
	TK_LOG0("But save diary page and todolist before doing so");
    this->mMouseClicked = false;
    this->mCurrentTodoList->Save();
    this->SaveOldPage(true);
	this->mNewSelDate = theDate;
    SYSTEMTIME lTime;
	lTime.wYear  = theDate.getYear();
    lTime.wMonth = theDate.getMonth();
	lTime.wDay   = theDate.getDays();
	m_pMonthCalendar.SetCurSel(&lTime);
    this->LoadNewPage();
    this->UpdateWeekday();
    this->UpdateStatus();
}

bool SDiaryDlg::TestPage(SDate pSelDate)
{
   TK_TRACER("SDiaryDlg::TestPage(SDate pSelDate)");
   TK_LOG0("Build the correct filename, and open the file");
   CString lFileName;
   lFileName.Format("%08d.dry", pSelDate.asDecimal());

   FILE *lFile = fopen(lFileName, "rb");

   if (lFile != NULL)
   {
      fclose(lFile);
	  return true;
   }
   return false;
}


void SDiaryDlg::OnBnClickedFiles()
{
    TK_TRACER("SDiaryDlg::OnBnClickedFiles()");
    CWaitCursor lWaiter;

    //this->SavePosition();	
   
    TK_LOG0("Files Dialog requested, complying...");
    ASSERT(mFilesDlg != NULL);
	this->mFilesDlg->DoModal();
    this->UpdateStatus();
}

void SDiaryDlg::OnBnClickedLblConfigure()
{
    TK_TRACER("SDiaryDlg::OnBnClickedConfig()");
    CWaitCursor lWaiter;

    //this->SavePosition();	
   
    TK_LOG0("Configure Dialog requested, complying...");
	if (this->mShortcutDlg->DoModal() == IDOK)
	{
		SetLabelCombobox();
		SetTodoCombobox();
	}
}

void SDiaryDlg::SetLabelCombobox()
{
	mComboShortcuts.ResetContent();
	mShortcutDlg->SetDiaryDlg();
	for (int i = 1; i <= MAX_SHORTCUT; i++)
	{
		// version 5.04 to 5.05:
		// this is done to weed out the ampersands from the shortcuts, 
		// that allowed for keyboard shortcuts when buttons drove them. 
		// Now we have a combobox, which has no use for them...

		CString theName = mShortcutDlg->GetLabel(i)->GetLabelText();
		std::string theFindString = theName;
		if (theFindString.find("&", 0) != std::string::npos)
		{
			// update shortcut object and write configuration
			CString theNewName = theName;
			theNewName.Replace("&", "");
			mShortcutDlg->GetLabel(i)->SetLabelText(theNewName);
			mShortcutDlg->GetLabel(i)->WriteToSettings(true);
		}

	    mShortcutDlg->GetLabel(i)->AddToCombo(&mComboShortcuts);
	}
}

void SDiaryDlg::SetTodoCombobox()
{
	mComboTodolists.ResetContent();

	// retrieve the string to insert, and insert it
	bool entriesThere = false;
	for (int i = 0; i <= MAX_TODO_LISTS; i++)
	{
		STodoList *theList = mShortcutDlg->GetTodoList(i);
		std::string theName = theList->GetName(false);

		// version 5.04 to 5.05:
		// this is done to weed out the ampersands from the todolist, 
		// that allowed for keyboard shortcuts when buttons drove them. 
		// Now we have a combobox, which has no use for them...

		if (theName.find("&", 0) != std::string::npos)
		{
		    // copy name to form without ampersand
			CString theFileName = theName.c_str();
			theFileName += ".dry";
			CString noAmpersand = theFileName;
			noAmpersand.Replace("&", "");

			// rename corresponding file to new form
			MoveFile(theFileName, noAmpersand);

			// update todolist object and write configuration
			CString theNewName = theName.c_str();
			theNewName.Replace("&", "");
			theList->SetName(theNewName);
			theList->WriteToSettings(true);
			theName = theNewName;
		}

		if (theName != "todolist")
		{
		    mComboTodolists.AddString(theName.c_str());
			entriesThere = true;
		}
		else if (i == 0)
		{
		    mComboTodolists.AddString("Default");
			entriesThere = true;
		}
	}

    SSetting *lSetting = this->GetSettings()->GetSetting(SET_TODOLIST_NAME);

	std::string theChosen;
    if (lSetting != NULL)
    {
        TK_LOG0("Position settings found, positioning window");
        theChosen = lSetting->GetValue();

		// determine which is which....
		for (int i = 0; i < mComboTodolists.GetCount(); i++)
		{
			CString theText;
			mComboTodolists.GetLBText(i, theText);
			if (strcmp(theText, theChosen.c_str()) == 0)
			{
				mComboTodolists.SetCurSel(i);
				break;
			}
		}

	}
	else
	{
	    mComboTodolists.SetCurSel(0);
	}
}


void SDiaryDlg::EnableAllControls(BOOL theStatus)
{
   this->GetDlgItem(IDC_MONTHCALENDAR1)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_SEARCH)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_IMAGE)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_FILES)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_INSERT_TIME)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_PREV)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_NEXT)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_NUMDAYS)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_PRINT)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL_CONFIGURE)->EnableWindow(theStatus);

   if (theStatus == TRUE)
   {
	   SetLabelCombobox();
	   SetTodoCombobox();
   }
}

void SDiaryDlg::InsertString(CString &theInsertString) 
{
   TK_TRACER("SDiaryDlg::InsertString)");
   CWaitCursor lWaiter;

   long lMin = 0;
   long lMax = 0;

   if (theInsertString == "")
   {
	   return;
   }

   TK_LOG0("set the selection to a single point of insertion");
   m_DiaryPage.GetSel(lMin, lMax);
   m_DiaryPage.SetSel(lMin, lMin);

   TK_LOG0("insert the string into the CRichEditCtrl of the diary page");
   int theSize = theInsertString.GetLength() - 1;
   if (theInsertString[theSize] != '\n')
   {
       theInsertString += " ";
   }
   m_DiaryPage.ReplaceSel(theInsertString.GetBuffer());
   m_DiaryPage.SetFocus();
}

void SDiaryDlg::SetNewDate(SDate theNewDate)
{
     // the CMonthCalCtrl sends spurious SelChange messages, for the 
     // already selected date. This if is supposed to catch these....

     TK_LOG0("If new date doesn't match previous date...");
     if (theNewDate != this->mNewSelDate)
     {
        TK_LOG0("Switch pages, saving the old and loading the new");
        this->mOldSelDate = this->mNewSelDate;
        this->mNewSelDate = theNewDate;
        SYSTEMTIME lTime;
		lTime.wYear  = theNewDate.getYear();
		lTime.wMonth = theNewDate.getMonth();
		lTime.wDay   = theNewDate.getDays();
		m_pMonthCalendar.SetCurSel(&lTime);
        this->SaveOldPage();
        this->LoadNewPage();
        this->mMouseClicked = false;
        this->mCurrentTodoList->Save();
        this->UpdateWeekday();
        this->UpdateStatus();

		m_DiaryPage.SetSel(-1, -1);  // continue at end of page.....
     }
}
void SDiaryDlg::OnBnClickedPrint()
{
    TK_LOG0("But save diary page and todolist before doing so");
    this->mMouseClicked = false;
    this->mCurrentTodoList->Save();
    this->SaveOldPage(false);

	// get number of pages
	int NumDays = 0;
	char theString[21];
	this->GetDlgItem(IDC_NUMDAYS)->GetWindowTextA(theString, 20);
	NumDays = atoi(theString);
	if (NumDays == 0)
	{
		NumDays = 1;
	}

	// and print...
	this->mExportDlg->PrintEntries(NumDays);
}

void SDiaryDlg::ActivateTodoList(STodoList *pTodoList)
{
    this->mMouseClicked = false;
	if (mCurrentTodoList != 0)
	{
		// save old todolist
		mCurrentTodoList->Save();
	}

	// save the new one
	mCurrentTodoList = pTodoList;

	// and load it...
	mCurrentTodoList->Load();
}

void SDiaryDlg::OnEnChangeNumdays()
{
   char *lNumDays = new char[MAX_STRING];
   this->GetDlgItem(IDC_NUMDAYS)->GetWindowTextA(lNumDays, MAX_STRING - 1);
   SSetting *lNumDaysSSetting = new SSetting(SET_NUMDAYS, lNumDays);
   this->mSettings.AddSetting(lNumDaysSSetting);
   delete [] lNumDays;

   this->mSettings.WriteFile();
}

void SDiaryDlg::OnWindowPosChanged(WINDOWPOS* lpwndpos)
{
	CResizeDlg::OnWindowPosChanged(lpwndpos);

	SavePosition(lpwndpos->x, lpwndpos->y, lpwndpos->cx, lpwndpos->cy);
}

bool SDiaryDlg::wasConfigRead()
{
	return mConfigRead;
}


void SDiaryDlg::OnCbnSelchangeComboShortcuts()
{
	int lChoice = mComboShortcuts.GetCurSel();
	CString lText;
	mComboShortcuts.GetLBText(lChoice, lText);
	if (lText == "")
	{
		return;
	}

	for (int i = 1; i <= MAX_SHORTCUT; i++)
	{
	    SShortcut *theShortcut = mShortcutDlg->GetLabel(i);
		if (theShortcut->GetLabelText() == lText)
		{
			// if timestamp is set, call Add timestamp
			if (theShortcut->NeedsTimestamp())
			{
				OnInsertTime();
			}

			// retrieve the string to insert, and insert it
			InsertString(theShortcut->GetInsertText());
		}
	}

	mComboShortcuts.SetCurSel(-1);
}


void SDiaryDlg::OnCbnSelchangeComboTodo()
{
	int lChoice = mComboTodolists.GetCurSel();
	CString lText;
	mComboTodolists.GetLBText(lChoice, lText);
	if (lText == "")
	{
		return;
	}

	for (int i = 0; i <= MAX_TODO_LISTS; i++)
	{
		if (lText == mShortcutDlg->GetTodoList(i)->GetName(false).c_str() ||
			(lText == "Default" && i == 0))
		{
			// store chosen name....
			SSetting *lNewSSettingObject = new SSetting(SET_TODOLIST_NAME, lText.GetBuffer() );
			this->GetSettings()->AddSetting(lNewSSettingObject);
			this->GetSettings()->WriteFile();

			// and activate the list...
        	ActivateTodoList(mShortcutDlg->GetTodoList(i));
            PerformHighlighting();
		}
	}
}

BOOL SDiaryDlg::PreTranslateMessage(MSG *pMsg)
{
	if (pMsg->message == WM_KEYDOWN)
	{
		int theShortcut = -1;
		switch (pMsg->wParam)
		{
		case VK_F1:		theShortcut = 1; break;
		case VK_F2:		theShortcut = 2; break;
		case VK_F3:		theShortcut = 3; break;
		case VK_F4:		theShortcut = 4; break;
		case VK_F5:		theShortcut = 5; break;
		case VK_F6:		theShortcut = 6; break;
		case VK_F7:		theShortcut = 7; break;
		case VK_F8:		theShortcut = 8; break;
		case VK_F9:		theShortcut = 9; break;
		default:
			break;
		}

		if (theShortcut > 0 && theShortcut <= 9)
		{
			SShortcut *theShortcutObject = mShortcutDlg->GetLabel(theShortcut);

			if (theShortcutObject != 0)
			{
				// if timestamp is set, call Add timestamp
				if (theShortcutObject->NeedsTimestamp())
				{
					OnInsertTime();
				}

				// retrieve the string to insert, and insert it
				InsertString(theShortcutObject->GetInsertText());
		    }
		}
	}
	
	if (pMsg->message == WM_LBUTTONDOWN)
	{
		mMouseClicked = true;
	}

	return CResizeDlg::PreTranslateMessage(pMsg);
}

void SDiaryDlg::JumpToURL(std::string &theWord)
{
	if (mMouseClicked == false)
	{
		return;
	}

	mMouseClicked = false;
	CWaitCursor lWaiter;

	// open an internet page...
	ShellExecute(0, _T("open"), theWord.c_str(), 
                           0, 0, SW_SHOWNORMAL);
}

void SDiaryDlg::JumpToMail(std::string &theWord)
{
	if (mMouseClicked == false)
	{
		return;
	}

	mMouseClicked = false;
	CWaitCursor lWaiter;

	std::string theMailTo = "mailto:";
	theMailTo += theWord;

	// open an internet page...
	ShellExecute(0, _T("open"), theMailTo.c_str(), 
                           0, 0, SW_SHOWNORMAL);
}

void SDiaryDlg::JumpToImageSlot(std::string &theWord)
{
	if (mMouseClicked == false)
	{
		return;
	}

	mMouseClicked = false;
	CWaitCursor lWaiter;
	int theSlot = mImageDlg->SlotNumberFromChar(theWord[4]);
    ASSERT(mImageDlg != NULL);
    mImageDlg->DoModal(theSlot);
}

void SDiaryDlg::JumpToDate(std::string &theWord)
{
	if (mMouseClicked == false)
	{
		return;
	}

	mMouseClicked = false;
    CWaitCursor lWaiter;
	int lYear  = atoi(theWord.substr(0, 4).c_str());
	int lMonth = atoi(theWord.substr(4, 2).c_str());
	int lDay   = atoi(theWord.substr(6, 2).c_str());
	if (SDate::isValidDate(lYear, lMonth, lDay))
	{
	    SDate theNewDate(lYear, lMonth, lDay);
	    this->SetNewDate(theNewDate); 
	}
}

void SDiaryDlg::OnLButtonDown(UINT nFlags, CPoint point)
{
	mMouseClicked = false;
}



void CAboutDlg::OnCbnSelchangeAutoUpgrade()
{
	int lChoice = mAutoUpgrade.GetCurSel();
	CString lText;
	mAutoUpgrade.GetLBText(lChoice, lText);
	if (lText == "")
	{
		return;
	}

	SSetting *lNewSSettingObject = new SSetting(SET_UPGRADE_ANSWER, lText.GetBuffer() );
	SDiaryDlg::Instance()->GetSettings()->AddSetting(lNewSSettingObject);
    SDiaryDlg::Instance()->GetSettings()->WriteFile();
}
