//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         diaryDlg.h
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  Declaration of Class diaryDlg.
//
//////////////////////////////////////////////////////////////////////////////////////////////
#ifndef DIARYDLG_H_INCLUDED
#define DIARYDLG_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "diary.h"
#include "diarypage.h"
#include "searchDlg.h"
#include "imagedlg.h"
#include "settingsfile.h"
#include "sdate.h"
#include "afxwin.h"
#include "importdlg.h"
#include "exportdlg.h"
#include "filesdlg.h"
#include "SLabelDlg.h"

#include "ResizeDlg.h"

class STodoList;

//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Class SDiaryDlg
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  this dialog handles the main user interface of the Diary application. It 
//                handles the calendar, the rich edit controls, and the various other controls
//                placed on the main dialog. This dialog is fully resizeable. 
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Date		Name                  Comments
//	20080921	Andr Slabber         Created
//
//////////////////////////////////////////////////////////////////////////////////////////////
class SDiaryDlg : public CResizeDlg
{
	friend class SImageDlg;

// Construction
public:
    static SDiaryDlg *Instance();
	static void DeleteInstance();

    bool SaveOldPage(bool pFinal = false);
    void LoadNewPage(void);

    SSettingsFile *GetSettings(void)   { return &this->mSettings; };

    void GetTimeFromPassedInFile(SYSTEMTIME *pTime);
    SDate GetCurrentDate(void)         { return this->mNewSelDate; };
    CString getFilenameForDate(SDate pDate);
    CString getWindowTitle(void);
    void EnableAllControls(BOOL theStatus);
    void UpdateWeekday(void);
    void SavePosition(int x = -1, int y = -1, int width = -1, int height = -1);
    SDiaryPage *getSDiaryPageForCurrentDate(void);
    SDiaryPage *getSDiaryPageForDate(SDate pDate);
    bool SDiaryDlg::TestPage(SDate theSelDate);
    MONTHDAYSTATE GetDayStatesForMonth(int pYear, int pMonth, int pMonthOffset);
    void SetLabelCombobox();
    void SetTodoCombobox();
    void InsertString(CString &theInsertString);
    void SetNewDate(SDate theNewDate);
    bool isListDirty() { return mListDirty; };
    void clearListDirty() { mListDirty = false; };
    bool wasConfigRead();
    void PerformHighlighting();
    void PerformHighlighting(CRichEditCtrl &pControl);
    void ColorSearchWords(CRichEditCtrl &pControl);

    bool IsHyperJump(std::string &theWord);
    bool IsURL(std::string &theWord);
    bool IsMailAddress(std::string &theWord);
    bool IsImageSlot(std::string &theWord);
    bool IsSearchWords(std::string &theWord);

    void HighlightHyperJump(CRichEditCtrl &theEdit, 
	 					    int theStart,
						    int theEnd, 
						    std::string &theWord);

    void HighlightURL(CRichEditCtrl &theEdit, 
					  int theStart, 
					  int theEnd,
					  std::string &theWord);

    void HighlightMail(CRichEditCtrl &theEdit, 
					   int theStart, 
					   int theEnd,
					   std::string &theWord);

    void HighlightImageSlot(CRichEditCtrl &theEdit,
						    int theStart,
						    int theEnd,
						    std::string &theWord);

    void ColorSearchWord(CRichEditCtrl &theEdit, 
					     int theStart,
					     int theEnd);

    void JumpToURL(std::string &theWord);
    void JumpToMail(std::string &theWord);
    void JumpToImageSlot(std::string &theWord);
    void JumpToDate(std::string &theWord);

    BOOL PreTranslateMessage(MSG *pMsg);

private:
	static SDiaryDlg *mInstance;

    bool mPageDirty;
    bool mListDirty;
    bool mShowImages;
    bool mKillChangeProcessing;
    bool mKillSelChangeProcessing;

    // stuff to correctly reposition if offscreen
    bool mWindowCentered;
    int  mTop;
    int  mLeft;

    SSearchDlg *mSearchDlg;      
    SImageDlg  *mImageDlg;
    SImportDlg *mImportDlg;
    SExportDlg *mExportDlg;
    SFilesDlg  *mFilesDlg;
    SLabelDlg  *mShortcutDlg;

    SDiaryPage *mDiaryPage;

    SSettingsFile mSettings;

	SDiaryDlg(CWnd* pParent = NULL);   // standard constructor

// Dialog Data
   //{{AFX_DATA(SDiaryDlg)
    enum { IDD = IDD_DIARY_DIALOG };
    CEdit           m_EncryptionKey;
    CRichEditCtrl   m_DiaryPage;
    CMonthCalCtrl   m_pMonthCalendar;
    //}}AFX_DATA

    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(SDiaryDlg)
protected:
    virtual void DoDataExchange(CDataExchange* pDX);   // DDX/DDV support
    //}}AFX_VIRTUAL

// Implementation
protected:
    HICON m_hIcon;
    SDate mOldSelDate;
    SDate mNewSelDate;

    void SetFont(int pFontSize);
    // Generated message map functions
    //{{AFX_MSG(SDiaryDlg)
    virtual BOOL OnInitDialog();
    afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
    afx_msg void OnPaint();
    afx_msg HCURSOR OnQueryDragIcon();
    afx_msg void OnGetdaystateMonthcalendar1(NMHDR* pNMHDR, LRESULT* pResult);
    afx_msg void OnSelchangeMonthcalendar1(NMHDR* pNMHDR, LRESULT* pResult);
    virtual void OnOK();
    virtual void OnCancel();
    afx_msg void OnWindowPosChanged(WINDOWPOS* lpwndpos);
    afx_msg void OnChangeRichedit1();
    afx_msg void OnChangeRichedit2();
    afx_msg void OnInsertTime();
    afx_msg void OnSearch();
    afx_msg void OnExport();
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()

public:
    CRichEditCtrl   m_TodoList;
    afx_msg void OnBnClickedImage();
    void UpdateStatus(void);
    int GetNumberOfLinesForDiaryWindow(void);
    int GetNumberOfLinesForTodoWindow(void);
    afx_msg void OnEnSelchangeRichedit1(NMHDR *pNMHDR, LRESULT *pResult);
    afx_msg void OnEnSelchangeRichedit2(NMHDR *pNMHDR, LRESULT *pResult);
    CDC mPrintDC;
    void ActivateTodoList(STodoList *pTodoList); 

private:
    CStatic mWeekday;
    int mSelectedTodoList;
    int mOldSelectedTodoList;
    bool mConfigRead;
    bool mMouseClicked;

public:
    STodoList *mCurrentTodoList;
    afx_msg void OnBnClickedImport();
    virtual ~SDiaryDlg(void);
    afx_msg void OnBnClickedButton1();
    afx_msg void OnBnClickedPrev();
    afx_msg void OnBnClickedNext();
    afx_msg void OnBnClickedFiles();
    afx_msg void OnBnClickedLblConfigure();
    afx_msg void OnBnClickedPrint();
    afx_msg void OnEnChangeNumdays();
    CComboBox mComboShortcuts;
    afx_msg void OnCbnSelchangeComboShortcuts();
    afx_msg void OnCbnSelchangeComboTodo();
    afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
    CComboBox mComboTodolists;
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_DIARYDLG_H__56E16265_1095_11D6_924F_000102213AEE__INCLUDED_)
