//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         diaryPage.h
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  This header file defines the SDiaryPage class, and should be included by any
//                file using the methods of this class. If just the class name is used in for 
//                instance the parameters of other methods, an empty class definition may be 
//                enough. 
//
//////////////////////////////////////////////////////////////////////////////////////////////

#ifndef DIARYPAGE_H_INCLUDED
#define DIARYPAGE_H_INCLUDED

#include "stdafx.h"
#include "afxtempl.h"
#include "sdate.h"
#include "exportdlg.h"
#include "diaryparagraph.h"

//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Class SDiaryPage
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  One of the text handling classes for merging diary pages. It represents a
//                full diary page, which consists of a number of SDiaryParagraph objects. 
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Date		Name                  Comments
//	20080921	Andr Slabber         Created
//
//////////////////////////////////////////////////////////////////////////////////////////////
class SDiaryPage
{
private:
   SDate mDate;
   char *mCryptKey;
   char *mFileSpec;
   CList<SDiaryParagraph *, SDiaryParagraph *> mParagraphList;

public:
   SDiaryPage(void);

   virtual ~SDiaryPage(void);
   SDiaryPage(CString pFileSpec, SDate pDate, char * pCryptKey);

   virtual void LoadParagraphs(void);
   virtual char *GetFileSpec(void);
   virtual void AddParagraphAtEnd(SDiaryParagraph *pNewParagraph);

   virtual void InsertBefore(SDiaryPage *pImport);
   virtual void InsertAfter(SDiaryPage *pImport);
   virtual void Merge(SDiaryPage *pImport);
   virtual void Merge(SDiaryParagraph *pNextParagraph);
   virtual void Write(void);
   virtual void DisplayDump(void);
   virtual void Export(FILE *pFile, CString pDirectory, OutputType pType);

   virtual void SortParagraphs(void); 
   virtual SDiaryParagraph *GetFirstParagraph(POSITION &pPos);
   virtual SDiaryParagraph *GetLastParagraph(POSITION &pPos);
   virtual void UnlinkFirstParagraph(void);
   virtual SDiaryParagraph *GetNextParagraph(POSITION &pPos);
   virtual SDiaryParagraph *GetPreviousParagraph(POSITION &pPos);
};

#endif
