// The logger class is a static class, for which no objects should 
// be instantiated. Hence the absence of constructors and destructor, 
// as these are only needed for object creation.  the Logger class 
// controls the global logfile handle, and has functionality to write 
// to the logfile using a printf-like method. 

#ifndef LOGGER_H_INCLUDED
#define LOGGER_H_INCLUDED

#include <stdio.h>

#include "types.h"
#include "settings.h"

class Logger
{
private:
   static FILE *cvLogFile;
   static int   cvIndent;

public:
   static RetCode open(void);
   static RetCode close(void);

   static void Log(MsgType pType, 
                   char   *pFile, 
                   int     pLine, 
                   RetCode pCode, 
                   char   *pFormat = NULL, 
                   ...); 

   static void singleLine(void);
   static void doubleLine(void);

   static void Indent(void)      { cvIndent++; };
   static void Undent(void)      { cvIndent--; };

#ifdef TOOLKIT_SELFTEST
   static RetCode selfTest(void);
#endif
};

#endif  //  LOGGER_H_INCLUDED