// This program is supposed to test the toolkit stuff extensively, 
// to avoid having errors in any other application

#include "toolkit.h"

int TestingRoutine1(long pOuterloop, long pDivider)
{
   TK_TRACER("TestingRoutine1");
   TK_TIMER_DEF(lTimer, "testing level 1");
   TK_TIMER_START(lTimer);

   for (long i = 0; i < pOuterloop; i++)
   {
      if (i % pDivider == pDivider - 1)
      {
         TK_TIMER_STOP(lTimer);
         TK_TIMER_START(lTimer);
      }
   }

   // going out of scope also stops timer
   return RC_OK;
}

// This routine is supposed to test any functionality that involves
// more than one class. The toolkit does not have any of these situations
// so the test performed here are not really system tests.

#ifdef TOOLKIT_SYSTEMTEST
RetCode systemTest(void)
{
   RetCode lReturn = RC_OK;

   // Note the start of the selfTest in the logfile
   Logger::doubleLine();
   TK_LOG0("Starting systemTest for the Toolkit");

   {
      TK_TRACER("systemTest");

      long lOuterloop =  2400000L;
      long lDivider   =   430000L;
      TK_LOG0("call some function, logging the parms");
      TK_PRMLONG("pOuterloop",  lOuterloop);
      TK_PRMINT("pDivider",     lDivider);
      TestingRoutine1(lOuterloop, lDivider);
   }
   
   // Note the end of the selfTest in the logfile
   TK_LOG0("End of systemTest for the Toolkit"); 
   Logger::doubleLine();
   
   return lReturn; 
}
#endif

RetCode main(int argc, char *argv[])
{
   RetCode lReturn = RC_OK;

   // attempt to open the logfile, saving the return code
   lReturn = TK_LOG_OPEN;

   if (lReturn != RC_OK)
   {
      // no logfile open, so send error to stderr
      fprintf(stderr, "%s: %s\n", TOOLKIT_LOGNAME, RC2Txt(lReturn));
   }
   else
   {
      Logger::Log(MT_INFO, _FILE_, __LINE__, RC_OK, 
                  "%s open, continue with rest of program", TOOLKIT_LOGNAME);
      try
      {
         // now seems like a nice time to test ourselves if needed
#ifdef TOOLKIT_SELFTEST
         Logger::selfTest();
         Timer::selfTest();
         Tracer::selfTest();
         Exception::selfTest();
#endif
#ifdef TOOLKIT_SYSTEMTEST
         ::systemTest();
#endif
         // Here's where normal programming would start...
      }
      TK_STANDARD_CATCHES;
    
      // and close the file after all is done
      lReturn = TK_LOG_CLOSE;
      if (lReturn != RC_OK)
      {
         // no logfile closed, so log the error to stderr
         fprintf(stderr, "%s: %s\n", TOOLKIT_LOGNAME, RC2Txt(lReturn));
      }
   }
 
   // returns either RC_OK, or error on open or close of logfile
   return lReturn;
}
