#ifndef TYPES_H_INCLUDED
#define TYPES_H_INCLUDED

// This file specifies the 'enumerations' needed in the 
// Programmer's Toolkit. They are not true enumerations, 
// because of size considerations: C++ enumerations default 
// to ints, usually squandering four bytes where two or even 
// one will suffice. This has been solved by using a typedef 
// to pin down the wanted memory size for the enumeration, 
// and using defines to set the values for the enumeration. 
// In usage this is no different from a true enumeration, 
// only that it leaves space for the enumeration type to 
// have a value that is not enumerated. 

typedef char  MsgType;
typedef short RetCode;

// prototypes for the string translation functions

char *MT2Txt(MsgType pType);
char *RC2Txt(RetCode pCode);

// the enumeration for the type of the messages in the toolkit

#define   MT_INFO                             ((MsgType)0x00)
#define   MT_WARNING                          ((MsgType)0x01)
#define   MT_ERROR                            ((MsgType)0x02)
#define   MT_FATAL                            ((MsgType)0x03)
#define   MT_LOGGING                          ((MsgType)0x04)
#define   MT_TRACING                          ((MsgType)0x05)
#define   MT_TIMING                           ((MsgType)0x06)

// the enumeration for the errors that are reported through the 
// toolkit. This collection is supposed to grow over time, as I
// (and hopefully others) will be using the toolkit in programming 
// projects. The error codes should be grouped in related collections
// like the general (0x00) and file related (0x01) groups. 

#define   RC_OK                               ((RetCode)0x0000)
#define   RC_FATAL                            ((RetCode)0x0001)
#define   RC_ASSERT                           ((RetCode)0x0002)
#define   RC_TESTFAILED                       ((RetCode)0x0003)

#define   RC_FILE_ALREADY_OPEN                ((RetCode)0x0100)
#define   RC_FILE_ALREADY_CLOSED              ((RetCode)0x0101)
#define   RC_FILE_CANNOT_BE_OPENED            ((RetCode)0x0102)
#define   RC_FILE_CANNOT_BE_CLOSED            ((RetCode)0x0103)

// defines to use in the toolkit for various purposes

#ifndef MAX_STRING
#define MAX_STRING                        (1024)
#endif
#define MAX_STRINGBUFFER       (MAX_STRING + 1)
#define MAX_LINESIZE                      (120)

#endif // TYPES_H_INCLUDED