
// DiaryInstallerDlg.cpp : implementation file
//

#include "stdafx.h"
#include <string>
#include "direct.h"
#include "DiaryInstaller.h"
#include "DiaryInstallerDlg.h"
#include "unzip.h"
#include "..\Diary Project\common.h"
#include "..\Diary Project\tools.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CDiaryInstallerDlg dialog




CDiaryInstallerDlg::CDiaryInstallerDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CDiaryInstallerDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CDiaryInstallerDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_BROWSE_INSTALLPATH, m_wndBrowseCtrl);
	DDX_Control(pDX, IDC_PROGRESSTEXT, mProgressText);
	DDX_Control(pDX, IDC_DISPLAY_MANUAL, mManualWanted);
}

BEGIN_MESSAGE_MAP(CDiaryInstallerDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDOK, &CDiaryInstallerDlg::OnBnClickedOk)
	ON_BN_CLICKED(IDCANCEL, &CDiaryInstallerDlg::OnBnClickedCancel)
END_MESSAGE_MAP()


// CDiaryInstallerDlg message handlers

BOOL CDiaryInstallerDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// set the browse control to directories only...
	m_wndBrowseCtrl.SetButtonStyle(m_wndBrowseCtrl.GetButtonStyle() | BC_CTL_FOLDERSONLY);
	m_wndBrowseCtrl.SetFolderFlags(BIF_EDITBOX); 
	m_wndBrowseCtrl.SetDialogTitle(_T("Please select a folder"));

	// set the default install directory, the current one...
	char thePath[MAX_STRING];
	_getcwd(thePath, MAX_STRING - 1);
	m_wndBrowseCtrl.SetPathName(thePath);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CDiaryInstallerDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CDiaryInstallerDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}


void CDiaryInstallerDlg::OnBnClickedOk()
{
	// perform the installation script... (step by step)
	mProgressText.SetWindowTextA("downloading diary.zip from www.moorelife.nl...");
	if (DownloadZipFile() == false)
	{
		mProgressText.SetWindowTextA("ZIP file could not be retrieved");
		return;
	}
    
	mProgressText.SetWindowTextA("unzipping diary.zip into chosen directory...");
    if (UnzipDiary() == false)
	{
		mProgressText.SetWindowTextA("ZIP file could not be unzipped");
		return;
	}

	// otherwise OK is just like Cancel....
	OnBnClickedCancel();
}

void CDiaryInstallerDlg::OnBnClickedCancel()
{
	// rematerialize Diary (the new version)..
    _chdir(m_wndBrowseCtrl.GetPathName());
	ShellExecute(0, _T("open"), "diary.exe", 
                 0, 0, SW_SHOWNORMAL);

	
	if (mManualWanted.GetCheck())
	{
		CString theFile = m_wndBrowseCtrl.GetPathName();
		theFile += "\\Diary Documentation\\diary.html";
    	ShellExecute(0, _T("open"), theFile, 
                     0, 0, SW_SHOWNORMAL);
	}

	// and wipe out the installer....
	OnCancel();
}

bool CDiaryInstallerDlg::DownloadZipFile()
{
	// compose the versioned URL for the ZIP file...
	CString theURL = "http://www.moorelife.nl/diary.";
	theURL += GetAvailableVersionNumber().c_str();
	theURL += ".zip";

	// and set the destination file for the ZIP file...
	CString theFile = m_wndBrowseCtrl.GetPathName();
	theFile += "\\diary.zip";
	
	return STools::ReadBinaryWebFile(theURL, 
		                             theFile, 
									 "unzipping diary.zip (%d%cB) into chosen directory...",
									 &mProgressText);
}

bool CDiaryInstallerDlg::UnzipDiary()
{
	// open the downloaded diary, and retrieve the info
    CString theFile = m_wndBrowseCtrl.GetPathName();
	theFile += "\\diary.zip";
	HZIP hz = OpenZip(theFile, 0);
	ZIPENTRY ze; 
	GetZipItem(hz,-1,&ze); // fetch overall details
	int numitems=ze.index;

	if (numitems <= 0)
	{
        mProgressText.SetWindowTextA("ZIP file cannot be unzipped");
		return false;
	}

	// iterate over all files in it...
	for (int zi = 0; zi < numitems; zi++)
	{ 
        // update the progress text...
		int thePercentage = 100 * zi / numitems;
		CString theMsg;
		theMsg.Format("unzipping diary.zip (%d%%) into chosen directory...", 
			          thePercentage);
        mProgressText.SetWindowTextA(theMsg);

		// unzip the file now...
		ZIPENTRY ze; 
		GetZipItem(hz,zi,&ze);   // fetch individual details
		std::string theFullPath = m_wndBrowseCtrl.GetPathName();
		theFullPath += "\\";
		theFullPath += ze.name;
	    UnzipItem(hz, zi, theFullPath.c_str());
	}
	CloseZip(hz);

	return true;
}